*DECK BIVF2H
      SUBROUTINE BIVF2H (DIFF0,SIGMA,NREG,NUN,LL4,ISPLH,NELEM,SIDE,MAT,
     1 KN,QFR,BFR,VOL,IDL,MU,SOURCE,R,RH,QH,RT,QT,FUNKNO,A11)
C
C-----------------------------------------------------------------------
C
C  MESH CORNER FINITE DIFFERENCE OR FINITE ELEMENT FLUX CALCULATION IN
C  A SINGLE ENERGY GROUP. HEXAGONAL GEOMETRY.
C
C  INPUT PARAMETERS:
C   DIFF0  : DIFFUSION COEFFICIENTS. DIMENSION DIFF0(NBMIX)
C   SIGMA  : REMOVAL MACROSCOPIC X-SECTIONS. DIMENSION SIGMA(NBMIX)
C            WHERE NBMIX IS THE NUMBER OF MIXTURES.
C   NREG   : NUMBER OF HEXAGONS IN BIVAC.
C   NUN    : DIMENSION OF VECTOR FUNKNO.
C   LL4    : ORDER OF THE MATRIX A11.
C   ISPLH  : =1 : HEXAGONAL ELEMENTS ; .GT.1 : TRIANGULAR ELEMENTS.
C   NELEM  : NUMBER OF FINITE ELEMENTS (HEXAGONS OR TRIANGLES) EXCLUDING
C            THE VIRTUAL ELEMENTS.
C   SIDE   : SIDE OF THE HEXAGONS.
C   MAT    : MIXTURE INDEX PER HEXAGON. DIMENSION MAT(NREG)
C   KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN((LH+1)C NELEM)
C            WHERE LH=6 (HEXAGONS) OR 3 (TRIANGLES).
C   QFR    : ELEMENT-ORDERED INFORMATION. DIMENSION QFR((LH+1)C NELEM)
C   BFR    : ELEMENT-ORDERED SURFACE FRACTIONS.
C            DIMENSION BFR((LH+1)C NELEM)
C   VOL    : VOLUME OF HEXAGONS. DIMENSION VOL(NREG)
C   IDL    : POSITION OF THE AVERAGE FLUX COMPONENT ASSOCIATED WITH
C            EACH HEXAGON. DIMENSION IDL(NEL)
C   MU     : INDICES USED WITH COMPRESSED DIAGONAL STORAGE MODE MATRIX
C            A11. DIMENSION MU(LL4)
C   SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(LL4)
C   R,RH,QH,RT,QT : UNIT MATRICES. DIMENSION R(2,2),RH(6,6),QH(6,6),
C                   RT(3,3),QT(3,3)
C
C  OUTPUT PARAMETER:
C   FUNKNO : FIRST LL4 VALUES CONTAINS THE FINITE ELEMENT UNKNOWNS;
C            NEXT NREG VALUES CONTAINS ELEMENT AVERAGED FLUXES.
C            SURFACE-AVERAGED FLUX IN POSITION FUNKNO(NUN)
C            DIMENSION FUNKNO(NUN)
C
C  SCRATCH STORAGE:
C   A11    : DIMENSION A11(MU(LL4))
C
C------------------------------------- AUTHOR: A. HEBERT ; 15/07/94 ----
C
      IMPLICIT NONE
C----
C LOCAL VARIABLES
C----
      INTEGER    ISPLH,I,J,K,NUN,NREG,LH,LL4,NUM1,NELEM,KHEX,L,IC,I1,
     &           J1,IND1,IND2,KEY,KEY1
      INTEGER    MAT(*),KN(*),IDL(*),MU(*),ISR(6,2),ISRH(6,2),ISRT(3,2)
C
      REAL       CONST,CONSB,AA,SIDE,BFR1,RR,SS ,QFR1,QQH,VOL0,RRH
      REAL       DIFF0(*),SIGMA(*),QFR(*),BFR(*),VOL(*),
     1           SOURCE(*),R(2,*),RH(6,*),QH(6,*),RT(3,*),QT(3,*),
     2           FUNKNO(*),A11(*),RH2(6,6),QH2(6,6),TH(6)
C
      DATA       ISRH
     &                /2,1,4,5,6,3,1,4,5,6,3,2/
      DATA       ISRT
     &                /1,2,3,2,3,1/
C
C RECOVER THE HEXAGONAL MASS (RH2) AND STIFFNESS (QH2) MATRICES.
C
      IF(ISPLH.EQ.1) THEN
C        HEXAGONAL BASIS.
         LH=6
         DO 10 I=1,6
         DO 10 J=1,2
   10    ISR(I,J)=ISRH(I,J)
         DO 20 I=1,6
         TH(I)=0.0
         DO 20 J=1,6
         RH2(I,J)=RH(I,J)
         QH2(I,J)=QH(I,J)
   20    TH(I)=TH(I)+RH(I,J)
         CONST=1.5*SQRT(3.0)
         CONSB=2.0*SQRT(3.0)/3.0
         AA=SIDE
      ELSE
*        TRIANGULAR BASIS.
         LH=3
         DO 30 I=1,3
         DO 30 J=1,2
   30    ISR(I,J)=ISRT(I,J)
         DO 40 I=1,3
         TH(I)=0.0
         DO 40 J=1,3
         RH2(I,J)=RT(I,J)
         QH2(I,J)=QT(I,J)
   40    TH(I)=TH(I)+RT(I,J)
         CONST=0.25*SQRT(3.0)
         CONSB=2.0*SQRT(3.0)
         AA=SIDE/REAL(ISPLH-1)
      ENDIF
*
* ASSEMBLY OF THE MAIN COEFFICIENT MATRIX.
      DO 45 I=1,MU(LL4)
   45 A11(I)=0.0
      NUM1=0
      DO 100 K=1,NELEM
      KHEX=KN(NUM1+LH+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 100
      L=MAT(KHEX)
      VOL0=QFR(NUM1+LH+1)
      DO 60 I=1,LH
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 60
      KEY1=MU(IND1)-IND1
      DO 50 J=1,LH
      IND2=KN(NUM1+J)
      IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 50
      QQH=QH2(I,J)/(CONST*AA*AA)
      RRH=RH2(I,J)/CONST
      IF((QQH.EQ.0.0).AND.(RRH.EQ.0.0)) GO TO 50
      KEY=KEY1+IND2
      A11(KEY)=A11(KEY)+(QQH*DIFF0(L)+RRH*SIGMA(L))*VOL0
   50 CONTINUE
   60 CONTINUE
      DO 90 IC=1,LH
      QFR1=QFR(NUM1+IC)
      IF(QFR1.EQ.0.0) GO TO 90
      DO 80 I1=1,2
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.EQ.0) GO TO 80
      KEY1=MU(IND1)-IND1
      DO 70 J1=1,2
      IND2=KN(NUM1+ISR(IC,J1))
      IF((IND2.EQ.0).OR.(IND2.GT.IND1)) GO TO 70
      RR=R(I1,J1)
      IF(RR.EQ.0.0) GO TO 70
      KEY=KEY1+IND2
      A11(KEY)=A11(KEY)+RR*QFR1
   70 CONTINUE
   80 CONTINUE
   90 CONTINUE
  100 NUM1=NUM1+LH+1
*
* L-D-L(T) FACTORIZATION AND RESOLUTION.
      CALL ALLDLF(LL4,A11,MU)
      DO 120 I=1,LL4
  120 FUNKNO(I)=SOURCE(I)
      CALL ALLDLS(LL4,MU,A11,FUNKNO)
*
* VOLUME-AVERAGED FLUXES.
      DO 130 KHEX=1,NREG
      IF(IDL(KHEX).NE.0) FUNKNO(IDL(KHEX))=0.0
  130 CONTINUE
      FUNKNO(NUN)=0.0
      NUM1=0
      DO 170 K=1,NELEM
      KHEX=KN(NUM1+LH+1)
      IF(VOL(KHEX).EQ.0.0) GO TO 170
      DO 140 I=1,LH
      IND1=KN(NUM1+I)
      IF(IND1.EQ.0) GO TO 140
      SS=TH(I)*QFR(NUM1+LH+1)/(CONST*VOL(KHEX))
      FUNKNO(IDL(KHEX))=FUNKNO(IDL(KHEX))+SS*FUNKNO(IND1)
  140 CONTINUE
*
* SURFACE-AVERAGED FLUX.
      DO 160 IC=1,LH
      BFR1=BFR(NUM1+IC)*CONSB
      IF(BFR1.EQ.0.0) GO TO 160
      DO 150 I1=1,2
      IND1=KN(NUM1+ISR(IC,I1))
      IF(IND1.GT.0) FUNKNO(NUN)=FUNKNO(NUN)+TH(I1)*FUNKNO(IND1)*BFR1
  150 CONTINUE
  160 CONTINUE
*
  170 NUM1=NUM1+LH+1
      RETURN
      END
