*DECK BIVGSO
      SUBROUTINE BIVGSO (IEX,NGCOND,IGR,NMERGE,FLXMER,SUNMER,CYLIND,
     1 IELEM,NREG,NUN,MAT,XX,DD,KN,QFR,VOL,MERG,COUR,FUNKNO,SOURCE,
     2 ZGAR1,ZGAR2)
C
C-----------------------------------------------------------------------
C
C  SOURCE TERM CALCULATION FOR A MIXED-DUAL FORMULATION OF THE FINITE
C  ELEMENT TECHNIQUE IN A 2D CARTESIAN GEOMETRY.
C
C  INPUT PARAMETERS:
C    IEX    : ITERATION NUMBER.
C    NGCOND : NUMBER OF GROUPS CONDENSED.
C    IGR    : ENERGY GROUP INDEX.
C    NMERGE : NUMBER OF MERGED REGIONS.
C    FLXMER : FLUX ESTIMATE PER MIXTURE. DIMENSION FLXMER(NMERGE,NGCOND)
C    SUNMER : INCOMING SOURCE (SCATTERING+FISSION) X-SECTIONS.
C             DIMENSION SUNMER(NMERGE,NGCOND)
C    CYLIND : =.TRUE. FOR CYLINDRICAL GEOMETRY.
C    IELEM  : DEGREE OF THE LAGRANGIAN FINITE ELEMENTS. =1 (LINEAR);
C             =2 (PARABOLIC); =3 (CUBIC); =4 (QUARTIC).
C    NREG   : NUMBER OF ELEMENTS IN BIVAC.
C    NUN    : NUMBER OF UNKNOWNS PER GROUP IN BIVAC.
C    MAT    : MIXTURE INDEX PER REGION. DIMENSION MAT(NREG)
C    XX     : X-DIRECTED MESH SPACINGS. DIMENSION XX(NREG)
C    DD     : USED WITH CYLINDRICAL GEOMETRY. DIMENSION DD(NREG)
C    KN     : ELEMENT-ORDERED UNKNOWN LIST. DIMENSION KN(4C NREG)
C    QFR    : ELEMENT-ORDERED BOUNDARY CONDITIONS.  QFR(4C NREG)
C    VOL    : VOLUME OF REGIONS. DIMENSION VOL(NREG)
C    MERG   : INDEX OF MERGED REGIONS. DIMENSION MERG(NREG)
C    COUR   : FOUR TIMES THE INCOMING CURRENT PER UNIT SURFACE.
C    FUNKNO : PREVIOUSLY CALCULATED FLUXES. DIMENSION FUNKNO(NUN,NGCOND)
C
C  OUTPUT PARAMETER:
C    SOURCE : FISSION AND DIFFUSION SOURCES. DIMENSION SOURCE(NREG)
C
C  SCRATCH STORAGE:
C    ZGAR1  : DIMENSION ZGAR1(NBMIX) WHERE NBMIX=MAX(MAT(I))
C    ZGAR2  : DIMENSION ZGAR2(NBMIX)
C
C------------------------------------- AUTHOR: A. HEBERT ; 11/08/92 ----
C
      IMPLICIT NONE
C----
C INTERFACE VARIABLES
C----
      INTEGER   IEX,NGCOND,IGR,NMERGE,IELEM,NREG,NUN
      INTEGER   MAT(*),KN(*),MERG(*)
      REAL      COUR
      REAL      FLXMER(NMERGE,*),SUNMER(NMERGE,*),XX(*),
     1          DD(*),QFR(*),VOL(*),FUNKNO(NUN,*),SOURCE(*),
     2          ZGAR1(*),ZGAR2(*)
      LOGICAL   CYLIND
C----
C LOCAL VARIABLES
C----
      INTEGER   I,K,NBMIX,NUM1,NUM2,IND1,IGR2,NL,L,I0
      REAL      DIN,DOT,SG,PV
C
C----
C
      NBMIX=0
      DO 10 I=1,NREG
   10 NBMIX=MAX(NBMIX,MAT(I))
      DO 20 I=1,NUN
   20 SOURCE(I)=0.0
*
* INCOMING CURRENT SOURCE.
      IF(COUR.NE.0.0) THEN
         NUM1=0
         NUM2=0
         DO 50 K=1,NREG
         IF(MAT(K).EQ.0) GO TO 50
         IF(VOL(K).EQ.0.0) GO TO 40
         IF(CYLIND) THEN
            DIN=1.0-0.5*XX(K)/DD(K)
            DOT=1.0+0.5*XX(K)/DD(K)
         ELSE
            DIN=1.0
            DOT=1.0
         ENDIF
*
         IF(KN(NUM1+2).NE.0) THEN
            IND1=ABS(KN(NUM1+2))
            SG=REAL(SIGN(1,KN(NUM1+2)))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*DIN*QFR(NUM2+1)
         ENDIF
         IF(KN(NUM1+3).NE.0) THEN
            IND1=ABS(KN(NUM1+3))
            SG=REAL(SIGN(1,KN(NUM1+3)))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*DOT*QFR(NUM2+2)
         ENDIF
         IF(KN(NUM1+4).NE.0) THEN
            IND1=ABS(KN(NUM1+4))
            SG=REAL(SIGN(1,KN(NUM1+4)))
            SOURCE(IND1)=SOURCE(IND1)-SG*0.5*COUR*QFR(NUM2+3)
         ENDIF
         IF(KN(NUM1+5).NE.0) THEN
            IND1=ABS(KN(NUM1+5))
            SG=REAL(SIGN(1,KN(NUM1+5)))
            SOURCE(IND1)=SOURCE(IND1)+SG*0.5*COUR*QFR(NUM2+4)
         ENDIF
   40    NUM1=NUM1+5
         NUM2=NUM2+4
   50    CONTINUE
      ENDIF
*
* DIFFUSION AND FISSION SOURCE.
      IGR2=NGCOND
      IF(IEX.EQ.1) IGR2=IGR-1
      DO 110 NL=1,IGR2
      DO 60 L=1,NBMIX
      ZGAR1(L)=0.0
   60 ZGAR2(L)=0.0
      NUM1=0
      DO 80 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 80
      IF(VOL(K).EQ.0.0) GO TO 70
      ZGAR1(L)=ZGAR1(L)+VOL(K)
      ZGAR2(L)=ZGAR2(L)+FUNKNO(KN(NUM1+1),NL)*VOL(K)
   70 NUM1=NUM1+5
   80 CONTINUE
      NUM1=0
      DO 110 K=1,NREG
      L=MAT(K)
      IF(L.EQ.0) GO TO 110
      IF(VOL(K).EQ.0.0) GO TO 100
      DO 90 I0=1,IELEM*IELEM
      IND1=KN(NUM1+1)+I0-1
      PV=SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)*ZGAR1(L)/ZGAR2(L)
   90 SOURCE(IND1)=SOURCE(IND1)+FUNKNO(IND1,NL)*VOL(K)*PV
  100 NUM1=NUM1+5
  110 CONTINUE
      IF(IEX.GT.1) RETURN
      NUM1=0
      DO 140 K=1,NREG
      IF(MAT(K).EQ.0) GO TO 140
      IF(VOL(K).EQ.0.0) GO TO 130
      IND1=KN(NUM1+1)
      PV=0.0
      DO 120 NL=IGR,NGCOND
  120 PV=PV+SUNMER(MERG(K),NL)*FLXMER(MERG(K),NL)
      SOURCE(IND1)=SOURCE(IND1)+VOL(K)*PV
  130 NUM1=NUM1+5
  140 CONTINUE
      RETURN
      END
