*DECK DOORPV
      SUBROUTINE DOORPV (CDOOR,IPSYS,NPSYS,IPTRK,IFTRAK,IMPX,NGRP,NREG,
     1 NBMIX,NANI,MAT,VOL,KNORM,IPIJK,LEAKSW,LNORM,TITR,PGAR,SGAR,SGAS)
*
*-----------------------------------------------------------------------
*
*Purpose:
* calculation of the collision probabilities. Simplified version for use
* in Dragon Version3.
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version
*
*Author(s): A. Hebert
*
*Parameters: input
* CDOOR   name of the geometry/solution operator.
* IPSYS   pointer to the PIJ LCM object (L_PIJ signature).
* NPSYS   index array set to zero if a group is not to be processed.
*         Usually, NPSYS(I)=I.
* IPTRK   pointer to the tracking (L_TRACK signature).
* IFTRAK  unit of the sequential binary tracking file.
* IMPX    print flag (equal to zero for no print).
* NGRP    number of energy groups.
* NREG    total number of merged blocks for which specific values
*         of the neutron flux and reactions rates are required.
* NBMIX   number of mixtures (NBMIX=max(MAT(i))).
* NANI    number of Legendre orders (usually equal to one).
* MAT     index-number of the mixture type assigned to each volume.
* VOL     volumes.
* KNORM   normalization scheme for PIJ matrices.
* IPIJK   pij option (=1 pij, =4 pijk).
* LEAKSW  leakage flag (=.true. if neutron leakage through external
*         boundary is present).
* LNORM   logical switch for removing leakage from collision
*         probabilities and keeping the PIS information.
* TITR    title.
*
*Parameters: scratch
* PGAR
* SGAR
* SGAS
*
*-----------------------------------------------------------------------
*
      INTEGER     ILCMUP,ILCMDN
      PARAMETER  (ILCMUP=1,ILCMDN=2)
*----
*  SUBROUTINE ARGUMENTS
*----
      CHARACTER CDOOR*12,TITR*72
      LOGICAL LEAKSW,LNORM
      INTEGER IPSYS,NPSYS(NGRP),IPTRK,IFTRAK,IMPX,NGRP,NREG,NBMIX,NANI,
     > MAT(NREG),KNORM,IPIJK
      REAL VOL(NREG),PGAR(NREG*(NREG+1)/2,IPIJK),SGAR(0:NBMIX),
     > SGAS(0:NBMIX,NANI)
*----
*  LOCAL VARIABLES
*----
      PARAMETER(IOUT=6)
      INTEGER   ITYPE
      CHARACTER CMS(3)*1,CGRPNM*12
      LOGICAL LSKIP
      COMMON BASE(1)
      INTEGER     NALBP
      REAL        ALBP
      SAVE CMS
      DATA CMS/'1','2','3'/
      INDPOS(I,J)=MAX(I,J)*(MAX(I,J)-1)/2+MIN(I,J)
*
      NELPIJ=NREG*(NREG+1)/2
      NB1=NBMIX+1
      ITYPE=1
*----
*  COMPUTE THE REDUCED PIJ MATRIX -- NON-VECTORIAL ALGORITHM.
*----
      CALL SETARA(BASE,4*NREG*NREG,IPCSCT)
      DO 100 IGR=1,NGRP
      IOFSET=NPSYS(IGR)
      IF(IOFSET.NE.0) THEN
         IF(IMPX.GT.10) WRITE(IOUT,'(/25H DOORPV: PROCESSING GROUP,I5,
     >   6H WITH ,A,1H.)') IGR,CDOOR
         WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
         CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
         CALL LCMGET(IPSYS,'DRAGON-TXSC',SGAR)
         CALL LCMGET(IPSYS,'DRAGON-S0XSC',SGAS)
         IF(CDOOR .EQ. 'EXCELL') THEN
            NSBG=1
            NPST=1
            JGL=0
            IFUITE=-1
            PROBKS=1.0
            KNORM=1
            NALBP=0
            CALL EXCELP(IPSYS ,IPTRK ,IFTRAK,IMPX  ,NREG  ,NBMIX,
     >                  MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                  TITR  ,SGAR  ,PGAR  ,NSBG  ,NPST  ,PROBKS,
     >                  JGL   ,NSBG  ,IFUITE,
     >                  NALBP ,ALBP  )
         ELSE IF(CDOOR(1:3).EQ.'JPM') THEN
            CALL JPMP  (IPSYS ,IPTRK ,IFTRAK,IMPX  ,NREG  ,NBMIX,
     >                  MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                  TITR  ,SGAR  ,PGAR)
         ELSE IF(CDOOR.EQ.'SYBIL') THEN
            CALL SYBILP(IPSYS ,IPTRK ,IFTRAK,IMPX  ,NREG  ,NBMIX,
     >                  MAT   ,VOL   ,KNORM ,NELPIJ,IPIJK ,LEAKSW,
     >                  TITR  ,SGAR  ,PGAR)
         ELSE
            CALL XABORT('DOORPV: UNKNOWN PIJ DOOR NAMED '//CDOOR//'.')
         ENDIF
*----
*  SCATTERING REDUCTION. NOTE: SCATTERING REDUCTION IS PERFORMED OUTSIDE
*  EXCELP/JPMP/SYBILP IN VERSION3.
*----
         LSKIP=.TRUE.
         DO 10 IBM=1,NBMIX
   10    LSKIP=LSKIP.AND.(SGAS(IBM,1).EQ.0.0)
         IOP=1
         IF(.NOT.LSKIP) THEN
            CALL PIJSMD(IMPX,NBMIX,NREG,MAT,VOL,SGAS(0,1),SGAR(0),
     >                  LEAKSW,PGAR(1,1),BASE(IPCSCT),IOP)
            CALL XDRSDB(NREG*NREG,BASE(IPCSCT),BASE(IPCSCT),1)
            DO 20 I=1,NREG
            DO 20 J=1,NREG
            INDPIJ=INDPOS(I,J)
   20       PGAR(INDPIJ,1)=BASE(IPCSCT+(J-1)*NREG+I-1)*VOL(I)
            IF(IPIJK.EQ.4) THEN
*              P1 SCATTERING REDUCTION OF THE DIRECTIONNAL CP MATRICES.
               IOP=4
               IF(NANI.LT.2) CALL XABORT('DOORPV: ANISOTROPIC SCAT MIS'
     >         //'SING.')
               DO 30 IJKS=1,3
               CALL PIJSMD(IMPX,NBMIX,NREG,MAT,VOL,SGAS(0,2),SGAR(0),
     >                     LEAKSW,PGAR(1,IJKS+1),BASE(IPCSCT),IOP)
               CALL XDRSDB(NREG*NREG,BASE(IPCSCT),BASE(IPCSCT),1)
               DO 30 I=1,NREG
               DO 30 J=1,NREG
               INDPIJ=INDPOS(I,J)
   30          PGAR(INDPIJ,IJKS+1)=BASE(IPCSCT+(J-1)*NREG+I-1)*VOL(I)
            ENDIF
         ENDIF
*----
*  REMOVE LEAKAGE FROM THE SCATTERING-REDUCED CP MATRIX.
*----
         IF(LNORM) THEN
            CALL SETARA(BASE,NREG,IPIS)
            CALL PIJNRM(NREG,NBMIX,ITYPE,MAT,VOL,SGAR(0),SGAS(0,1),PGAR,
     >      BASE(IPIS))
            IF(LEAKSW) THEN
               CALL LCMPUT(IPSYS,'DRAGON-WIS',NREG,2,BASE(IPIS))
            ENDIF
            CALL RLSARA(BASE(IPIS))
         ENDIF
*----
*  FORMAT THE REDUCED PIJ MATRIX AS A SQUARE MATRIX.
*----
         CALL SETARA(BASE,NREG*NREG,IPREG)
         DO 40 I=1,NREG
         DO 40 J=1,NREG
   40    BASE(IPREG+(J-1)*NREG+I-1)=PGAR(INDPOS(I,J),1)/VOL(I)
         CALL LCMPUT(IPSYS,'DRAGON-PCSCT',NREG*NREG,2,BASE(IPREG))
         IF(IPIJK.EQ.4) THEN
            DO 60 IJKS=1,3
            DO 50 I=1,NREG
            DO 50 J=1,NREG
   50       BASE(IPREG+(J-1)*NREG+I-1)=PGAR(INDPOS(I,J),IJKS+1)/VOL(I)
            CALL LCMPUT(IPSYS,'DRAGON'//CMS(IJKS)//'PCSCT',NREG*NREG,2,
     >      BASE(IPREG))
   60       CONTINUE
         ENDIF
         CALL RLSARA(BASE(IPREG))
         CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
      ENDIF
  100 CONTINUE
      CALL RLSARA(BASE(IPCSCT))
      RETURN
      END
