*DECK EDI
      SUBROUTINE EDI(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*-----------------------------------------------------------------------
*
* EDITION MODULE FOR DRAGON-2.
*
* INPUT/OUTPUT PARAMETERS:
*  NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*  HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*  IENTRY : =1 LINKED LIST; =2 XSM FILE; =3 SEQUENTIAL BINARY FILE;
*           =4 SEQUENTIAL ASCII FILE.
*  JENTRY : =0 THE LINKED LIST OR FILE IS CREATED.
*           =1 THE LINKED LIST OR FILE IS OPEN FOR MODIFICATIONS;
*           =2 THE LINKED LIST OR FILE IS OPEN IN READ-ONLY MODE.
*  KENTRY : =FILE UNIT NUMBER; =LINKED LIST ADDRESS OTHERWISE.
*           DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),JENTRY(NENTRY),
*           KENTRY(NENTRY)
*
* DATA STRUCTURES: (ORDER IS ARBITRARY FOR 2,3,4)
*  HENTRY(1) : CREATE OR MODIFICATION TYPE(L_EDIT).
*  HENTRY(2) : READ-ONLY TYPE(L_FLUX)
*  HENTRY(3) : READ-ONLY TYPE(L_MACROLIB OR L_LIBRARY)
*  HENTRY(4) : READ-ONLY TYPE(L_TRACK)
* FOR SPH CALCULATIONS ONLY: (ORDER IS ARBITRARY FOR 5,6,7)
*  HENTRY(5) : OPTIONAL READ-ONLY TYPE(L_GEOM)         ORIGINAL GEOMETRY
*  HENTRY(6) : OPTIONAL READ-ONLY TYPE(L_GEOM)         SPH MACROGEOMETRY
*  HENTRY(7) : OPTIONAL READ-ONLY TYPE(L_SYS)  PIJ FOR ORIGINAL GEOMETRY
*
*-------------------------------------- AUTHOR: A. HEBERT ; 23/11/94 ---
*------------------------------------ MODIFIED: G.MARLEAU ; 97-02-05 ---
*
      IMPLICIT    NONE
      INTEGER     IOUT,NSTATE,NTC,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,NSTATE=40,NTC=18,ILCMUP=1,ILCMDN=2,
     >            NAMSBR='EDI   ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     NENTRY
      CHARACTER   HENTRY(NENTRY)*12
      INTEGER     IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IENERG,IENERV,IMAT,IVOL,IIDL,ISONAM,ISOCAR,
     >            IGCOND,IMERGE,IACTI,IMIXME,IGCR,IECR,JFLUXE
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   TEXT12*12,HSIGN*12,CDOOR*12,CNDOOR*12,
     >            MACGEO*12,CURNAM*12,OLDNAM*12,TITLE*72
      CHARACTER   GEONAM*12
      LOGICAL     LEAKSW
      INTEGER     ISTATE(NSTATE),KCHAR(18),ISOTXS(2),IPEDIT,
     >            IPFLUX,IKFLUX,IPTRK,IKTRK,IPLIB,IKLIB,IEN,
     >            IPGEO1,IKGEO1,IPGEO2,IKGEO2,IPSYS,IKSYS,
     >            ILCMLN,ILCMTY,ITC,NREG,MAXISM,ILEAKS,NWGTH,
     >            NGROUP,NBMIX,NL,NIFISS,NEDMAC,ITRANC,
     >            NMERGE,NGCOND,IFFAC,ILUPS,NACTI,NSTATS,NSPH,
     >            KSPH,NBMICR,IPRINT,NSAVES,MAXISK,MAXCND,MAXMRG,
     >            IXEDI,MAXPTS,MAXISO,IGROUP,IREGIO,ICALL,INTLIR,
     >            NGCR,NBISO,NEMICR,NTENER
      INTEGER     IKTRK2,IPTRK2,IFTRK2,IPMRG,ITM
*----
*  Surface fluxes
*----
      INTEGER     MATALB,KEYMRG,VOLSUR,SVDB,FUNSUR
      INTEGER     IFSUR,NVOL,NSUR,ITTRK,NVS,ISV
      REAL        FLXNOR
C----
C PARAMETER VALIDATION.
C----
      GEONAM='        '
      IF(NENTRY .LT. 4) CALL XABORT(NAMSBR//
     >': AT LEAST FOUR DATA STRUCTURES EXPECTED.')
      IF(IENTRY(1) .NE. 1 .AND. IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IPEDIT=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_EDIT'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_EDIT') THEN
          TEXT12=HENTRY(1)
          CALL XABORT(NAMSBR//
     >    ': INVALID SIGNATURE OF EDITING STRUCTURE '//TEXT12//
     >    ' SIGNATURE FOUND  '//HSIGN//' L_EDIT EXPECTED')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >  ': EDITING STRUCTURE IN CREATE OR MODIFY MODE EXPECTED')
      ENDIF
C----
C  SCAN READ-ONLY MODE DATA STRUCTURE ENTRY(2) TO ENTRY(4)
C  FOR FLUX, TRACK AND LIB
C----
      IF(JENTRY(2) .NE. 2) CALL XABORT(NAMSBR//
     >': DATA STRUCTURE IN READ-ONLY MODE EXPECTED AT RHS.')
      IPFLUX=0
      IKFLUX=0
      IPTRK=0
      IKTRK=0
      IPLIB=0
      IKLIB=0
      DO 100 IEN=2,4
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(HSIGN .EQ. 'L_FLUX' .AND. IKFLUX .EQ. 0) THEN
            IPFLUX=KENTRY(IEN)
            IKFLUX=IEN
          ELSE IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK .EQ. 0) THEN
            IPTRK=KENTRY(IEN)
            IKTRK=IEN
          ELSE IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ELSE IF(HSIGN .EQ. 'L_MACROLIB' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=-IEN
          ENDIF
        ENDIF
 100  CONTINUE
      IF(IKFLUX .EQ. 0) CALL XABORT(NAMSBR//
     >': NO FLUXUNK STRUCTURE FOUND.')
      IF(IKTRK.EQ.0) CALL XABORT(NAMSBR//
     >': NO TRACKING STRUCTURE FOUND.')
      IF(IKLIB.EQ.0) CALL XABORT(NAMSBR//
     >': NO MACROLIB OR MICROLIB DATA STRUCTURE FOUND.')
C----
C  READ MACROLIB INFORMATION
C----
      CALL XDISET(ISTATE,NSTATE,0)
      IF(IKLIB .GT. 0) THEN
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        MAXISM=ISTATE(22)
        CALL LCMSIX(IPLIB,'MACROLIB',ILCMUP)
      ELSE
        MAXISM=1
      ENDIF
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      NGROUP=ISTATE(1)
      NBMIX=ISTATE(2)
      NL=ISTATE(3)
      NIFISS=ISTATE(4)
      NEDMAC=ISTATE(5)
      ITRANC=ISTATE(6)
      CALL SETARA(RBASE,2*NGROUP+1,IENERG)
      CALL SETARA(RBASE,NGROUP,IENERV)
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB,'MACROLIB',ILCMDN)
C----
C  READ GROUP STRUCTURE
C----
      CALL LCMLEN(IPLIB,'ENERGY',ILCMLN,ILCMTY)
      NTENER=ILCMLN
      IF(NTENER .EQ. NGROUP+1) THEN
        CALL LCMGET(IPLIB,'ENERGY',RBASE(IENERG))
      ELSE IF(NTENER .NE. 0) THEN
        CALL XABORT(NAMSBR//': INVALID NUMBER OF GROUP ON MACROLIB')
      ENDIF
C----
C  READ GEOMETRIES AND SYSTEM (SPH ONLY)
C----
      IPGEO1=0
      IKGEO1=0
      IPGEO2=0
      IKGEO2=0
      IPSYS=0
      IKSYS=0
      IKTRK2=0
      IPTRK2=0
      IFTRK2=0
      IF(NENTRY .GT. 4) THEN
        DO 101 IEN=5,NENTRY
          IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
            CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
            WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
            IF(HSIGN .EQ. 'L_GEOM') THEN
              IF(IKGEO1 .EQ. 0) THEN
                IPGEO1=KENTRY(IEN)
                IKGEO1=IEN
              ELSE IF(IKGEO2 .EQ. 0) THEN
                IPGEO2=KENTRY(IEN)
                IKGEO2=IEN
              ENDIF
            ELSE IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK2 .EQ. 0) THEN
              IPTRK2=KENTRY(IEN)
              IKTRK2=IEN
            ELSE IF(HSIGN .EQ. 'L_PIJ' .AND. IKSYS .EQ. 0) THEN
              IPSYS=KENTRY(IEN)
              IKSYS=IEN
            ENDIF
          ELSE IF(IENTRY(IEN) .EQ. 3) THEN
            IF(IKTRK2 .EQ. 0) CALL XABORT(NAMSBR//
     >      ': Second tracking structure must be provided '//
     >      ' before tracking file')
            IFTRK2=KENTRY(IEN)
          ENDIF
 101    CONTINUE
      ENDIF
C----
C  RECOVER GENERAL TRACKING INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'TRACK-TYPE',KCHAR)
      WRITE(CDOOR,'(3A4)') (KCHAR(ITC),ITC=1,3)
      IF(CDOOR .NE. 'JPM'    .AND.
     >   CDOOR .NE. 'SYBIL'  .AND.
     >   CDOOR .NE. 'EXCELL'   ) CALL XABORT(NAMSBR//
     >': INVALID TRACKING TYPE FOR EDITING : '//CDOOR)
      NREG=ISTATE(1)
      NVOL=NREG
      LEAKSW=ISTATE(3) .EQ. 0
      NSUR=ISTATE(5)
      ITTRK=ISTATE(7)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IIDL)
      CALL LCMGET(IPTRK,'MATCOD',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX',IBASE(IIDL))
      CALL LCMLEN(IPTRK,'TITLE',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN
        CALL LCMGET(IPTRK,'TITLE',KCHAR)
        WRITE(TITLE,'(18A4)') (KCHAR(ITC),ITC=1,NTC)
      ELSE
        TITLE='*** NO TITLE PROVIDED FOR THE REFERENCE CASE ***'
        READ(TITLE,'(18A4)') (KCHAR(ITC),ITC=1,NTC)
      ENDIF
      CALL LCMPUT(IPEDIT,'TITLE',NTC,3,KCHAR)
C----
C  RECOVER FLUX INFORMATION
C----
      ILEAKS=0
      NWGTH=0
      IF(LEAKSW) ILEAKS=1
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPFLUX,'STATE-VECTOR',ISTATE)
      IF(ISTATE(1) .GT. 1 .AND.
     >   ISTATE(1) .LT. 6 ) THEN
        ILEAKS=2
        NWGTH=1
        IF(ISTATE(2) .GE. 5) THEN
          ILEAKS=3
          NWGTH=3
        ENDIF
      ENDIF
      IFSUR=ISTATE(12)
C----
C  READ EDITING INFORMATION
C----
      ISOTXS(1)=0
      ISOTXS(2)=0
      IF(JENTRY(1) .EQ. 0) THEN
        NMERGE=NREG
        NGCOND=NGROUP
        IFFAC=0
        ILUPS=0
        NACTI=0
        NSTATS=0
        NSPH=0
        KSPH=0
        NBMICR=0
        IPRINT=1
        NSAVES=0
        MAXISK=0
        MAXCND=0
        MAXMRG=0
        IXEDI=0
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPEDIT,'STATE-VECTOR',ISTATE)
        NMERGE=ISTATE(1)
        NGCOND=ISTATE(2)
        IFFAC=ISTATE(3)
        ILUPS=ISTATE(4)
        NACTI=ISTATE(5)
        NSTATS=ISTATE(6)
        NSPH=ISTATE(7)
        KSPH=ISTATE(8)
        NBMICR=ISTATE(9)
        IPRINT=ISTATE(10)
        NSAVES=ISTATE(11)
        MAXISK=ISTATE(13)
        MAXCND=ISTATE(14)
        MAXMRG=ISTATE(15)
        IXEDI=ISTATE(16)
      ENDIF
      MAXISM=MAX(MAXISK,MAXISM)
      MAXPTS=NREG
      MAXISO=MAXISM*MAX(NREG,NBMIX)
      CALL SETARA(IBASE,3*MAXISO,ISONAM)
      CALL SETARA(IBASE,2*MAXISO,ISOCAR)
      CALL SETARA(IBASE,NGROUP,IGCOND)
      CALL SETARA(IBASE,NREG,IMERGE)
      DO 110 IGROUP=1,NGROUP
        IBASE(IGCOND+IGROUP-1)=IGROUP
 110  CONTINUE
      DO 111 IREGIO=1,NREG
        IBASE(IMERGE+IREGIO-1)=IREGIO
 111  CONTINUE
      CALL SETARA(IBASE,NBMIX,IACTI)
      CALL SETARA(IBASE,NBMIX+1,IMIXME)
      ICALL=0
      CURNAM=' '
      IF(JENTRY(1) .EQ. 0) THEN
        OLDNAM=' '
        CNDOOR=' '
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMLEN(IPEDIT,'LAST-EDIT',ILCMLN,ILCMTY)
        IF(ILCMLN .GT. 0) THEN
          CALL LCMGET(IPEDIT,'LAST-EDIT',KCHAR)
          WRITE(OLDNAM,'(3A4)') (KCHAR(ITC),ITC=1,3)
          INTLIR=0
          READ(OLDNAM,'(8X,I4)',ERR=115) INTLIR
 115      ICALL=MAX(ICALL,INTLIR)
        ENDIF
        CALL LCMLEN(IPEDIT,'REF:IMERGE',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ. NREG) THEN
          CALL LCMGET(IPEDIT,'REF:IMERGE',IBASE(IMERGE))
        ELSE
          WRITE(IOUT,9000) NAMSBR,ILCMLN,NREG
        ENDIF
        CALL LCMLEN(IPEDIT,'REF:IGCOND',ILCMLN,ILCMTY)
        IF(ILCMLN .LE. NGROUP) THEN
          CALL LCMGET(IPEDIT,'REF:IGCOND',IBASE(IGCOND))
          IF(IBASE(IGCOND+ILCMLN-1) .NE. NGROUP) THEN
            WRITE(IOUT,9001) NAMSBR,IBASE(IGCOND+ILCMLN-1),NGROUP
            DO 112 IGROUP=1,NGROUP
              IBASE(IGCOND+IGROUP-1)=IGROUP
 112        CONTINUE
          ENDIF
        ELSE
          WRITE(IOUT,9001) NAMSBR,ILCMLN,NGROUP
        ENDIF
        IF(NBMICR .GT. 1 .OR. NBMICR .LT. -1) THEN
          NEMICR=ABS(NBMICR)-1
          IF(NEMICR .GT. MAXISO) CALL XABORT(NAMSBR//
     >    ': EXCESSIVE NUMBER OF ISOTOPE TO EDIT')
           CALL LCMGET(IPEDIT,'CARISO',IBASE(ISOCAR))
        ENDIF
        IF(NACTI .GT. 0) CALL LCMGET(IPEDIT,'IACTI',IBASE(IACTI))
        IF(NSPH .GE. 2) THEN
          CALL LCMGET(IPEDIT,'SPH-TRK',KCHAR)
          WRITE(CNDOOR,'(3A4)') (KCHAR(ITC),ITC=1,3)
        ELSE
          CNDOOR=' '
        ENDIF
      ENDIF
      MACGEO=' '
      NGCR=0
      CALL SETARA(IBASE,NGROUP+1,IGCR)
      CALL SETARA(RBASE,NGROUP+1,IECR)
      CALL XDISET(IBASE(IGCR),NGROUP+1,NGROUP)
      CALL XDRSET(RBASE(IECR),NGROUP+1,0.0)
      CALL EDIGET(NGROUP,NGCR  ,NREG  ,NBMIX ,NMERGE,IFFAC ,
     >            ILUPS ,NSAVES,NSTATS,CURNAM,OLDNAM,NSPH  ,
     >            KSPH  ,CNDOOR,MACGEO,NWGTH ,MAXISO,NBMICR,
     >            NACTI ,IPRINT,MAXPTS,ICALL ,ISOTXS,
     >            IBASE(IGCR)  ,RBASE(IECR)  ,IBASE(IMERGE),
     >            IBASE(IMAT)  ,IBASE(ISOCAR),IBASE(IACTI),
     >            IBASE(IMIXME))
      CALL RLSARA(IBASE(IMIXME))
      IF(NMERGE .EQ. -1) THEN
C----
C CALL EDIMRG TO FIND MERGE INDEX ASSOCIATED WITH
C SECOND GEOMETRY OR TRACK FILE
C----
        IF(IKGEO2 .GT. 0) THEN
          ITM=-1
          IPMRG=IPGEO2
          GEONAM=HENTRY(IKGEO2)
        ELSE IF(IKGEO1 .GT. 0) THEN
          ITM=-1
          IPMRG=IPGEO1
          GEONAM=HENTRY(IKGEO1)
        ELSE IF(IKTRK2 .GT. 0) THEN
          ITM=1
          IPMRG=IPTRK2
        ELSE
          ITM=0
          IPMRG=IPTRK
        ENDIF
        CALL EDIMRG(IPTRK ,IPMRG ,IPRINT,GEONAM,ITM   ,NREG  ,
     >              NMERGE,IBASE(IMERGE))
      ENDIF
C----
C  TEST ENERGY CONDENSATION INPUT
C----
      CALL EDIENE(NGROUP,NGCR  ,NGCOND,NTENER,
     >            IBASE(IGCR)  ,RBASE(IECR)  ,IBASE(IGCOND),
     >            RBASE(IENERG),RBASE(IENERV))
C----
C  TEST MICROSCOPIC EDIT NAMES
C----
      IF(IKLIB .LT. 0) THEN
        IF(NBMICR .NE. 0) THEN
          WRITE(IOUT,9002) NAMSBR,NBMICR,0
          NBMICR=0
        ENDIF
        IF(NACTI .NE. 0) THEN
          WRITE(IOUT,9003) NAMSBR,NACTI,0
          NACTI=0
        ENDIF
      ELSE
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
        NBISO=ISTATE(2)
        IF(NBISO .EQ. 0) THEN
          WRITE(IOUT,9004) NAMSBR
          NBMICR=0
          NACTI=0
        ELSE
          CALL LCMGET(IPLIB,'ISOTOPESUSED',IBASE(ISONAM))
          IF(ABS(NBMICR) .GT. 1) THEN
            CALL EDITIS(MAXISO,NBISO ,NBMICR,
     >                  IBASE(ISONAM),IBASE(ISOCAR))
          ENDIF
        ENDIF
      ENDIF
      CALL RLSARA(RBASE(IECR))
      CALL RLSARA(IBASE(IGCR))
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMERGE
      ISTATE(2)=NGCOND
      ISTATE(3)=IFFAC
      ISTATE(4)=ILUPS
      ISTATE(5)=NACTI
      ISTATE(6)=NSTATS
      ISTATE(7)=NSPH
      ISTATE(8)=KSPH
      ISTATE(9)=NBMICR
      ISTATE(10)=IPRINT
      ISTATE(11)=NSAVES
C----
C  IF P1 HOMOGENIZATION NOT COMPATIBLE WITH LEAKAGE METHOD
C  SELECT BEST TECHNIQUE ASSOCIATED WITH THIS LEAKAGE METHOD
C----
      IF(ILEAKS .EQ. 3) THEN
        NWGTH=MIN(NWGTH,3)
      ELSE IF(ILEAKS .EQ. 2) THEN
        NWGTH=MIN(NWGTH,1)
      ELSE
        NWGTH=0
      ENDIF
      ISTATE(12)=NWGTH
      ISTATE(13)=MAXISM
      ISTATE(14)=MAX(NGCOND,MAXCND)
      ISTATE(15)=MAX(NMERGE,MAXMRG)
      ISTATE(16)=IXEDI+(ISOTXS(1)+ISOTXS(2))*NMERGE
      IF(IPRINT .GT. 0) THEN
        WRITE(IOUT,6000) 13,NAMSBR,(ISTATE(ITC),ITC=1,8)
        WRITE(IOUT,6001) (ISTATE(ITC),ITC=9,13)
        IF(NSPH .GE. 2) WRITE(IOUT,6010) CNDOOR
        WRITE(IOUT,6020)
     >  (ITC,' -> ',IBASE(IMERGE+ITC-1),ITC=1,NREG)
      ENDIF
      CALL LCMPUT(IPEDIT,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPEDIT,'REF:IMERGE',NREG,1,IBASE(IMERGE))
      CALL LCMPUT(IPEDIT,'REF:IGCOND',NGCOND,1,IBASE(IGCOND))
      IF(NSAVES .GE. 2) THEN
        READ(CURNAM,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'LAST-EDIT',3,3,KCHAR)
      ENDIF
      IF(NBMICR .GT. 1 .OR. NBMICR .LT. -1) THEN
        NEMICR=ABS(NBMICR)-1
        CALL LCMPUT(IPEDIT,'CARISO',2*NEMICR,3,IBASE(ISOCAR))
      ENDIF
      IF(NACTI.GT.0) CALL LCMPUT(IPEDIT,'IACTI',NACTI,1,IBASE(IACTI))
      IF(NSPH .EQ. 5) THEN
        IF(IKTRK2 .EQ. 0) CALL XABORT(NAMSBR//
     >  ': no traking structure for SPH calculations')
        CALL LCMGET(IPTRK2,'TRACK-TYPE',KCHAR)
        WRITE(CNDOOR,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(CNDOOR .NE. 'JPM'    .AND.
     >     CNDOOR .NE. 'SYBIL'  .AND.
     >     CNDOOR .NE. 'NXT'    .AND.
     >     CNDOOR .NE. 'EXCELL'   ) CALL XABORT(NAMSBR//
     >  ': INVALID TRACKING TYPE FOR EDITING : '//CNDOOR)
        IF(CNDOOR .EQ. 'EXCELL') THEN
          IF(IFTRK2 .EQ. 0) CALL XABORT(NAMSBR//
     >  ': Tracking file missing for SPH with  '//CNDOOR)
        ELSE IF(CNDOOR .EQ. 'NXT') THEN
          IF(IFTRK2 .EQ. 0) CALL XABORT(NAMSBR//
     >  ': Tracking file missing for SPH with  '//CNDOOR)
        ENDIF 
      ELSE IF(NSPH .GT. 0) THEN
        IF(IKGEO1 .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO GEOMETRY STRUCTURE FOUND FOR SPH CALCULATIONS')
        IF(MACGEO .NE. ' ') THEN
          IF(HENTRY(IKGEO1) .EQ. MACGEO) THEN
            IKGEO2=IKGEO1
            IPGEO2=IPGEO1
          ELSE
            IF(IKGEO2 .EQ. 0) THEN
              CALL XABORT(NAMSBR//
     >        ': NO GEOMETRY STRUCTURE FOUND FOR SPH CALCULATIONS')
            ELSE
              IF(HENTRY(IKGEO2) .NE. MACGEO) CALL XABORT(NAMSBR//
     >        ': SPH GEOMETRY '// MACGEO//' NOT FOUUND')
            ENDIF
          ENDIF
        ENDIF
      ENDIF
      IF(NSPH .GE. 2) THEN
        READ(CNDOOR,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'SPH-TRK',3,3,KCHAR)
      ENDIF
C----
C EDITING
C----
      IF(NREG .EQ. 0) CALL XABORT(NAMSBR//': NREG = 0.')
      IF(NGROUP .EQ. 0) CALL XABORT(NAMSBR//': NGROUP = 0.')
      CALL SETARA(RBASE,NREG*NGROUP,JFLUXE)
      CALL EDIDRV(IPEDIT,IPFLUX,IPLIB ,IPSYS ,IPGEO1,IPGEO2,
     >            MAXPTS,MAXISO,MAXISM,CDOOR ,NGROUP,NBMIX ,NREG  ,
     >            LEAKSW,NWGTH ,NIFISS,NEDMAC,TITLE ,NL    ,
     >            ITRANC,NGCOND,NMERGE,IFFAC ,ILUPS ,NSAVES,
     >            NSTATS,CURNAM,OLDNAM,NSPH  ,KSPH  ,CNDOOR,
     >            NBMICR,NACTI ,IPRINT,NTENER,IXEDI ,ISOTXS,
     >            IBASE(IMAT)  ,RBASE(IVOL)  ,IBASE(IIDL)  ,
     >            IBASE(IGCOND),IBASE(IMERGE),IBASE(ISOCAR),
     >            IBASE(IACTI) ,RBASE(JFLUXE),RBASE(IENERG),
     >            RBASE(IENERV),IBASE(ISONAM),
     >            IPTRK2,IFTRK2)
      IF(NSAVES .EQ. 2 .AND. IFSUR .EQ. 1 .AND. NMERGE .EQ. 1) THEN
        IF(ITTRK .EQ. 4) THEN
          NVS=NSUR+NVOL+1
          CALL SETARA(IBASE,NVS,KEYMRG)
          CALL SETARA(IBASE,NVS,MATALB)
          CALL SETARA(RBASE,NVS,VOLSUR) 
          CALL SETARA(RBASE,2*NVS,SVDB) 
          CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMUP)
          CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
          CALL LCMGET(IPTRK,'KEYMRG      ',IBASE(KEYMRG))
          CALL LCMGET(IPTRK,'SAreaRvolume',RBASE(SVDB))
          CALL XDRSDB(NVS,RBASE(VOLSUR),RBASE(SVDB),1)
          DO ISV=0,NSUR-1
            RBASE(VOLSUR+ISV)=RBASE(VOLSUR+ISV)/4.0
          ENDDO
          CALL RLSARA(RBASE(SVDB)) 
          CALL LCMSIX(IPTRK,'NXTRecords  ',ILCMDN)
        ELSE IF(ITTRK .GE.1) THEN
          CALL LCMSIX(IPTRK,'EXCELL      ',ILCMUP)
          CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
          NSUR=ISTATE(2)
          NVOL=ISTATE(3)
          CALL SETARA(IBASE,NSUR+NVOL+1,MATALB)
          CALL SETARA(IBASE,NSUR+NVOL+1,KEYMRG)
          CALL SETARA(RBASE,NSUR+NVOL+1,VOLSUR) 
          CALL LCMGET(IPTRK,'MATALB      ',IBASE(MATALB))
          CALL LCMGET(IPTRK,'KEYMRG      ',IBASE(KEYMRG))
          CALL LCMGET(IPTRK,'VOLSUR      ',RBASE(VOLSUR))
          CALL LCMSIX(IPTRK,'EXCELL      ',ILCMDN)
        ENDIF
        CALL SETARA(RBASE,NSUR,FUNSUR) 
        CALL LCMSIX(IPEDIT,CURNAM,ILCMUP)
        CALL LCMLEN(IPEDIT,'FLXNORMALIZE',ILCMLN,ILCMTY)
        IF(ILCMLN .EQ .1) THEN
          CALL LCMGET(IPEDIT,'FLXNORMALIZE',FLXNOR)
        ELSE
          FLXNOR=1.0
        ENDIF
        CALL LCMSIX(IPEDIT,'MACROLIB    ',ILCMUP)
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMUP)
        CALL EDIADF(IPEDIT,IPFLUX,NGROUP,NSUR  ,NVOL  ,NGCOND,
     >              IPRINT,FLXNOR,
     >              IBASE(IGCOND),IBASE(MATALB),IBASE(KEYMRG),
     >              RBASE(VOLSUR),RBASE(FUNSUR))
        CALL LCMSIX(IPFLUX,'FLUXDIRECT  ',ILCMDN)
        CALL LCMSIX(IPEDIT,'MACROLIB    ',ILCMDN)
        CALL LCMSIX(IPEDIT,CURNAM,ILCMDN)
        CALL RLSARA(RBASE(FUNSUR))
        CALL RLSARA(RBASE(VOLSUR)) 
        CALL RLSARA(IBASE(KEYMRG))
        CALL RLSARA(IBASE(MATALB))
      ENDIF
      CALL RLSARA(RBASE(JFLUXE))
C----
C  RELEASE CONDENSATION/MERGE VECTOR
C----
      CALL RLSARA(IBASE(IACTI))
      CALL RLSARA(IBASE(IMERGE))
      CALL RLSARA(IBASE(IGCOND))
      CALL RLSARA(IBASE(ISOCAR))
      CALL RLSARA(IBASE(ISONAM))
C----
C  RELEASE GENERAL TRACKING INFORMATION.
C----
      CALL RLSARA(IBASE(IIDL))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
      CALL RLSARA(RBASE(IENERV))
      CALL RLSARA(RBASE(IENERG))
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(1X,'EDITION STATE-VECTOR (',I2,') ELEMENTS FROM: ',A6,
     >       /1X,32('-')/
     > ' NMERGE =',I8,' (NUMBER OF MERGED REGIONS)'/
     > ' NGCOND =',I8,' (NUMBER OF CONDENSED ENERGY GROUPS)'/
     > ' IFFAC  =',I8,' (= 0: NO 4 FACTORS CALCULATIONS;'/
     >            17X,'  = 1: 4 FACTORS CALCULATION REQUESTED)'/
     > ' ILUPS  =',I8,' (= 0: KEEP UP-SCATTERING CONTRIBUTION;'/
     >            17X,'  = 1: REMOVE UP-SCATTERING CONTRIBUTIONS)'/
     > ' NACTI  =',I8,' (NUMBER OF MIXTURES WITH ACTIVATION EDITS)'/
     > ' NSTATS =',I8,' (TYPE OF STATISTIC CALCULATIONS)'/
     > ' NSPH   =',I8,' (= 0: NO SPH ;'/
     >            17X,'  = 1: READ SPH FACTORS;'/
     >            17X,'  = 1: TYPE OF CALCULATION FOR SPH FACTORS)'/
     > ' KSPH   =',I8,' (= 1: AVERAGE FLUX SPH NORMALIZATION;'/
     >            17X,'  > 1 : SELENGUT NORMALIZATION)')
 6001 FORMAT(
     > ' NBMICR =',I8,' (<-1: FULL LIBRARY -NBMICR-1 ISOTOPE;'/
     >            17X,'  =-1: FULL LIBRARY ALL ISOTOPE;'/
     >            17X,'  = 0: NO ISOTOPE;'/
     >            17X,'  = 1: MICROSCOPIC ALL ISOTOPE;'/
     >            17X,'  > 1: MICROSCOPIC NBMICR-1 ISOTOPE)'/
     > ' IPRINT =',I8,' (PRINT LEVEL)'/
     > ' NSAVES =',I8,' (= 0: NO COMPUTE/NO SAVE;'/
     >            17X,'  = 1: COMPUTE/NO SAVE;'/
     >            17X,'  = 2: COMPUTE/SAVE;'/
     >            17X,'  = 3: PERTURBATION/SAVE)'/
     > ' NWGTH  =',I8,' (= 0: DIRECT FLUX WEIGHTING P1-SCAT;'/
     >            17X,'  = 1: DIRECT CURRENT WEIGHTING P1-SCAT;'/
     >            17X,'  = 2: COHERENT WEIGHTING FOR P1-SCAT;'/
     >            17X,'  = 3: DIRECTIONAL WEIGHTING FOR P1-SCAT)'/
     > ' MAXISM =',I8,' (MAXIMUM ISOTOPES PER MATERIAL)'/
     >        1X,32('-'))
 6010 FORMAT(' SPH MACRO-CALCULATION TRACKING MODULE IS : ',A8)
 6020 FORMAT(' MERGING INDEX (INITIAL -> FINAL):'/
     >       8(1X,I5,A4,I5))
 9000 FORMAT(1X,A6,': ****** WARNING ******'/
     >       ' MERGE OR TAKE OPTION DE-ACTIVATED '/
     >       ' NUMBER OF REGIONS IN PREVIOUS EDITING      = ',I10/
     >       ' DIFFERENT FROM THAT IN CURRENT CALCULATION = ',I10/
     >       ' **************************')
 9001 FORMAT(1X,A6,': ****** WARNING ******'/
     >  ' CONDENSE OPTION DE-ACTIVATED '/
     >  ' NUMBER OF GROUPS IN PREVIOUS EDITING       = ',I10/
     >  ' DIFFERENT FROM THAT IN CURRENT CALCULATION = ',I10/
     >  ' **************************')
 9002 FORMAT(1X,A6,': ****** WARNING ******'/
     >  ' NO MICROSCOPIC CROSS SECTION AVAILABLE '/
     >  ' NUMBER OF MICROSCOPIC ISOTOPES REQUESTED = ',I10/
     >  ' NUMBER OF MICROSCOPIC ISOTOPES USED      = ',I10/
     >  ' **************************')
 9003 FORMAT(1X,A6,': ****** WARNING ******'/
     >  ' NO MICROSCOPIC CROSS SECTION AVAILABLE '/
     >  ' NUMBER OF ACTIVATION MATERIALS REQUESTED = ',I10/
     >  ' NUMBER OF ACTIVATION MATERIALS USED      = ',I10/
     >  ' **************************')
 9004 FORMAT(1X,A6,': ****** WARNING ******'/
     >  ' NO MICROSCOPIC CROSS SECTION AVAILABLE '/
     >  ' **************************')
      END
