*DECK EDIBAL
      SUBROUTINE EDIBAL(IPEDIT,IPRINT,NGCOND,NMERGE,NTAUXT,IFFAC ,
     >                  EIGENK,B2    ,ILEAK ,
     >                  RATECM,SCATTD,ITYPER,DRATE ,DLEAK )
C
C---------------------------  EDIBAL  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDIBAL
C      FUNCTION : FOUR FACTOR CALCULATION
C      MODIFIED : 97-02-10 (G.M.)
C                 93-07-12 (MARIE-THERESE CAILLARD)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      IPEDIT   : POINTER TO THE EDIT                I
C      IPRINT   : PRINT LEVEL                        I
C                 =1 NEUTRON BALANCE OR FOUR FACTOR
C                 =2 MATERIAL ANALYSIS (IF AVAILABLE)
C      NGCOND   : NUMBER OF GROUPS CONDENSED         I
C      NMERGE   : NUMBER OF REGIONS MERGED           I
C      NTAUXT   : NUMBER OF REACTION RATE EDITS      I
C      IFFAC    : NUMBER OF NEUTRONS IN BILAN        I
C      EIGENK   : PROBLEM EIGENVALUE                 R
C      B2       : SQUARE BUCKLING                    R(4)
C                 FOR ILEAKS = 2 B2(4) IS HOMOGENEOUS
C                 FOR ILEAKS = 3 B2(1),B2(2),B2(3)
C                 ARE DIRECTIONAL HETEROGENEOUS
C                 AND B2(4) IS HOMOGENEOUS
C      ILEAK    : LEAK EVALUATION CONTROL            I
C      RATECM   : AVERAGED REGION/GROUP X-S          D(0:NMERGE,
C                                                 NGCOND,NTAUXT)
C      SCATTD   : SCATTERING MATRIX                  D(NMERGE,NGCOND,
C                                                      NGCOND)
C      ITYPER   : REGION TYPE                        I(NMERGE)
C      DRATE    : DOUBLE PRECISION GROUP RATES       D(2,2,NGCOND+1)
C                 DRATE(1,*,*)  FOR FUEL
C                 DRATE(2,*,*)  FOR NON-FUEL
C                 DRATE(*,1,*)  PRODUCTION
C                 DRATE(*,2,*)  ABSORPTION
C                 DRATE(*,*,NGCOND+1)  GROUP TOTAL
C      DLEAK    : DOUBLE PRECISION GROUP LEAK RATE   D(2,NGCOND+1)
C                 DLEAK(1,*)  LEAKAGE
C                 DLEAK(2,*)  LEAKAGE + ABSORPTION
C                 DLEAK(*,NGCOND+1)  GROUP TOTAL
C
C---------------------------   EDIBAL  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IOUT,INAMAT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,INAMAT=2,NAMSBR='EDIBAL')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPEDIT,IPRINT,NGCOND,NMERGE,NTAUXT,IFFAC,
     >                 ILEAK
      INTEGER          ITYPER(NMERGE)
      REAL             EIGENK,B2(4)
      DOUBLE PRECISION RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                 SCATTD(NMERGE,NGCOND,NGCOND),
     >                 DRATE(2,2,NGCOND+1),DLEAK(2,NGCOND+1)
C----
C  LOCAL PARAMETERS
C----
      INTEGER          IGR,IRAT,NGC1,IREG,JGR,JJ
      REAL             XN(8),FLXRGE,FLXREN,XKINF,XKEFF,
     >                 XN1,XN2,XN3,XN4,XN5,XN6,XA,XLAMF,XLAMTH,
     >                 XAUX,XNORMF
      DOUBLE PRECISION DACCK,BUCKL2,XNF(3)
C----
C  DATA
C----
      CHARACTER        CNAMAT(INAMAT)*15
      SAVE             CNAMAT
      DATA            (CNAMAT(JJ),JJ=1,INAMAT)
     >           /'  FUEL         ','NON-FUEL       '/
C----
C  LOCALIZE FUEL AND NON-FUEL REGION
C----
      NGC1=NGCOND+1
      DO 100 IGR=1,NGC1
        DO 110 IRAT=1,2
          DRATE(1,IRAT,IGR)=0.0D0
          DRATE(2,IRAT,IGR)=0.0D0
 110    CONTINUE
        DLEAK(1,IGR)=0.0D0
        DLEAK(2,IGR)=0.0D0
 100  CONTINUE
      DO 120 IREG=1,NMERGE
        ITYPER(IREG)=2
 120  CONTINUE
      DO 130 IGR=1,NGCOND
        DO 140 IREG=1,NMERGE
          IF(RATECM(IREG,IGR,4) .GT. 0.0D0) THEN
            ITYPER(IREG)=1
          ENDIF
 140    CONTINUE
 130  CONTINUE
      BUCKL2=DBLE(B2(4))
      IF(EIGENK .EQ. 0.0) THEN
        WRITE(IOUT,7000) NAMSBR
        FLXRGE=1.0
        FLXREN=1.0
        DACCK=1.0D0
        WRITE(IOUT,6000)
      ELSE
        FLXRGE=REAL(IFFAC)
        FLXREN=REAL(IFFAC)*EIGENK
        DACCK=DBLE(EIGENK)
        IF(NGCOND .NE. 3) THEN
          WRITE(IOUT,7001) NAMSBR,NGCOND
          WRITE(IOUT,6001)
        ELSE
          WRITE(IOUT,6002)
        ENDIF
      ENDIF
      IF(IPRINT .EQ. 1) THEN
        WRITE(IOUT,6100)
        WRITE(IOUT,6101)(IREG,CNAMAT(ITYPER(IREG)),IREG=1,NMERGE)
      ENDIF
      IF(IPRINT .GE. 1) THEN
        WRITE(IOUT,6200)
      ENDIF
      DO 170 IGR=1,NGCOND
C----
C  REACTION RATES PER GROUP AND MATERIAL TYPE
C----
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,6201) IGR,NGCOND
        ENDIF
        DO 180 IREG=1,NMERGE
C----
C  ADD PRODUCTION AND ABSORPTION IN MATERIAL TYPE
C----
          IF(ITYPER(IREG) .EQ. 1) THEN
            DRATE(1,1,IGR)=DRATE(1,1,IGR)+RATECM(IREG,IGR,5)*
     >                     RATECM(IREG,IGR,1)
            DRATE(1,2,IGR)=DRATE(1,2,IGR)+RATECM(IREG,IGR,3)*
     >                     RATECM(IREG,IGR,1)
          ELSE
            DRATE(2,1,IGR)=DRATE(2,1,IGR)+RATECM(IREG,IGR,5)*
     >                     RATECM(IREG,IGR,1)
            DRATE(2,2,IGR)=DRATE(2,2,IGR)+RATECM(IREG,IGR,3)*
     >                     RATECM(IREG,IGR,1)
          ENDIF
C----
C  PRINT PRODUCTION AND ABSORPTION PER REGION
C----
          IF(IPRINT .GE. 2) THEN
            IF(IFFAC .EQ. 1000) THEN
              WRITE(IOUT,6300) IREG,CNAMAT(ITYPER(IREG)),
     >                  FLXRGE*RATECM(IREG,IGR,5)*RATECM(IREG,IGR,1),
     >                  FLXREN*RATECM(IREG,IGR,3)*RATECM(IREG,IGR,1)
            ELSE
              WRITE(IOUT,6301) IREG,CNAMAT(ITYPER(IREG)),
     >                  FLXRGE*RATECM(IREG,IGR,5)*RATECM(IREG,IGR,1),
     >                  FLXREN*RATECM(IREG,IGR,3)*RATECM(IREG,IGR,1)
            ENDIF
          ENDIF
C----
C  ADD GROUP LEAKAGE
C----
          IF(ILEAK .EQ. 3) THEN
            DLEAK(1,IGR)=DLEAK(1,IGR)+RATECM(IREG,IGR,1)
     >       *(RATECM(IREG,IGR,11)*B2(1)
     >        +RATECM(IREG,IGR,12)*B2(2)
     >        +RATECM(IREG,IGR,13)*B2(3))
          ELSE IF(ILEAK .EQ. 2) THEN
            DLEAK(1,IGR)=DLEAK(1,IGR)+RATECM(IREG,IGR,6)
     >                        *B2(4)*RATECM(IREG,IGR,1)
          ELSE
            DO 190 JGR=1,NGCOND
              IF(JGR .NE. IGR)
     >          DLEAK(2,IGR)=DLEAK(2,IGR)+SCATTD(IREG,IGR,JGR)
     >               *RATECM(IREG,JGR,1)
 190        CONTINUE
            DLEAK(2,IGR)=DLEAK(2,IGR)+RATECM(IREG,IGR,1)*
     >                     ( RATECM(IREG,IGR,5)/DACCK
     >                      -RATECM(IREG,IGR,2)
     >                      +RATECM(IREG,IGR,3)
     >                      +SCATTD(IREG,IGR,IGR) )
          ENDIF
 180    CONTINUE
        IF(ILEAK .EQ. 2 .OR. ILEAK .EQ. 3) THEN
          DLEAK(2,IGR)=DLEAK(1,IGR)+DRATE(1,2,IGR)+DRATE(2,2,IGR)
        ELSE
          DLEAK(1,IGR)=DLEAK(2,IGR)-DRATE(1,2,IGR)-DRATE(2,2,IGR)
        ENDIF
C----
C  PRINT PRODUCTION AND ABSORPTION PER MATERIAL TYPE
C  AND TOTAL PRODUCTION, ABSORPTION, LEAKAGE AND LEAKAGE+ABSORPTION
C----
        IF(IPRINT .GE. 1) THEN
          IF(IFFAC .EQ. 1000) THEN
            WRITE(IOUT,6302) 'TOTAL FUEL     ',
     >                  FLXRGE*DRATE(1,1,IGR),
     >                  FLXREN*DRATE(1,2,IGR)
            WRITE(IOUT,6302) 'TOTAL NON-FUEL ',
     >                  FLXRGE*DRATE(2,1,IGR),
     >                  FLXREN*DRATE(2,2,IGR)
            WRITE(IOUT,6302) 'FUEL + NON-FUEL',
     >                  FLXRGE*(DRATE(1,1,IGR)+DRATE(2,1,IGR)),
     >                  FLXREN*(DRATE(1,2,IGR)+DRATE(2,2,IGR)),
     >                  FLXREN*DLEAK(1,IGR),FLXREN*DLEAK(2,IGR)
          ELSE
            WRITE(IOUT,6303) 'TOTAL FUEL     ',
     >                  FLXRGE*DRATE(1,1,IGR),
     >                  FLXREN*DRATE(1,2,IGR)
            WRITE(IOUT,6303) 'TOTAL NON-FUEL ',
     >                  FLXRGE*DRATE(2,1,IGR),
     >                  FLXREN*DRATE(2,2,IGR)
            WRITE(IOUT,6303) 'FUEL + NON-FUEL',
     >                  FLXRGE*(DRATE(1,1,IGR)+DRATE(2,1,IGR)),
     >                  FLXREN*(DRATE(1,2,IGR)+DRATE(2,2,IGR)),
     >                  FLXREN*DLEAK(1,IGR),FLXREN*DLEAK(2,IGR)
          ENDIF
        ENDIF
C----
C  GROUP SUM
C----
        DRATE(1,1,NGC1)=DRATE(1,1,NGC1)+DRATE(1,1,IGR)
        DRATE(2,1,NGC1)=DRATE(2,1,NGC1)+DRATE(2,1,IGR)
        DRATE(1,2,NGC1)=DRATE(1,2,NGC1)+DRATE(1,2,IGR)
        DRATE(2,2,NGC1)=DRATE(2,2,NGC1)+DRATE(2,2,IGR)
        DLEAK(1,NGC1)=DLEAK(1,NGC1)+DLEAK(1,IGR)
        DLEAK(2,NGC1)=DLEAK(2,NGC1)+DLEAK(2,IGR)
 170  CONTINUE
      IF(NGCOND .GT. 1) THEN
        WRITE(IOUT,6202)
        IF(IFFAC .EQ. 1000) THEN
          WRITE(IOUT,6302) 'TOTAL FUEL     ',
     >                FLXRGE*DRATE(1,1,NGC1),
     >                FLXREN*DRATE(1,2,NGC1)
          WRITE(IOUT,6302) 'TOTAL NON-FUEL ',
     >                FLXRGE*DRATE(2,1,NGC1),
     >                FLXREN*DRATE(2,2,NGC1)
          WRITE(IOUT,6302) 'FUEL + NON-FUEL',
     >                FLXRGE*(DRATE(1,1,NGC1)+DRATE(2,1,NGC1)),
     >                FLXREN*(DRATE(1,2,NGC1)+DRATE(2,2,NGC1)),
     >                FLXREN*DLEAK(1,NGC1),FLXREN*DLEAK(2,NGC1)
        ELSE
          WRITE(IOUT,6303) 'TOTAL FUEL     ',
     >                FLXRGE*DRATE(1,1,NGC1),
     >                FLXREN*DRATE(1,2,NGC1)
          WRITE(IOUT,6303) 'TOTAL NON-FUEL ',
     >                FLXRGE*DRATE(2,1,NGC1),
     >                FLXREN*DRATE(2,2,NGC1)
          WRITE(IOUT,6303) 'FUEL + NON-FUEL',
     >                FLXRGE*(DRATE(1,1,NGC1)+DRATE(2,1,NGC1)),
     >                FLXREN*(DRATE(1,2,NGC1)+DRATE(2,2,NGC1)),
     >                FLXREN*DLEAK(1,NGC1),FLXREN*DLEAK(2,NGC1)
        ENDIF
      ENDIF
      IF(EIGENK .GT. 0.0 .AND. NGCOND .EQ. 3) THEN
C----
C  FOUR FACTOR CALCULATION
C----
        XNF(1)=0.0D0
        XNF(2)=0.0D0
        XNF(3)=0.0D0
        DO 200 IREG=1,NMERGE
C----
C  ADD NUSIGF AND TOTAL RATES IN FUEL
C----
          IF(ITYPER(IREG) .EQ. 1) THEN
            XNF(3)=XNF(3)+RATECM(IREG,3,4)*RATECM(IREG,3,1)
            XNF(2)=XNF(2)+RATECM(IREG,2,4)*RATECM(IREG,2,1)
            XNF(1)=XNF(1)+RATECM(IREG,1,4)*RATECM(IREG,1,1)
          ENDIF
 200    CONTINUE
        XKINF=DRATE(1,1,4)/(DRATE(1,2,4)+DRATE(2,2,4))
        XKEFF=DRATE(1,1,4)/DLEAK(2,4)
        XN1=(DRATE(1,1,4)/XKEFF)-DLEAK(1,4)
        XN2=XN1-(DRATE(1,2,1)+DRATE(2,2,1))
        XN3=XN2-(DRATE(1,2,2)+DRATE(2,2,2))
        XN4=XN3-DRATE(2,2,3)
        XN5=XNF(2)+XNF(3)
        XN6=XN5+XNF(1)
C----
C  COMPUTATION OF EPSILON, F, ETA, P, LAMBDAF, LAMBDATH,
C                 KINF, KEFF
C----
        XN(1)=(XN6-DRATE(1,2,1)-DRATE(2,2,1))/(XN5)
        XN(5)=XN4/XN3
        XN(6)=XN5/XN4
        XN(3)=XKINF/(XN(1)*XN(5)*XN(6))
        XA=DRATE(1,2,4)+DRATE(2,2,4)
        XLAMF=1/(1+((DLEAK(1,1)+DLEAK(1,2))/XA))
        XLAMTH=1/(1+(DLEAK(1,3)/XA))
        XAUX=XLAMF*XLAMTH
        XNORMF=XKEFF/(XKINF*XAUX)
        XN(2)=XLAMF*SQRT(XNORMF)
        XN(4)=XLAMTH*SQRT(XNORMF)
        XN(7)=XN(1)*XN(3)*XN(5)*XN(6)
        XN(8)=XN(7)*XN(2)*XN(4)
        IF(IPRINT .GE. 1) THEN
          WRITE(IOUT,6400) (XN(JJ),JJ=1,8)
        ENDIF
        CALL LCMPUT(IPEDIT,'FOUR-FACTOR ',8,2,XN)
      ENDIF
      RETURN
C----
C  PRINT FORMATS
C----
 6000 FORMAT(/20X,'F I X E D    S O U R C E',
     >'    N E U T R O N    B A L A N C E')
 6001 FORMAT(/20X,'S I M P L E    N E U T R O N    B A L A N C E')
 6002 FORMAT(/20X,'F O U R    F A C T O R    C A L C U L A T I O N')
 6100 FORMAT(4(4X,'REGION',5X,'MATERIAL TYPE  '))
 6101 FORMAT(4(5X,I5,5X,A15))
 6200 FORMAT(15X,' REGION',3X,'MATERIAL TYPE  ',3X,'NEUTRON PRODUCTION',
     >       3X,'NEUTRON ABSORPTION',3X,'NEUTRON LEAKAGE   ',
     >       3X,'ABSORPTION+LEAKAGE')
 6201 FORMAT(' GROUP :',I3,'/',I3)
 6202 FORMAT(' SUM OVER GROUPS')
 6300 FORMAT(17X,I5,3X,A15,3X,F12.1,8X,F12.1)
 6301 FORMAT(17X,I5,3X,A15,3X,1P,E15.7,5X,E15.7)
 6302 FORMAT(25X,A15,3X,F12.1,8X,F12.1,8X,F12.1,8X,F12.1)
 6303 FORMAT(25X,A15,3X,1P,E15.7,5X,E15.7,5X,E15.7,5X,E15.7)
 6400 FORMAT(/' FOUR FACTORS'/1P,
     >        ' EPSILON (FAST FISSION FACTOR)  =',E15.7/
     >        ' LAMBDAF (FAST NON-LEAKAGE)     =',E15.7/
     >        ' P (ANTITRAP FACTOR)            =',E15.7/
     >        ' LAMBDAT (THERMAL NON-LEAKAGE)  =',E15.7/
     >        ' F (THERMAL UTILIZATION FACTOR) =',E15.7/
     >        ' ETA (THERMAL REPRODUCTION)     =',E15.7/
     >        ' INFINITE MULTIPLICATION (4F)   =',E15.7/
     >        ' EFFECTIVE MULTIPLICATION       =',E15.7)
C----
C  WARNING FORMATS
C----
 7000 FORMAT(1X,A6,' * * *  W A R N I N G  * * * ',/
     >       ' NO FOUR FACTOR CALCULATION PERMITTED FOR FIXED SOURCE',
     >       ' PROBLEM',/' A SIMPLE GROUP BY GROUP NEUTRON',
     >       ' BALANCE PERFORMED HERE')
 7001 FORMAT(1X,A6,' * * *  W A R N I N G  * * * ',/
     >       ' FOUR FACTOR CALCULATION REQUIRES 3 GROUPS, NUMBER OF',
     >       ' GROUPS HERE IS =',I10/' A SIMPLE GROUP BY GROUP NEUTRON',
     >       ' BALANCE PERFORMED HERE')
      END
