*DECK EDIHFC
      SUBROUTINE EDIHFC(IPEDIT,IPLIB ,NGROUP,NGCOND,NREGIO,NMERGE,
     >                  NBISO ,NREAC ,NDEPL ,MATCOD,ISONAM,CURNAM,
     >                  IGCOND,IMERGE,
     >                  INADPL,ISONRF,MIX   ,FLXINT,FLXMER,DEN   ,
     >                  IDR   ,RER   ,SIGF  ,FACTH ,DHFCT ,EJOULE,
     >                  DFISS ,DEPRO )
C
C---------------------------  EDIHFC  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDIHFC
C      USE      : EVALUATE H-FACTORS USING INFORMATION RECOVERED FROM
C                 THE REFERENCE INTERNAL LIBRARY AND STORE THEM ON LCM
C      MODIFIED : 97-02-05 (G.M.)
C                 91-12-04 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C      IPEDIT   : POINTER TO THE EDIT                  I
C      IPLIB    : POINTER TO THE REF. INTERNAL LIBRARY I
C      NGROUP   : NUMBER OF GROUPS                     I
C      NGCOND   : NUMBER OF GROUPS CONDENSED           I
C      NREGIO   : NUMBER OF REGIONS                    I
C      NMERGE   : NUMBER OF REGIONS MERGED             I
C      NBISO    : NUMBER OF ISOTOPES                   I
C      NREAC    : NUMBER OF DEPLETION REACTIONS        I
C      NDEPL    : NUMBER OF DEPLETING ISOTOPES         I
C      MATCOD   : MATERIAL PER REGION                  I(NREGIO)
C      ISONAM   : ISOTOPES NAMES                       I(3,NBISO)
C      CURNAM   : NAME OF LCM DIRECTORY WHERE THE
C                 MERGED/CONDENSED X-S ARE STORED      C*12
C      IGCOND   : LIMIT CONDENSED GROUPS               I(NGCOND)
C      IMERGE   : INDEX OF MERGED REGION               I(NREGIO)
C      INADPL   : NAME OF DEPLETING ISOTOPES           I(3,NEDPL)
C      ISONRF   : ISOTOPES LIBRARY NAMES               I(3,NBISO)
C      MIX      : MIXTURE ASSOCIATED TO ISOTOPES       I(NBISO)
C      FLXINT   : NORMALIZED MULTIGROUP INT FLUX       R(NREGIO,NGROUP)
C      FLXMER   : AVERAGED REGION/GROUP FLUX           D(0:NMERGE,
C                                                      NGCOND)
C      DEN      : ISOTOPE DENSITY                      R(NBISO)
C      IDR      : DEPLETION REACTIONS                  I(NREAC,NDEPL)
C      RER      : FISSION & CAPTURE PRODUCTION ENERGY  R(NREAC,NDEPL)
C      SIGF     : FISSION X-S                          R(NGROUP)
C      FACTH    : H-FACTORS IN A MACROGROUP            R(NMERGE)
C      DHFCT    : H FACTORS                            D(NMERGE,NGCOND)
C      EJOULE   : ENERGY IN MEGA-JOULES FOR 10**24     R(NBISO)
C                 FISSION
C      DFISS    : FISSION                              D(NMERGE)
C      DEPRO    : ENERGY PRODUCTION MEGA-JOULES        D(NMERGE)
C
C---------------------------   EDIHFC  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IOUT,KFISSI,ILCMUP,ILCMDN
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,KFISSI=6,ILCMUP=1,ILCMDN=2,
     >                 NAMSBR='EDIHFC')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPEDIT,IPLIB,NGROUP,NGCOND,NREGIO,NMERGE,NBISO,
     >                 NREAC,NDEPL
      CHARACTER        CURNAM*12
      INTEGER          MATCOD(NREGIO),ISONAM(3,NBISO),
     >                 IGCOND(NGCOND),IMERGE(NREGIO),INADPL(3,NDEPL),
     >                 ISONRF(3,NBISO),MIX(NBISO),IDR(NREAC,NDEPL)
      REAL             FLXINT(NREGIO,NGROUP),DEN(NBISO),
     >                 RER(NREAC,NDEPL),SIGF(NGROUP),FACTH(NMERGE),
     >                 EJOULE(NBISO)
      DOUBLE PRECISION FLXMER(0:NMERGE,NGCOND),DHFCT(NMERGE,NGCOND),
     >                 DFISS(NMERGE),DEPRO(NMERGE)
      DOUBLE PRECISION XDRCST
C----
C  LOCAL PARAMETERS
C----
      CHARACTER        HNAMIS*12,CGRPNM*12
      INTEGER          IZFISS,IGC,IMR,ISO,IFIS,IDP,ITC,ILCMLN,ILCMTY,
     >                 IREG,IGRFIN,IGRDEB,IGR
      DOUBLE PRECISION DCONT
C----
C  COMPUTE H-FACTORS
C----
      IZFISS=0
      DO 100 IGC=1,NGCOND
        DO 101 IMR=1,NMERGE
          DHFCT(IMR,IGC)=0.0D0
 101    CONTINUE
 100  CONTINUE
      CALL XDDSET(DFISS,NMERGE,0.0D0)
      CALL XDDSET(DEPRO,NMERGE,0.0D0)
      DO 110 ISO=1,NBISO
        EJOULE(ISO)=0.0
        IFIS=0
        DO 120 IDP=1,NDEPL
          IF(ISONRF(1,ISO) .EQ. INADPL(1,IDP) .AND.
     >       ISONRF(2,ISO) .EQ. INADPL(2,IDP) .AND.
     >       ISONRF(3,ISO) .EQ. INADPL(3,IDP)) THEN
            IF(MOD(IDR(5,IDP),10) .EQ. KFISSI) THEN
              IFIS=IDP
              EJOULE(ISO)=RER(5,IDP)*1.0E24*XDRCST('eV','J')
              GO TO 125
            ENDIF
          ENDIF
 120    CONTINUE
 125    CONTINUE
        IF(IFIS .GT. 0) THEN
          WRITE(HNAMIS,'(3A4)') (ISONAM(ITC,ISO),ITC=1,3)
          CALL LCMSIX(IPLIB,HNAMIS,ILCMUP)
          CALL LCMLEN(IPLIB,'NFTOT',ILCMLN,ILCMTY)
          IF(ILCMLN .EQ. NGROUP) THEN
            IZFISS=IZFISS+1
            CALL LCMGET(IPLIB,'NFTOT',SIGF)
            DO 130 IREG=1,NREGIO
              IMR=IMERGE(IREG)
              IF(IMR .GT. 0 .AND. MATCOD(IREG) .EQ. MIX(ISO)) THEN
                IGRFIN=0
                DO 131 IGC=1,NGCOND
                  IGRDEB=IGRFIN+1
                  IGRFIN=IGCOND(IGC)
                  DO 132 IGR=IGRDEB,IGRFIN
                    DCONT=DBLE(FLXINT(IREG,IGR)*DEN(ISO)
     >                         *SIGF(IGR)*EJOULE(ISO))
                    DHFCT(IMR,IGC)=DHFCT(IMR,IGC)+DCONT
                    DEPRO(IMR)=DEPRO(IMR)+DCONT
                    DFISS(IMR)=DFISS(IMR)+DBLE(FLXINT(IREG,IGR)
     >                         *DEN(ISO)*SIGF(IGR))
 132              CONTINUE
 131            CONTINUE
              ENDIF
 130        CONTINUE
          ENDIF
          CALL LCMSIX(IPLIB,HNAMIS,2)
        ENDIF
 110  CONTINUE
C----
C  COMPUTE THE HOMOGENIZED/CONDENSED HFACTOR
C----
      IF(IZFISS.NE.0) THEN
        DO 140 IGC=1,NGCOND
          DO 141 IMR=1,NMERGE
            IF(FLXMER(IMR,IGC).GT.0.0D0) THEN
              DHFCT(IMR,IGC)=DHFCT(IMR,IGC)/FLXMER(IMR,IGC)
            ELSE
              DHFCT(IMR,IGC)=0.0D0
            ENDIF
 141      CONTINUE
 140    CONTINUE
      ENDIF
C----
C  SAVE ON LCM
C----
      CALL LCMSIX(IPEDIT,CURNAM,ILCMUP)
      CALL LCMSIX(IPEDIT,'MACROLIB',ILCMUP)
      DO 150 IGC=1,NGCOND
        WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGC,NGCOND
        CALL LCMSIX(IPEDIT,CGRPNM,ILCMUP)
        CALL XDRSDB(NMERGE,FACTH,DHFCT(1,IGC),1)
        CALL LCMPUT(IPEDIT,'H-FACTORS',NMERGE,2,FACTH)
        CALL LCMSIX(IPEDIT,CGRPNM,ILCMDN)
 150  CONTINUE
C----
C  COMPUTE THE HOMOGENIZED/CONDENSED EJOULE
C----
      CALL XDRSET(FACTH,NMERGE,0.0)
      DO 160 IMR=1,NMERGE
        IF(DFISS(IMR).GT.0.0D0) THEN
          FACTH(IMR)=DEPRO(IMR)/DFISS(IMR)
        ENDIF
 160  CONTINUE
      CALL LCMPUT(IPEDIT,'MACRO-EMJ',NMERGE,2,FACTH)
      CALL LCMSIX(IPEDIT,'MACROLIB',ILCMDN)
      CALL LCMSIX(IPEDIT,CURNAM,ILCMDN)
      RETURN
      END
