*DECK EDIMRR
      SUBROUTINE EDIMRR(NGROUP,NGCOND,NPROC,MINLEG,MAXLEG,NMERGE,
     >                  IMR,ILEAKS,IGCOND,INDPRO,ITYPRO,XSREC,XSSCT,
     >                  DFLXDN,DRATE,DRSCT)
C
C---------------------------  EDIMRR  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDIMRR
C      USE      : EVALUATE ISOTOPIC REACTION RATES FROM
C                 MICROSCOPIC CROSS SECTION
C      MODIFIED : 97-02-06 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      NGROUP   : NUMBER OF GROUPS               I
C      NGCOND   : NUMBER OF GROUPS CONDENSED     I
C      NPROC    : NUMBER OF TERMS TO PROCESS     I
C      MINLEG   : MIMIMUM LEGENDRE ORDER TO      I
C                 PROCESS FOR SCATTERING
C      MAXLEG   : MIMIMUM LEGENDRE ORDER TO      I
C                 PROCESS FOR SCATTERING
C      NMERGE   : NUMBER OF MERGED REGIONS       I
C      IMR      : CURRENT MERGED REGION          I
C      ILEAKS   : LEAK CALCULATION               I
C                 = 0 NO LEAKAGE
C                 = 1 LEAKAGE BY SURFACE (ALBEDO>0.0)
C                 = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
C                 = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
C      IGCOND   : LIMIT CONDENSED GROUPS         I(NGCOND)
C      INDPRO   : VECTOR FOR CROSS SECTION TO    I(NPROC)
C                 PROCESS
C                 = 0 -> DO NOT PROCESS
C                 > 0 -> PROCESS
C      ITYPRO   : VECTOR FOR CROSS SECTION       I(NPROC)
C                 = 0 -> ABSENT  (NOT PROCESSED)
C                 > 0 -> PRESENT (PROCESSED)
C      XSREC    : GENERAL XS                     R(NGROUP,NPROC)
C      XSSCT    : SCATTERING XS                  R(NGROUP,NGROUP,
C                 FOR ORDER MINLEG TO MAXLEG     MAXLEG-MINLEG+1)
C      DFLXDN   : FLUXES                         D(NGROUP,4)
C    OUTPUT
C      DRATE     : GENERAL REACTION RATE         D(NGCOND,NPROC,
C                                                         NMERGE)
C      DRSCT     : SCATTERING REACTION RATE      D(NGCOND,NGCOND,
C                  FOR ORDER MINLEG TO MAXLEG    MAXLEG-MINLEG+1,
C
C---------------------------   EDIMRR  --------------------------------
C
      IMPLICIT          NONE
      INTEGER           IOUT,NDPROC
      CHARACTER         NAMSBR*6
      PARAMETER        (IOUT=6,NDPROC=20,NAMSBR='EDIMRR')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER           NGROUP,NGCOND,NPROC,MINLEG,MAXLEG,NMERGE,
     >                  IMR,ILEAKS
      INTEGER           IGCOND(NGROUP),INDPRO(NPROC),ITYPRO(NPROC)
      REAL              XSREC(NGROUP,NPROC),
     >                  XSSCT(NGROUP,NGROUP,MAXLEG-MINLEG+1)
      DOUBLE PRECISION  DFLXDN(NGROUP,4),DRATE(NGCOND,NPROC,NMERGE),
     >                  DRSCT(NGCOND,NGCOND,MAXLEG-MINLEG+1,NMERGE)
C----
C  LOCAL PARAMETERS
C----
      INTEGER           JGRD,JGR,JGRC,JGRF,JXSR,
     >                  IGRD,IGR,IGRC,IGRF,IXSR,IL
      DOUBLE PRECISION  DXFACT
C----
C  CONDENSE PRIMARY GROUP
C----
      IGRF=0
      DO 100 IGRC=1,NGCOND
        IGRD=IGRF+1
        IGRF=IGCOND(IGRC)
        DO 110 IGR=IGRD,IGRF
C----
C  PROCESS TOTAL, TRAN, NUSIGF AND NFTOT IF PRESENT
C  PROCESS ADDITIONAL XS IF PRESENT
C----
          DO 120 IXSR=1,4
            IF(INDPRO(IXSR).GT.0.AND.ITYPRO(IXSR).GT.0) THEN
              DRATE(IGRC,IXSR,IMR)=DRATE(IGRC,IXSR,IMR)
     >                      +DBLE(XSREC(IGR,IXSR))*DFLXDN(IGR,1)
            ENDIF
 120      CONTINUE
C----
C  PROCESS ADDITIONAL XS IF PRESENT
C----
          DO 130 IXSR=7,NDPROC
            IF(INDPRO(IXSR).GT.0.AND.ITYPRO(IXSR).GT.0) THEN
              DRATE(IGRC,IXSR,IMR)=DRATE(IGRC,IXSR,IMR)
     >                      +DBLE(XSREC(IGR,IXSR))*DFLXDN(IGR,1)
            ENDIF
 130      CONTINUE
C----
C  PROCESS SCATTERING XS IF PRESENT
C----
          DXFACT=DFLXDN(IGR,2)+DFLXDN(IGR,3)+DFLXDN(IGR,4)
          IL=0
          DO 140 IXSR=NDPROC+MINLEG+1,NDPROC+MAXLEG+1
            IL=IL+1
            IF (IL.EQ.2.AND.ILEAKS.EQ.3) THEN
C----
C  FOR B1 HETEROGENOUS LEAKAGE MODEL
C  CORRECT LINEARLY ANISOTROPIC SCATTERING
C  AS  SUM_{K}[ J_{K} (SIGS(1)-TOTAL) ]
C-----
              DRATE(IGRC,IXSR,IMR)=DRATE(IGRC,IXSR,IMR)
     >          +DBLE(XSREC(IGR,IXSR))*DXFACT
     >          -DBLE(XSREC(IGR,1))*DXFACT
              JGRF=0
              DO 150 JGRC=1,NGCOND
                JGRD=JGRF+1
                JGRF=IGCOND(JGRC)
                DO 151 JGR=JGRD,JGRF
                  DRSCT(JGRC,IGRC,IL,IMR)=DRSCT(JGRC,IGRC,IL,IMR)
     >            +DBLE(XSSCT(JGR,IGR,IL))*DXFACT
                  IF(IGR.EQ.JGR) THEN
                    DRSCT(JGRC,IGRC,IL,IMR)=DRSCT(JGRC,IGRC,IL,IMR)
     >                -DBLE(XSREC(IGR,1))*DXFACT
                  ENDIF
 151            CONTINUE
 150          CONTINUE
            ELSE
C----
C  FOR OTHER LEAKAGE MODEL AND OTHER SCATTERING ORDER
C  CORRECT SCATTERING
C  AS  [ PHI  SIGS ]
C-----
              IF(INDPRO(IXSR).GT.0.AND.ITYPRO(IXSR).GT.0) THEN
                DRATE(IGRC,IXSR,IMR)=DRATE(IGRC,IXSR,IMR)
     >                      +DBLE(XSREC(IGR,IXSR))*DFLXDN(IGR,1)
                JGRF=0
                DO 160 JGRC=1,NGCOND
                  JGRD=JGRF+1
                  JGRF=IGCOND(JGRC)
                  DO 161 JGR=JGRD,JGRF
                  DRSCT(JGRC,IGRC,IL,IMR)=DRSCT(JGRC,IGRC,IL,IMR)
     >              +DBLE(XSSCT(JGR,IGR,IL))*DFLXDN(IGR,1)
 161              CONTINUE
 160            CONTINUE
              ENDIF
            ENDIF
 140      CONTINUE
C----
C  PROCESS CHI IF PRESENT
C----
          IXSR=5
          JXSR=3
          IF(INDPRO(IXSR).GT.0.AND.ITYPRO(IXSR).GT.0) THEN
            JGRF=0
            DO 170 JGRC=1,NGCOND
              JGRD=JGRF+1
              JGRF=IGCOND(JGRC)
              DO 180 JGR=JGRD,JGRF
                DRATE(JGRC,IXSR,IMR)=DRATE(JGRC,IXSR,IMR)
     >          +DBLE(XSREC(IGR,JXSR)*XSREC(JGR,IXSR))*DFLXDN(IGR,1)
 180          CONTINUE
 170        CONTINUE
          ENDIF
C----
C  PROCESS NU IF PRESENT
C----
          IXSR=6
          JXSR=4
          IF(INDPRO(IXSR).GT.0.AND.ITYPRO(IXSR).GT.0) THEN
            DRATE(IGRC,IXSR,IMR)=DRATE(IGRC,IXSR,IMR)
     >       +DBLE(XSREC(IGR,JXSR)*XSREC(IGR,IXSR))*DFLXDN(IGR,1)
          ENDIF
 110    CONTINUE
 100  CONTINUE
      RETURN
      END
