*DECK EDIMXS
      SUBROUTINE EDIMXS(NGCOND,NPROC ,MINLEG,MAXLEG,NMERGE,IMR,
     >                  ILEAKS,INDPRO,DFLDNC,DRATE,DRSCT,PERT,
     >                  XSREC,XSSCT,DXSTRD)
C
C---------------------------  EDIMXS  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDIMXS
C      USE      : EVALUATE MICROSCOPIC CROSS SECTION
C                 FROM ISOTOPIC REACTION RATES
C      MODIFIED : 97-02-06 (G.M.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      NGCOND   : NUMBER OF CONDENSED GROUPS     I
C      NPROC    : NUMBER OF TERMS TO PROCESS     I
C      MINLEG   : MIMIMUM LEGENDRE ORDER TO      I
C                 PROCESS FOR SCATTERING
C      MAXLEG   : MIMIMUM LEGENDRE ORDER TO      I
C                 PROCESS FOR SCATTERING
C      NMERGE   : NUMBER OF MERGED REGIONS       I
C      IMR      : CURRENT MERGED REGION          I
C      ILEAKS   : LEAK CALCULATION               I
C                 = 0 NO LEAKAGE
C                 = 1 LEAKAGE BY SURFACE (ALBEDO>0.0)
C                 = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
C                 = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
C      DFLDNC   : FLUXES                         D(NGCOND,3)
C      DRATE    : GENERAL REACTION RATE          D(NGCOND,NPROC,
C                                                         NMERGE)
C      DRSCT    : SCATTERING REACTION RATE       D(NGCOND,NGCOND,
C                 FOR ORDER MINLEG TO MAXLEG     MAXLEG-MINLEG+1,
C                                                         NMERGE)
C      DXSTRD   : EQUIVALENT XS TRANSPORT        D(4,NGCOND)
C      PERT     : TERM FOR PERTURBATION          R
C    OUTPUT
C      XSREC    : GENERAL XS                     R(NGCOND,NPROC)
C      XSSCT    : SCATTERING XS                  R(NGCOND,NGCOND,
C                 FOR ORDER MINLEG TO MAXLEG     MAXLEG-MINLEG+1)
C
C---------------------------   EDIMXS  --------------------------------
C
      IMPLICIT          NONE
      INTEGER           IOUT,NDPROC
      CHARACTER         NAMSBR*6
      PARAMETER        (IOUT=6,NDPROC=20,NAMSBR='EDIMXS')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER           NGCOND,NPROC,MINLEG,MAXLEG,NMERGE,IMR,
     >                  ILEAKS
      REAL              PERT,XSREC(NGCOND,NPROC),
     >                  XSSCT(NGCOND,NGCOND,MAXLEG-MINLEG+1)
      DOUBLE PRECISION  DFLDNC(NGCOND,3),DRATE(NGCOND,NPROC,NMERGE),
     >                  DRSCT(NGCOND,NGCOND,MAXLEG-MINLEG+1,NMERGE),
     >                  DXSTRD(4,NGCOND)
C----
C  LOCAL PARAMETERS
C----
      INTEGER           INDPRO(NPROC),IXSR,JXSR,ISTRD,IGRC,JGRC,IL
      DOUBLE PRECISION  DTEMP,DFACTJ,DFACTX
C----
C  PROCESS TOTAL, TRAN, NUSIGF AND NFTOT IF PRESENT
C  PROCESS ADDITIONAL XS IF PRESENT
C----
      IF(PERT.EQ.0.0) THEN
        DO 100 IXSR=1,4
          IF(INDPRO(IXSR).GT.0) THEN
            DO 101 IGRC=1,NGCOND
              XSREC(IGRC,IXSR)=DRATE(IGRC,IXSR,IMR)*DFLDNC(IGRC,1)
 101        CONTINUE
          ENDIF
 100    CONTINUE
C----
C  PROCESS ADDITIONAL XS IF PRESENT
C----
        DO 110 IXSR=7,NDPROC-4
          IF(INDPRO(IXSR).GT.0) THEN
            DO 111 IGRC=1,NGCOND
              XSREC(IGRC,IXSR)=DRATE(IGRC,IXSR,IMR)*DFLDNC(IGRC,1)
 111        CONTINUE
          ENDIF
 110    CONTINUE
C----
C  PROCESS SCATTERING XS IF PRESENT
C----
        IL=0
        DO 120 IXSR=NDPROC+MINLEG+1,NDPROC+MAXLEG+1
          IL=IL+1
          IF(INDPRO(IXSR).GT.0) THEN
            DO 121 IGRC=1,NGCOND
              IF(ILEAKS.EQ.3.AND.IL.EQ.2) THEN
                DFACTJ=DFLDNC(IGRC,2)
                DFACTX=XSREC(IGRC,1)+DFLDNC(IGRC,3)
              ELSE
                DFACTJ=DFLDNC(IGRC,1)
                DFACTX=0.0D0
              ENDIF
              XSREC(IGRC,IXSR)=DRATE(IGRC,IXSR,IMR)*DFACTJ+DFACTX
              DO 122 JGRC=1,NGCOND
                IF(IGRC.EQ.JGRC) THEN
                  XSSCT(JGRC,IGRC,IL)=
     >              DRSCT(JGRC,IGRC,IL,IMR)*DFACTJ+DFACTX
                ELSE
                  XSSCT(JGRC,IGRC,IL)=
     >             DRSCT(JGRC,IGRC,IL,IMR)*DFACTJ
                ENDIF
 122          CONTINUE
 121        CONTINUE
          ENDIF
 120    CONTINUE
C----
C  PROCESS CHI IF PRESENT
C----
        IXSR=5
        JXSR=3
        IF(INDPRO(IXSR).GT.0) THEN
          DTEMP=0.0D0
          DO 130 IGRC=1,NGCOND
            DTEMP=DTEMP+DRATE(IGRC,JXSR,IMR)
 130      CONTINUE
          IF(DTEMP.NE.0.0D0) DTEMP=1.0D0/DTEMP
          DO 131 IGRC=1,NGCOND
            XSREC(IGRC,IXSR)=DRATE(IGRC,IXSR,IMR)*DTEMP
 131      CONTINUE
        ENDIF
C----
C  PROCESS NU IF PRESENT
C----
        IXSR=6
        JXSR=4
        IF(INDPRO(IXSR).GT.0) THEN
          DO 140 IGRC=1,NGCOND
            IF(DRATE(IGRC,JXSR,IMR).NE.0.0D0) THEN
              XSREC(IGRC,IXSR)=DRATE(IGRC,IXSR,IMR)
     >                        /DRATE(IGRC,JXSR,IMR)
            ELSE
              XSREC(IGRC,IXSR)=0.0
            ENDIF
 140      CONTINUE
        ENDIF
C----
C  PROCESS STRD
C----
        ISTRD=0
        DO 150 IXSR=NDPROC-3,NDPROC
          ISTRD=ISTRD+1
          IF(INDPRO(IXSR).GT.0) THEN
            DO 151 IGRC=1,NGCOND
              XSREC(IGRC,IXSR)=(XSREC(IGRC,1)-XSREC(IGRC,NDPROC+2))
     >                      *DXSTRD(ISTRD,IGRC)
 151        CONTINUE
          ENDIF
 150    CONTINUE
      ELSE
        DO 200 IXSR=1,4
          IF(INDPRO(IXSR).GT.0) THEN
            DO 201 IGRC=1,NGCOND
              XSREC(IGRC,IXSR)=(DRATE(IGRC,IXSR,IMR)*DFLDNC(IGRC,1)
     >                        -XSREC(IGRC,IXSR))*PERT
 201        CONTINUE
          ENDIF
 200    CONTINUE
C----
C  PROCESS ADDITIONAL XS IF PRESENT
C----
        DO 210 IXSR=7,NDPROC-4
          IF(INDPRO(IXSR).GT.0) THEN
            DO 211 IGRC=1,NGCOND
              XSREC(IGRC,IXSR)=(DRATE(IGRC,IXSR,IMR)*DFLDNC(IGRC,1)
     >                        -XSREC(IGRC,IXSR))*PERT
 211        CONTINUE
          ENDIF
 210    CONTINUE
C----
C  PROCESS SCATTERING XS IF PRESENT
C----
        IL=0
        DO 220 IXSR=NDPROC+MINLEG+1,NDPROC+MAXLEG+1
          IL=IL+1
          IF(INDPRO(IXSR).GT.0) THEN
            DO 221 IGRC=1,NGCOND
              IF(ILEAKS.EQ.3.AND.IL.EQ.2) THEN
                DFACTJ=DFLDNC(IGRC,2)
                DFACTX=DRATE(IGRC,1,IMR)*DFLDNC(IGRC,1)+DFLDNC(IGRC,3)
              ELSE
                DFACTJ=DFLDNC(IGRC,1)
                DFACTX=0.0D0
              ENDIF
              XSREC(IGRC,IXSR)=(DRATE(IGRC,IXSR,IMR)*DFACTJ+DFACTX
     >                        -XSREC(IGRC,IXSR))*PERT
              DO 222 JGRC=1,NGCOND
                IF(IGRC.EQ.JGRC) THEN
                  XSSCT(JGRC,IGRC,IL)=
     >              (DRSCT(JGRC,IGRC,IL,IMR)*DFACTJ+DFACTX
     >              -XSSCT(JGRC,IGRC,IL))*PERT
                ELSE
                  XSSCT(JGRC,IGRC,IL)=
     >             (DRSCT(JGRC,IGRC,IL,IMR)*DFACTJ
     >              -XSSCT(JGRC,IGRC,IL))*PERT
                ENDIF
 222          CONTINUE
 221        CONTINUE
          ENDIF
 220    CONTINUE
C----
C  PROCESS CHI IF PRESENT
C----
        IXSR=5
        JXSR=3
        IF(INDPRO(IXSR).GT.0) THEN
          DTEMP=0.0D0
          DO 230 IGRC=1,NGCOND
            DTEMP=DTEMP+DRATE(IGRC,JXSR,IMR)
 230      CONTINUE
          IF(DTEMP.NE.0.0D0) DTEMP=1.0D0/DTEMP
          DO 231 IGRC=1,NGCOND
            XSREC(IGRC,IXSR)=(DRATE(IGRC,IXSR,IMR)*DTEMP
     >                      -XSREC(IGRC,IXSR))*PERT
 231      CONTINUE
        ENDIF
C----
C  PROCESS NU IF PRESENT
C----
        IXSR=6
        JXSR=4
        IF(INDPRO(IXSR).GT.0) THEN
          DO 240 IGRC=1,NGCOND
            IF(DRATE(IGRC,JXSR,IMR).NE.0.0D0) THEN
              XSREC(IGRC,IXSR)=(DRATE(IGRC,IXSR,IMR)
     >                        /DRATE(IGRC,JXSR,IMR)
     >                       -XSREC(IGRC,IXSR))*PERT
            ELSE
              XSREC(IGRC,IXSR)=0.0
            ENDIF
 240      CONTINUE
        ENDIF
C----
C  PROCESS STRD
C----
        IF(MAXLEG.GE.1) THEN
          ISTRD=0
          DO 250 IXSR=NDPROC-3,NDPROC
            ISTRD=ISTRD+1
            IF(INDPRO(IXSR).GT.0) THEN
              DO 251 IGRC=1,NGCOND
                XSREC(IGRC,IXSR)=(XSREC(IGRC,1)-XSREC(IGRC,22))
     >                          *DXSTRD(ISTRD,IGRC)
 251          CONTINUE
            ENDIF
 250      CONTINUE
        ELSE
          ISTRD=0
          DO 252 IXSR=NDPROC-3,NDPROC
            ISTRD=ISTRD+1
            IF(INDPRO(IXSR).GT.0) THEN
              DO 253 IGRC=1,NGCOND
                XSREC(IGRC,IXSR)=XSREC(IGRC,1)
     >                          *DXSTRD(ISTRD,IGRC)
 253          CONTINUE
            ENDIF
 252      CONTINUE
        ENDIF
      ENDIF
      RETURN
      END
