*DECK EDIPRR
      SUBROUTINE EDIPRR(IPRINT,NGCOND,NMERGE,NL    ,NIFISS,NEDMAC,
     >                  NTAUXT,ILEAKS,NTENER,B2    ,ENERGY,SPH   ,
     >                  VOLMER,RATECM,SCATTD)
C
C---------------------------  EDIPRR  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDIPRR
C      USE      : PRINT REACTION RATES
C      MODIFIED : 97-02-06 (G.M.)
C                 94-07-21 (G.M. + I.P.)
C                 91-12-04 (G.M.)
C                 91-04-03 (G.M.)
C                 89-08-24 (G.M.)
C                 88-10-25 (A.H.)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C      IPRINT   : PRINT LEVEL                        I
C                 = 0 NO PRINT
C                 = 1 PRINT FLUXES
C                 = 2 1+PRINT REACTION RATES
C                 = 3 2+PRINT HOMOGENIZED X-S
C      NGCOND   : NUMBER OF GROUPS CONDENSED         I
C      NMERGE   : NUMBER OF REGIONS MERGED           I
C      NL       : NUMBER OF LEGENDRE ORDERS          I
C      NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C      NEDMAC   : NUMBER OF EXTRA EDIT VECTORS       I
C      NTAUXT   : NUMBER OF REACTION RATE EDITS      I
C      ILEAKS   : LEAK CALCULATION                   I
C                 = 0 NO LEAKAGE
C                 = 1 LEAKAGE BY SURFACE (ALBEDO>0.0)
C                 = 2 HOMOGENEOUS LEAKAGE COEFFICIENTS
C                 = 3 DIRECTIONAL LEAKAGE COEFFICIENTS
C      NTENER   : NUMBER OF ENERGY GROUPS LIMITS     I
C      B2       : SQUARE BUCKLING                    R(4)
C                 FOR ILEAKS = 2 B2(4) IS HOMOGENEOUS
C                 FOR ILEAKS = 3 B2(1),B2(2),B2(3)
C                 ARE DIRECTIONAL HETEROGENEOUS
C                 AND B2(4) IS HOMOGENEOUS
C      ENERGY   : ENERGY GROUP LIMITS                R(NGCOND+1)
C      SPH      : SPH HOMOGENIZATION FACTORS         R(NMERGE,NGCOND)
C      VOLMER   : VOLUME OF REGION MERGED            D(NMERGE)
C      RATECM   : AVERAGED REGION/GROUP X-S          D(0:NMERGE,
C                 = RATECM(*,1) = FLUXES             NGCOND,NTAUXT)
C                 = RATECM(*,2) = TOTAL
C                 = RATECM(*,3) = ABS
C                 = RATECM(*,4) = FISSIONS
C                 = RATECM(*,5) = PRODUCTIONS
C                 = RATECM(*,6) = TOTAL LEAKAGES
C                 = RATECM(*,7) = TOTAL OUT OF GROUP SCATTERING
C                 = RATECM(*,8) = DIAGONAL SCATTERING X-S
C                 = RATECM(*,9) = CHI
C                 = RATECM(*,10) = WIMS TYPE TRANSPORT CORRECTION
C                 = RATECM(*,11) = X-DIRECTED LEAKAGES
C                 = RATECM(*,12) = Y-DIRECTED LEAKAGES
C                 = RATECM(*,13) = Z-DIRECTED LEAKAGES
C                 = RATECM(*,13+JJ) NUSIGF FOR JJ
C                 = RATECM(*,13+NIFISS+JJ) CHI FOR JJ
C                 = RATECM(*,13+2*NIFISS+JJ) NFTOT FOR JJ
C                 = RATECM(*,13+3*NIFISS+IL) SIGS FOR IL
C                 = RATECM(*,13+3*NIFISS+NL+IE) EDIT IE
C                 = RATECM(*,NTAUXT)= 1/V MERGE CONDENSED
C      SCATTD   : DOUBLE PRECISION SCATTERING RATES  D(NMERGE,
C                                                      NGCOND,NGCOND,NL)
C
C---------------------------   EDIPRR  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IOUT
      CHARACTER        NAMSBR*6,APG*3
      PARAMETER       (IOUT=6,NAMSBR='EDIPRR',APG=' > ')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPRINT,NGCOND,NMERGE,NL,NIFISS,NEDMAC,NTAUXT,
     >                 ILEAKS,NTENER
      REAL             B2(4),ENERGY(NGCOND+1),SPH(NMERGE,NGCOND)
      DOUBLE PRECISION VOLMER(NMERGE),RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                 SCATTD(NMERGE,NGCOND,NGCOND,NL)
C----
C  LOCAL PARAMETERS
C----
      INTEGER          IMRG,IGRC,JGRC,NSPH
      DOUBLE PRECISION SCATTN,SCATTS
C----
C  PRINT REACTION RATES
C----
      WRITE(IOUT,6000)
      WRITE(IOUT,6001) (IMRG,VOLMER(IMRG),IMRG=1,NMERGE)
      IF(NTENER .GT. 0) THEN
        WRITE(IOUT,6002) (ENERGY(IGRC),APG,IGRC,APG,IGRC=1,NGCOND),
     >                        ENERGY(NGCOND+1)
      ENDIF
      NSPH=0
      DO 140 IGRC=1,NGCOND
        DO 141 IMRG=1,NMERGE
          IF(SPH(IMRG,IGRC) .NE. 1.0) THEN
            NSPH=1
            GO TO 145
          ENDIF
 141    CONTINUE
 140  CONTINUE
 145  CONTINUE
      IF(NSPH .EQ. 0) THEN
        WRITE(IOUT,6100)
      ELSE
        WRITE(IOUT,6101)
        DO 150 IGRC=1,NGCOND
          WRITE(IOUT,6102) IGRC
          WRITE(IOUT,6103) (SPH(IMRG,IGRC),IMRG=1,NMERGE)
 150    CONTINUE
      ENDIF
      WRITE(IOUT,6003)
      DO 100 IGRC=1,NGCOND
        IF(IPRINT .EQ. 1) THEN
          WRITE(IOUT,6010) IGRC
          WRITE(IOUT,6012) (RATECM(IMRG,IGRC,1),IMRG=1,NMERGE)
          WRITE(IOUT,6011)
          WRITE(IOUT,6012)
     >      (RATECM(IMRG,IGRC,1)/VOLMER(IMRG),IMRG=1,NMERGE)
        ELSE
          IF(ILEAKS .EQ. 2) THEN
            WRITE(IOUT,6013) IGRC
          ELSE
            WRITE(IOUT,6014) IGRC
          ENDIF
          DO 110 IMRG=1,NMERGE
            SCATTN=0.0D0
            DO 120 JGRC=1,NGCOND
              IF(JGRC .NE. IGRC) SCATTN=SCATTN+SCATTD(IMRG,JGRC,IGRC,1)
 120        CONTINUE
            SCATTS=SCATTN+SCATTD(IMRG,IGRC,IGRC,1)
            IF(ILEAKS .EQ. 2) THEN
              WRITE(IOUT,6015) IMRG,RATECM(IMRG,IGRC,1)/VOLMER(IMRG),
     >          RATECM(IMRG,IGRC,1),RATECM(IMRG,IGRC,2),
     >          RATECM(IMRG,IGRC,10),RATECM(IMRG,IGRC,3)-SCATTS,
     >          RATECM(IMRG,IGRC,4),RATECM(IMRG,IGRC,6)*B2(4),
     >          RATECM(IMRG,IGRC,5),SCATTD(IMRG,IGRC,IGRC,1),SCATTN
            ELSE
              WRITE(IOUT,6015) IMRG,RATECM(IMRG,IGRC,1)/VOLMER(IMRG),
     >          RATECM(IMRG,IGRC,1),RATECM(IMRG,IGRC,2),
     >          RATECM(IMRG,IGRC,10),RATECM(IMRG,IGRC,3)-SCATTS,
     >          RATECM(IMRG,IGRC,4),RATECM(IMRG,IGRC,5),
     >          SCATTD(IMRG,IGRC,IGRC,1),SCATTN
            ENDIF
 110      CONTINUE
          IF(ILEAKS .EQ. 3) THEN
            WRITE(IOUT,6022)
            DO 130 IMRG=1,NMERGE
              WRITE(IOUT,6023) IMRG,RATECM(IMRG,IGRC,11)*B2(1)
     >        +RATECM(IMRG,IGRC,12)*B2(2)+RATECM(IMRG,IGRC,13)*B2(3),
     >        RATECM(IMRG,IGRC,11)*B2(1),RATECM(IMRG,IGRC,12)*B2(2),
     >        RATECM(IMRG,IGRC,13)*B2(3),RATECM(IMRG,IGRC,6)*B2(4)
 130        CONTINUE
          ENDIF
        ENDIF
 100  CONTINUE
      RETURN
C----
C  FORMAT
C----
 6000 FORMAT(////5(5X,'REGION',6X,'VOLUME  '))
 6001 FORMAT(1P,5(5X,I4,4X,E12.5))
 6002 FORMAT(/' E N E R G Y   L I M I T S   (EV)'/1P,
     >6(2X,E10.4,A3,I3,A3))
 6003 FORMAT(/' F L U X E S   A N D    R E A C T I O N    R A T E S'/
     >1X,51(1H-))
 6010 FORMAT(/' G R O U P   :',I4/' REGION INTEGRATED FLUX')
 6011 FORMAT(' AVERAGED REGIONAL FLUX')
 6012 FORMAT(1P,7(3X,E15.7))
 6013 FORMAT(/' G R O U P   : ',I3/
     >' REGION     AVERAGE  INTEGRATED   COLLISION   TRANSPORT',
     >'  ABSORPTION  NU*FISSION     LEAKAGE  PRODUCTION',
     >'      SCATTERING RATE   '/
     >'               FLUX        FLUX        RATE  CORRECTION',
     >'        RATE        RATE        RATE        RATE ',
     >'   IN GROUP  FROM GROUP')
 6014 FORMAT(/' G R O U P   : ',I3/
     >' REGION     AVERAGE  INTEGRATED   COLLISION   TRANSPORT',
     >'  ABSORPTION  NU*FISSION  PRODUCTION      SCATTERING RATE   '/
     >'               FLUX        FLUX        RATE  CORRECTION',
     >'        RATE        RATE        RATE    IN GROUP  FROM GROUP')
 6015 FORMAT(1X,I6,1P,10E12.4)
 6022 FORMAT(/' REGION  TOTAL LEAKAGE     X-LEAKAGE',
     >        '     Y-LEAKAGE     Z-LEAKAGE   HOMOGENEOUS'/
     >        '              RATE           RATE   ',
     >        '       RATE          RATE     LEAKAGE RATE')
 6023 FORMAT(1X,I6,1X,1P,5E14.5)
 6100 FORMAT(/' SPH CORRECTION FACTOR ALL EQUAL TO 1.0')
 6101 FORMAT(/' SPH CORRECTION FACTOR')
 6102 FORMAT(' IN GROUP :',I4)
 6103 FORMAT(1P,7(3X,E15.7))
      END
