*DECK EDISCT
      SUBROUTINE EDISCT(IPMACR,IPRINT,NGROUP,NGCOND,NBMIX ,NREGIO,
     >                  NMERGE,NL    ,NIFISS,NTAUXT,ILEAKS,NWGTH ,
     >                  MATCOD,IGCOND,IMERGE,SPH   ,RATECM,FLXINT,
     >                  SCATTD,B2    ,DIRC  ,
     >                  SIGMA,INGSCT ,IFGSCT,IPOSCT,XSCAT ,DLBDG)
C
C---------------------------  EDISCT  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : EDISCT
C      USE      : EVALUATE SCATTERING REACTION RATES
C      MODIFIED : 99-02-19 (G.M.)
C                 -> COMPUTE DIRECTIONAL WEIGHTING FACTOR TO
C                    CORRECT BUCKLING WEIGHTED ANISOTROPIC SCATTERING
C                 98-03-09 (G.M. + E.D.)
C                 -> CONSISTENT B1 HOMOGENIZATION FOR BUCKLING
C                    WEIGHTED ANISOTROPIC SCATTERING
C      AUTHOR   : G.MARLEAU + E. DEBOS
C
C  2- ROUTINE PARAMETERS:
C      IPMACR   : POINTER TO THE MACROLIB            I
C      IPRINT   : PRINT LEVEL                        I
C                 = 0 NO PRINT
C                 = 1 PRINT FLUXES
C                 = 2 1+PRINT REACTION RATES
C                 = 3 2+PRINT HOMOGENIZED X-S
C      NGROUP   : NUMBER OF GROUPS                   I
C      NGCOND   : NUMBER OF GROUPS CONDENSED         I
C      NBMIX    : NUMBER OF MIXTURES                 I
C      NREGIO   : NUMBER OF REGIONS                  I
C      NMERGE   : NUMBER OF REGIONS MERGED           I
C      NL       : NUMBER OF LEGENDRE ORDERS          I
C      NIFISS   : NUMBER OF FISSILE ISOTOPES         I
C      NTAUXT   : NUMBER OF REACTION RATE EDITS      I
C      MATCOD   : MATERIAL PER REGION                I(NREGIO)
C      IGCOND   : LIMIT CONDENSED GROUPS             I(NGCOND)
C      IMERGE   : INDEX OF MERGED REGION             I(NREGIO)
C      SPH      : SPH HOMOGENIZATION FACTORS         R(NMERGE,NGCOND)
C      B2       : SQUARE BUCKLING                    R(4)
C                 FOR ILEAKS = 2 B2(4) IS HOMOGENEOUS
C                 FOR ILEAKS = 3 B2(1),B2(2),B2(3)
C                 ARE DIRECTIONAL HETEROGENEOUS
C                 AND B2(4) IS HOMOGENEOUS
C      RATECM   : AVERAGED REGION/GROUP X-S          D(0:NMERGE,
C                 = RATECM(*,*,1) = FLUXES             NGCOND,NTAUXT)
C                 = RATECM(*,*,2) = TOTAL
C                 = RATECM(*,*,3) = ABS
C                 = RATECM(*,*,4) = FISSIONS
C                 = RATECM(*,*,5) = PRODUCTIONS
C                 = RATECM(*,*,6) = TOTAL LEAKAGES
C                 = RATECM(*,*,7) = TOTAL OUT OF GROUP SCATTERING
C                 = RATECM(*,*,8) = DIAGONAL SCATTERING X-S
C                 = RATECM(*,*,9) = CHI
C                 = RATECM(*,*,10) = WIMS TYPE TRANSPORT CORRECTION
C                 = RATECM(*,*,11) = X-DIRECTED LEAKAGES
C                 = RATECM(*,*,12) = Y-DIRECTED LEAKAGES
C                 = RATECM(*,*,13) = Z-DIRECTED LEAKAGES
C                 = RATECM(*,*,13+JJ) NUSIGF FOR JJ
C                 = RATECM(*,*,13+NIFISS+JJ) CHI FOR JJ
C                 = RATECM(*,*,13+2*NIFISS+JJ) NFTOT FOR JJ
C                 = RATECM(*,*,13+3*NIFISS+IL) SIGS FOR IL
C                 = RATECM(*,*,13+3*NIFISS+NL+IE) EDIT IE
C                 = RATECM(*,*,NTAUXT)= 1/V MERGE CONDENSED
C      FLXINT   : INTEGRATED FLUX                    R(NREGIO,NGROUP,4)
C      SCATTD   : DOUBLE PRECISION SCATTERING RATES  D(NMERGE,
C                                                      NGCOND,NGCOND,NL)
C      DIRC     : ADDED 99-02-19                     D(0:NMERGE,
C                 DIRECTIONAL CORRECTION               NGCOND,3)
C    WORK
C      SIGMA    : DUMMY CROSS SECTION VECTOR         R(0:NBMIX)
C      IFGSCT   : FIRST SCATTERING GROUP             I(NBMIX)
C      INGSCT   : NUMBER OF SCATTERING GROUP         I(NBMIX)
C      IPOSCT   : SCATTERING POSITION                I(NBMIX)
C      XSCAT    : COMPRESS SCATTERING MATRIX         R(NBMIX*NGROUP)
C      DLBDG    : GROUP DEPENDENT LAMBDA             D(NGROUP)
C
C---------------------------   EDISCT  --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IUNOUT,ILCMUP,ILCMDN,ILCMLN,ITYLCM
      PARAMETER       (IUNOUT=6,ILCMUP=1,ILCMDN=2)
      INTEGER          IPMACR,IPRINT,NGROUP,NGCOND,NBMIX,NREGIO,NMERGE,
     >                 NL,NIFISS,NTAUXT,ILEAKS,NWGTH
      INTEGER          MATCOD(NREGIO),IGCOND(NGCOND),IMERGE(NREGIO),
     >                 IFGSCT(NBMIX),INGSCT(NBMIX),IPOSCT(NBMIX)
      REAL             FLXINT(NREGIO,NGROUP,4),SIGMA(0:NBMIX),
     >                 XSCAT(NBMIX*NGROUP),SPH(NMERGE,NGCOND),B2(4)
      DOUBLE PRECISION SCATTD(NMERGE,NGCOND,NGCOND,NL),
     >                 RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                 DIRC(0:NMERGE,NGCOND,3),DLBDG(NGROUP)
C----
C  VARIABLES LOCALES
C----
      INTEGER          IGR,IGRC,JGR,JGRC,ILSC,IGRDEB,IGRFIN,IL,IDIR,IKK,
     >                 IREGIO,MATNUM,NGSCAT,IGSCAT,IPOSIT,JGRFIN,JGRDEB,
     >                 J2,J1,IPO
      CHARACTER        CGRPNM*12,CM*2
      DOUBLE PRECISION DCOR,DTOTL,DFLUX
      ILSC=13+3*NIFISS
C----
C  SCATTERING NEUTRONS
C  99-02-19
C  COMPUTE DLBDG GROUP DEPENDENT CORRECTION FACTOR
C----
      CALL XDDSET(DLBDG,NGROUP,0.0D0)
      IF(NWGTH .GT. 1) THEN
        DO 100 IGR=1,NGROUP
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMGET(IPMACR,'TOTAL',SIGMA(1))
          DFLUX=0.0D0
          DTOTL=0.0D0
          DO 101 IREGIO=1,NREGIO
            MATNUM=MATCOD(IREGIO)
            IKK=IMERGE(IREGIO)
            DFLUX=DFLUX+FLXINT(IREGIO,IGR,1)
            DTOTL=DTOTL+FLXINT(IREGIO,IGR,1)
     >                 *SIGMA(MATNUM)
 101      CONTINUE
          IF(DFLUX .NE. 0.0D0) THEN
            DTOTL=DTOTL/DFLUX
            DLBDG(IGR)=DTOTL
            CALL EDILBD(1,DLBDG(IGR),B2(4))
            DLBDG(IGR)=DTOTL*(1.0D0-1.0D0/DLBDG(IGR))
          ENDIF
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 100    CONTINUE
      ENDIF
C----
C  SCATTERING NEUTRONS
C  99-02-19
C  INITIALIZE DOUBLE PRECISION VECTOR "DIRC" FOR DIRECTIONAL
C  ANISOTROPIC SCATTERING CORRECTION
C----
      CALL XDDSET(DIRC,(NMERGE+1)*NGCOND*3,0.0D0)
      CALL XDDSET(SCATTD,NMERGE*NGCOND*NGCOND*NL,0.0D0)
      IGRFIN=0
      DO 110 IGRC=1,NGCOND
        IGRDEB=IGRFIN+1
        IGRFIN=IGCOND(IGRC)
        DO 120 IGR=IGRDEB,IGRFIN
          WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGROUP
          CALL LCMSIX(IPMACR,CGRPNM,ILCMUP)
          CALL LCMGET(IPMACR,'TOTAL',SIGMA(1))
          DO 130 IL=1,NL
            WRITE (CM,'(I2)') IL-1
            CALL LCMLEN(IPMACR,'NJJD'//CM,ILCMLN,ITYLCM)
            IF(ILCMLN.GT.0) THEN
              CALL LCMGET(IPMACR,'NJJD'//CM,INGSCT)
              CALL LCMGET(IPMACR,'IJJD'//CM,IFGSCT)
              CALL LCMGET(IPMACR,'IPOD'//CM,IPOSCT)
              CALL LCMGET(IPMACR,'SCAD'//CM,XSCAT)
              IF( (IL.EQ.2).AND.(NWGTH.GE.1)) THEN
                DO 140 IDIR=1,3
                  DO 141 IREGIO=1,NREGIO
                    MATNUM=MATCOD(IREGIO)
                    IKK=IMERGE(IREGIO)
                    IF((IKK.GT.0).AND.(MATNUM.GT.0)) THEN
                      NGSCAT=INGSCT(MATNUM)
                      IGSCAT=IFGSCT(MATNUM)
                      IPOSIT=IPOSCT(MATNUM)
                      JGRFIN=0
                      DO 142 JGRC=1,NGCOND
                        JGRDEB=JGRFIN+1
                        JGRFIN=IGCOND(JGRC)
                        J2=MIN(JGRFIN,IGSCAT)
                        J1=MAX(JGRDEB,IGSCAT-NGSCAT+1)
                        IPO=IPOSIT+IGSCAT-J2
                        DO 143 JGR=J2,J1,-1
C----
C  99-02-19
C  CONTRIBUTION FROM NON-VOID REGIONS
C  COMPUTE DIRC WHICH IS TOTAL SCATTERING FOR EACH DIRECTION
C  CHANGE EXPRESSIONS FOR RATECM AND SCATT TO INCLUDE B2(K)
C  WHICH IS NO LONGUER IN FLXINT.
C  ADD LAMBDA CORRECTION ( CROSS SECTION AND FLUX )
C----
                          IF(NWGTH.GT.1) THEN
                            IF(JGR.EQ.IGR) THEN
                              DCOR=( XSCAT(IPO)-SIGMA(MATNUM)
     >                              + DLBDG(JGR) )*SPH(IKK,JGRC)
     >                             *FLXINT(IREGIO,JGR,1+IDIR)
                            ELSE
                              DCOR=XSCAT(IPO)
     >                            *FLXINT(IREGIO,JGR,1+IDIR)
     >                            *SPH(IKK,JGRC)
                            ENDIF
                          ELSE
                            DCOR=XSCAT(IPO)
     >                          *FLXINT(IREGIO,JGR,1+IDIR)
                          ENDIF
                          DIRC(IKK,IGRC,IDIR)=DCOR
     >                        +DIRC(IKK,IGRC,IDIR)
                          IF(ILEAKS.EQ.3) THEN
                            DCOR=DCOR*B2(IDIR)
                          ELSE
                            DCOR=DCOR*B2(4)
                          ENDIF
                          SCATTD(IKK,IGRC,JGRC,IL)=DCOR
     >                       +SCATTD(IKK,IGRC,JGRC,IL)
                          RATECM(IKK,JGRC,ILSC+IL)=DCOR
     >                       +RATECM(IKK,JGRC,ILSC+IL)
                          IPO=IPO+1
 143                    CONTINUE
 142                  CONTINUE
                    ELSE IF((IKK.GT.0).AND.(MATNUM.EQ.0)) THEN
                      JGRFIN=0
                      DO 144 JGRC=1,NGCOND
                        JGRDEB=JGRFIN+1
                        JGRFIN=IGCOND(JGRC)
                        J2=MIN(JGRFIN,IGR)
                        J1=MAX(JGRDEB,IGR)
                        IF(J1.LE.J2) THEN
C----
C  99-02-19
C  CONTRIBUTION FROM VOID REGIONS
C  ONLY DIAGONAL ELEMENT IN SCATTERING MATRIX
C  CONTRIBUTION IN LAMBDA FROM FLUX
C----
                          IF(NWGTH.GT.1) THEN
                            DCOR=DLBDG(IGR)*SPH(IKK,JGRC)
     >                          *FLXINT(IREGIO,IGR,1+IDIR)
                            DIRC(IKK,IGRC,IDIR)=DCOR
     >                        +DIRC(IKK,IGRC,IDIR)
                            IF(ILEAKS.EQ.3) THEN
                              DCOR=DCOR*B2(IDIR)
                            ELSE
                              DCOR=DCOR*B2(4)
                            ENDIF
                            SCATTD(IKK,IGRC,JGRC,IL)=DCOR
     >                       +SCATTD(IKK,IGRC,JGRC,IL)
                            RATECM(IKK,JGRC,ILSC+IL)=DCOR
     >                       +RATECM(IKK,JGRC,ILSC+IL)
                          ENDIF
                        ENDIF
 144                  CONTINUE
                    ENDIF
 141              CONTINUE
 140            CONTINUE
C----
C  DANS SCATTD ON N'A PAS LE TAUX DE REACTION DE DIFFUSION
C  SCATTD(1,H) ATTENDU, MAIS SCATTD(1,H) - SIGMA(H,H) (AVEC
C  SIGMA(H,H)=RATECM(*,*,2)/RATECM(*,*,1) LA SECTION
C  EFFICACE TOTALE HOMOGENEISEE.)
C  POUR L'OBTENTION DU SCATTD(1,H) ATTENDU, VOIR ROUTINE
C  EDIPXS
C----
              ELSE
                DO 150 IREGIO=1,NREGIO
                  MATNUM=MATCOD(IREGIO)
                  IKK=IMERGE(IREGIO)
                  IF((IKK.GT.0).AND.(MATNUM.GT.0)) THEN
                    NGSCAT=INGSCT(MATNUM)
                    IGSCAT=IFGSCT(MATNUM)
                    IPOSIT=IPOSCT(MATNUM)
                    JGRFIN=0
                    DO 151 JGRC=1,NGCOND
                      JGRDEB=JGRFIN+1
                      JGRFIN=IGCOND(JGRC)
                      J2=MIN(JGRFIN,IGSCAT)
                      J1=MAX(JGRDEB,IGSCAT-NGSCAT+1)
                      IPO=IPOSIT+IGSCAT-J2
                      DO 152 JGR=J2,J1,-1
                        DCOR=XSCAT(IPO)*FLXINT(IREGIO,JGR,1)
                        SCATTD(IKK,IGRC,JGRC,IL)=DCOR
     >                      +SCATTD(IKK,IGRC,JGRC,IL)
                        RATECM(IKK,JGRC,ILSC+IL)=DCOR
     >                      +RATECM(IKK,JGRC,ILSC+IL)
                        IPO=IPO+1
 152                  CONTINUE
 151                CONTINUE
                  ENDIF
 150            CONTINUE
              ENDIF
            ENDIF
 130      CONTINUE
          CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
 120    CONTINUE
 110  CONTINUE
      RETURN
      END
