*DECK FLUFUI
      FUNCTION FLUFUI(IPSYS ,NBMIX ,NREGIO,NUNKNO,MATCOD,VOLUME,
     >                KEYFLX,FUNKNO,SUNKNO,SIGT0 ,SIGS0 )
C
C---------------------------  FLUFUI  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     :  FLUFUI
C       USE      :  COMPUTE THE DB2 VALUE CORRESPONDING TO THE
C                   ACTUAL LEAKAGE
C       MODIFIED : MODIFY DEFINITION OF ITYPEC
C                  2000/02/08  (G. MARLEAU)
C                  1988/10/13  (A. HEBERT)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C         IPSYS   : POINTER TO THE PIJ                 I
C         NBMIX   : NUMBER OF MIXTURES                 I
C         NREGIO  : NUMBER OF REGIONS CONSIDERED       I
C         NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.   I
C         MATCOD  : MATERIAL CODE IN REGION            I(NREGIO)
C         VOLUME  : VOLUME OF REGION                   R(NREGIO)
C         KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM    I(NREGIO)
C         FUNKNO  : UNKNOWN VECTOR                     R(NUNKNO)
C         SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN       R(NUNKNO)
C         SIGT0   : TOTAL CROSS SECTION VECTOR         R(0:NBMIX)
C                   SIGT0(0)=0.0 IS FOR VOID
C         SIGS0   : WITHIN-GROUP SCATTERING            R(0:NBMIX)
C                   SIGS0(0)=0.0 IS FOR VOID
C    OUTPUT:
C         FLUFUI  : LEAKAGE DB2 PARAMETER              R
C
C--------------------------    FLUFUI    ------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='FLUFUI')
C----
C  ROUTINE PARAMETERS
C----
      REAL        FLUFUI
      INTEGER     IPSYS,NBMIX,NREGIO,NUNKNO
      INTEGER     MATCOD(NREGIO),KEYFLX(NREGIO)
      REAL        VOLUME(NREGIO),FUNKNO(NUNKNO),
     >            SUNKNO(NUNKNO),SIGT0(0:NBMIX),SIGS0(0:NBMIX)
C----
C  LOCAL PARAMETERS
C----
      INTEGER     ILCMLN,ILCMTY,IREG,IBM,IFPOS
      DOUBLE PRECISION ZNUM,ZDEN,SSS
C----
C  READ THE TRANSPORT CORRECTED CROSS SECTIONS.
C----
      CALL LCMLEN(IPSYS,'DRAGON-TXSC',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. NBMIX+1) THEN
        CALL LCMGET(IPSYS,'DRAGON-TXSC',SIGT0)
      ELSE
        CALL LCMLIB(IPSYS)
        CALL XABORT(NAMSBR//
     >  ': READ ERROR LCM RECORD= DRAGON-TXSC')
      ENDIF
      CALL LCMLEN(IPSYS,'DRAGON-S0XSC',ILCMLN,ILCMTY)
      IF(ILCMLN .EQ. NBMIX+1) THEN
        CALL LCMGET(IPSYS,'DRAGON-S0XSC',SIGS0)
      ELSE
        CALL LCMLIB(IPSYS)
        CALL XABORT(NAMSBR//
     >  ': READ ERROR LCM RECORD= DRAGON-S0XSC')
      ENDIF
C
C----- COMPUTE DB2
C
      ZNUM=0.0D0
      ZDEN=0.0D0
      DO 100 IREG=1,NREGIO
        IBM=MATCOD(IREG)
        IFPOS=KEYFLX(IREG)
        SSS=DBLE((SIGT0(IBM)-SIGS0(IBM))*FUNKNO(IFPOS))
        ZNUM=ZNUM+DBLE(VOLUME(IREG))*(DBLE(SUNKNO(IFPOS))-SSS)
        ZDEN=ZDEN+DBLE(VOLUME(IREG))*DBLE(FUNKNO(IFPOS))
 100  CONTINUE
      FLUFUI=0.0
      IF (ZDEN .GT. 0.0D0) FLUFUI=REAL(ZNUM/ZDEN)
      RETURN
      END
