*DECK FLUSUR
      SUBROUTINE FLUSUR(IPFLUX,IPSYS ,IPMACR,IPRINT,NGROUP,NBMIX ,
     >                  NREGIO,NIFISS,ITYPEC,ILEAK ,NUNKNO,NUNSUR,NSUR,
     >                  NVOL,MATCOD,KEYFLX,FUNKNO,EIGENK,B2    ,SUNKNO,
     >                  SIGF  ,XSA   ,MATALB,ALBEDO,BCRFTR,QTOTL,DIFF,
     >                  IJJ   ,NJJ   ,IPOS,KEYMRG,COND,
     >                  XSCAT ,FUNPOS,FUNSUR,PSVFUI,ALBD)
C
C---------------------------  FLUSUR  ---------------------------------
C
C   1- PROGRAMME STATISTICS:
C       NAME     : FLUSUR
C       USE      : COMPUTE EXTERNAL SURFACE FLUX 
C       MODIFIED : MODIFY DEFINITION OF ITYPEC
C                  2003/03/212 (P. Dufour)
C
C   2- ROUTINE PARAMETERS:
C    INPUT:
C      IPFLUX  : POINTER TO THE SOLUTION                   I
C      IPSYS   : POINTER TO THE PIJ                        I
C      IPMACR  : POINTER TO THE MACROLIB                   I
C      IPRINT  : PRINT SELECTION FOR FLUX MODULES          I
C                = 0/1/2/3 NO PRINT/SHORT PRINT/LONG
C      NGROUP  : NUMBER OF GROUPS.                         I
C      NBMIX   : NUMBER OF MIXTURES                        I
C      NREGIO  : NUMBER OF REGIONS AFTER MERGE             I
C      NIFISS  : NUMBER OF FISSILE ISOTOPES PER MIXTURE    I
C      ITYPEC  : TYPE OF FLUX EVALUATION                   I
C                = 0  SKIP THE FLUX CALCULATION
C                = 1  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE
C                = 2  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     FIXED DB2 BUCKLING
C                = 3  DIRECT PROBLEM FISSION SOURCES/
C                     K EFFECTIVE CONVERGENCE/
C                     DB2 BUCKLING EVALUATION
C                = 4  DIRECT PROBLEM FISSION SOURCES/
C                     DB2 BUCKLING CONVERGENCE
C                = 5  DIRECT PROBLEM DB2 SOURCES
C                = 6  DIRECT PROBLEM FIXED SOURCES
C      ILEAK   : TYPE OF LEAKAGE                           I
C      NUNKNO  : NUMBER OF UNKNOWN IN THE SYSTEM.          I
C      NUNSUR  : NUMBER OF EXTERNAL SURFACES AFTER MERGE   I 
C      NSUR    : NUMBER OF EXTERNAL SURFACES BEFORE MERGE  I
C      NVOL    : NUMBER OF REGIONS BEFORE MERGE            I
C      MATCOD  : MIXTURE PER REGION                        I(NREGIO)
C      KEYFLX  : FLUX ELEMENTS IN UNKNOWN SYSTEM.          I(NREGIO)
C      FUNKNO  : UNKNOWN VECTOR SOLVED FOR.                R(NUNKNO,NGROUP)
C      EIGENK  : \keff{} and \kinf{} EIGENVALUES           R(2)
C      B2      :  X, Y, Z, HOMO BUCKLINGS                  R(4)
C      SIGF    : NUSIGF AND CHI STORAGE VECTOR             R(NBMIX,NIFISS)
C      XSA     : ARBITRARY XS VECTOR                       R(0:NBMIX)
C      MATALB  : ALBEDO/MIXTURE INDEX                      I(-NSUR:NVOL)
C      ALBEDO  : ALBEDO OF EXTERNAL SURFACES               R(NSUR)
C      BCRFTR  : BOUNDARY CONDITION 
C                (REFLECTION OR TRANSLATION)               I(NUNSUR)
C      QTOTL   : TOTAL FISSION SOURCE                      R(NREGIO,NIFISS)
C      DIFF    : STORAGE FOR HOMOGENEOUS DIFFUSION         R(NGROUP)
C                COEFFICIENTS
C      IJJ     : LAST SCATTERING GROUP                     I(0:NBMIX)
C                IJJ(0) = 0
C      NJJ     : NUMBER OF SCATTERING GROUP                I(0:NBMIX)
C                NJJ(0) = -NGROUP
C      IPOS    : POSITION SELF SCATTERING IN XSCAT         I(0:NBMIX)
C                IPOS(0)= NGROUP+1
C      KEYMRG  : SURFACE/REGION MERGE INDEX                I(-NSUR:NVOL)
C      COND    : CONDITION TO COMPUTE FUNSUR               I(NUNSUR) 
C      XSCAT   : SCATTERING VECTOR                         R(0:NBMIX*NGROUP)
C                XSCAT(0) = 0.0
C    WORK VECTORS:
C      SUNKNO  : SOURCE FOR SYSTEM OF UNKNOWN.             R(NUNKNO,NGROUP)
C      FUNPOS  : EXTERNAL SURFACE FLUX VECTOR (OUTER FLUX) R(NUNSUR,NGROUP)
C      FUNSUR  : TOTAL EXTERNAL SURFACE FLUX VECTOR.       R(NUNSUR,NGROUP)
C      PSVFUI  : LEAKAGE PROBABILITY                       R(NUNSUR,NREGIO)
C
C
C---------------------------   FLUSUR  --------------------------------
C
      IMPLICIT    NONE
      INTEGER     IOUT,ILCMUP,ILCMDN
      CHARACTER   NAMSBR*6
      PARAMETER  (IOUT=6,ILCMUP=1,ILCMDN=2,
     >            NAMSBR='FLUSUR')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER     IPFLUX,IPSYS,IPMACR,IPRINT,NGROUP,NBMIX,NREGIO,
     >            NIFISS,ITYPEC,ILEAK,NUNKNO,NUNSUR,MATCOD,NSUR,NVOL,
     >            KEYFLX(NREGIO),IJJ(0:NBMIX),NJJ(0:NBMIX),
     >            IPOS(0:NBMIX),MATALB(-NSUR:NVOL),BCRFTR(NUNSUR),
     >            KEYMRG(-NSUR:NVOL),COND(NUNSUR)
      REAL        SUNKNO(NUNKNO,NGROUP),FUNPOS(NUNSUR,NGROUP),
     >            FUNSUR(NUNSUR,NGROUP),ALBEDO(NSUR),
     >            FUNKNO(NUNKNO,NGROUP),EIGENK(2),
     >            B2(4),XSA(0:NBMIX),DIFF(NGROUP),
     >            XSCAT(0:NBMIX*NGROUP),SIGF(NBMIX,NIFISS)
      DOUBLE PRECISION PSVFUI(NUNSUR,NREGIO),
     >                 QTOTL(NREGIO,NIFISS)
      REAL        ALBD(6)
      CHARACTER   CFIXSR*12
C----
C  LOCAL PARAMETERS
C----
      CHARACTER   CGRPNM*12,CFLUNM*12
      INTEGER     IG,IS,IV
      INTEGER     ILCMLN,ILCMTY
      CFIXSR='FIXE        '
      CALL XDRSET(SUNKNO,NUNKNO*NGROUP,0.0)
C----
C  AJOUTER LES SOURCES FIXES
C----
      IF(ITYPEC .EQ. 6) THEN
        CALL FLUQFX(IPMACR,IPRINT,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >              MATCOD,KEYFLX,CFIXSR,SUNKNO,XSA   )
      ENDIF
C----
C  AJOUTER LES SOURCES DE FISSION
C----
      IF(NIFISS .GT. 0) THEN
        CALL FLUAQF(IPMACR,NGROUP,NBMIX ,NREGIO,NIFISS,NUNKNO,
     >              MATCOD,KEYFLX,FUNKNO,EIGENK,SUNKNO,SIGF  ,QTOTL )
      ENDIF
C----
C  AJOUTER LES FUITES (DB2)
C----
      IF(ILEAK .GT. 0) THEN
        CALL FLUAQL(IPFLUX,IPSYS ,NGROUP,NREGIO,NUNKNO,ILEAK ,
     >              KEYFLX,FUNKNO,B2    ,SUNKNO,DIFF  )
      ENDIF
C----
C  AJOUTER LES SOURCES DE DIFFUSION
C----
      CALL FLUAQS(IPMACR,IPSYS ,NGROUP,NBMIX ,NREGIO,NUNKNO,
     >            MATCOD,KEYFLX,FUNKNO,SUNKNO,
     >            IJJ   ,NJJ   ,IPOS  ,XSCAT ,XSA   )
C----
C  CREATION DE LA MATRICE D'ALBEDO
C----
      DO 50 IS=1,NSUR
         ALBEDO(IS)=ALBD(-MATALB(-IS))
 50   CONTINUE
C----
C  LOOP OVER GROUPS
C---- 
      DO 100  IG=1,NGROUP
         WRITE(CGRPNM,1000) IG,NGROUP
         CALL LCMSIX(IPSYS,CGRPNM,ILCMUP)
         CALL LCMLEN(IPSYS,'DRAGON-PCFUI',ILCMLN,ILCMTY)
         IF(ILCMLN .GT. 2*NUNSUR*NREGIO) CALL XABORT(NAMSBR//
     >': Space allocated for PCFUI insufficient')
         CALL LCMGET(IPSYS,'DRAGON-PCFUI',PSVFUI)
C----
C  [FUNPOS]=[PSVFUI]*[SUNKNO]
C  SOURCE DANS REGION I =SUNKNO(KEYFLX(I),IGRP)   
C----
         DO 110 IS=1,NUNSUR
            FUNPOS(IS,IG)=0.0
            DO 120 IV=1,NREGIO
               FUNPOS(IS,IG)=FUNPOS(IS,IG)+(PSVFUI(IS,IV)*
     >                       SUNKNO(KEYFLX(IV),IG))
 120        CONTINUE
 110     CONTINUE
         CALL LCMSIX(IPSYS,CGRPNM,ILCMDN)
C----
C  [FUNSUR]=([FUNPOS]+[FUNNEG])/2.0
C  avec [FUNNEG]=[ALBEDO]*[FUNPOS]
C----
         CALL XDISET(COND,NUNSUR,0)
         DO 130 IS=1,NSUR
            IF(COND(-KEYMRG(-IS)).EQ.0) THEN
               FUNSUR(-KEYMRG(-IS),IG)=(FUNPOS(-KEYMRG(-IS),IG)+
     >             ALBEDO(IS)*FUNPOS(BCRFTR(-KEYMRG(-IS)),IG))/2.0
               COND(-KEYMRG(-IS))=1
            ENDIF
 130     CONTINUE
C----
C  SAVE [FUNSUR] AT 'IPFLUX' ADRESS
C----
         WRITE(CFLUNM,2000) IG
         CALL LCMPUT(IPFLUX,CFLUNM,NUNSUR,2,FUNSUR(1,IG))
 100  CONTINUE

      RETURN
C----
C  FORMATS
C---
 1000 FORMAT('GROUP',I3,'/',I3)
 2000 FORMAT('FUNSUR',I3)
      END
