*DECK MOCAJS
      SUBROUTINE MOCAJS(IPFLUX,IPMACR,IAJMOD,NMAT,NSOUT,NGRP,NREG,
     >                  ADJSOU,ADJFXS,MATALB,LMODE,VOL)

* ---------------------------  MOCAJS  ---------------------------------
*  1- PROGRAMME STATISTICS:
* 
*      NAME     :  MOCAJS
*      USE      :  TO INITIALIZE ADJOINT SOURCE AND CREATE DIRECTORY 
*                  TO STORE THE ADJOINT SOURCE VALUES IN IPFLUX.
*      MODIFIED BY :  MONCHAI A. (10/04/04)  
* 
*      CALLED BY MOC2AJ.F
* 
*         IPFLUX : POINTER TO FLUX FILE                   I
*         IPMACR : POINTER TO MACRO LIB                   I                  
*         NGRP   : NO. OF GROUPS                          I
*         NMAT   : NO. OF MIXTURES                        I
*         NREG   : NO. OF REGIONS                         I
*         MATALB : THE ALBEDO/MIXTURE INDEX ASSOCIATED    I(-NSOUT:NREG)
*                  WITH EACH SURFACE AND REGION.
*         IAJMOD  : FLAG FOR TYPE OF ADJOINT CALCULATION  I
*           IAJMOD = 1 ADJOINT FLUX CALCULATION WITHOUT 
*                     EXTERNAL SOURCE
*           IAJMOD = 2 ADJOINT FLUX CALCULATION WITH 
*                     EXTERNAL SOURCE
*           IAJMOD = 3 ADJOINT FLUX CALCULATION WITH 
*                     EXTERNAL GENERALIZED ADJOINT SOURCE
*         
*   INPUT/OUTPUT
* 
*   OUTPUT
*     ADJFXS     : REAL ARRAY CONTAINING ADJOINT SOURCE  R(0:NMAT,NGRP)
*                  INDEXED BY MIXTURE AND GROUP NUMBERS 
*     ADJSOU     : REAL ARRAY CONTAINING ADJOINT SOURCE  R(NREG,NGRP)
*                  INDEXED BY REGION AND GROUP NUMBERS
* --------------------------   MOCAJS    ------------------------------

      IMPLICIT NONE
      CHARACTER*12  NAMFLX,CFIXSR,CGRPNM
      INTEGER       NGRP,NSOUT,NREG,IAJMOD,IPFLUX,IPMACR,NMAT,
     >              MATALB(-NSOUT:NREG)
      INTEGER       IGR,IREG,ILCMUP,ILCMDN,IOUT,ILCMLN,ILCMTY

      REAL          ADJFXS(0:NMAT,NGRP),ADJSOU(NREG,NGRP)
      LOGICAL       LMODE(20)
      REAL          VOL(NREG)
      PARAMETER     (IOUT=6,ILCMUP=1,ILCMDN=2)
      CFIXSR='FIXA        '
      DO 100 IGR=1,NGRP
        IF(IAJMOD.EQ.1) THEN
           CALL XDRSET(ADJSOU(1,IGR),NREG,0.00)
        ELSE IF(IAJMOD.EQ.2 .OR. IAJMOD.EQ.3) THEN
           CALL XDRSET(ADJSOU(1,IGR),NREG,0.00)
           WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGR,NGRP
           CALL LCMSIX(IPMACR,CGRPNM,ILCMUP) 
           CALL   LCMLEN(IPMACR,CFIXSR,ILCMLN,ILCMTY)
           IF(ILCMLN .EQ. NMAT) THEN
             ADJFXS(1,IGR)=0.0
             CALL LCMGET(IPMACR,CFIXSR,ADJFXS(1,IGR))
           ELSE IF(ILCMLN .EQ. NMAT+1) THEN
             CALL LCMGET(IPMACR,CFIXSR,ADJFXS(0,IGR))
           ELSE
             CALL XABORT('MOCAJS:INVALID DATA LENGTH ADJFIXSOURCE')
           ENDIF
           CALL LCMSIX(IPMACR,CGRPNM,ILCMDN)
           DO 10 IREG=1,NREG
             ADJSOU(IREG,IGR)= ADJFXS(MATALB(IREG),IGR)
10         CONTINUE
        ELSE 
          CALL XABORT( 'MOCAJS: INVALID IAJMOD VALUES')           
        ENDIF
        IF (IAJMOD.EQ.3 .AND. LMODE(5))  THEN
          DO 21 IREG=1,NREG
               ADJSOU(IREG,IGR) = ADJSOU(IREG,IGR)/VOL(IREG)
21        CONTINUE
        ENDIF
        IF(IAJMOD.GT.1) THEN
          WRITE(NAMFLX,'(6HADJSOU,I3,1X)') IGR
          CALL LCMPUT(IPFLUX,NAMFLX,NREG,2,ADJSOU(1,IGR))
        ENDIF
 100  CONTINUE
      RETURN
      END









