*DECK MOCIK3
      SUBROUTINE MOCIK3(NDIM,NL,ISIGNR,KEYANI)
*
*     WRITTEN BY R.ROY
*
*  1. GENERATES ALL SIGNS ISIGNR(L,M,K) FOR SPHERICAL HARMONICS R(L,M)
*               FOR 0 <= L <= NL (AND FOR -L <= M <= L)
*     ON THE 8 OCTANT ANGULAR MODES FOR 1 <= K <= 8.
*
*     DEFINITION OF THESE SIGNS:
*     ISIGNR(L,M,1)= +1
*     ISIGNR(L,M,2)= SIGN(M)*(-1)**M
*     ISIGNR(L,M,3)= SIGN(M)
*     ISIGNR(L,M,4)= (-1)**M
*     ISIGNR(L,M,5)= (-1)**(L+M)
*     ISIGNR(L,M,6)= SIGN(M)*(-1)**L
*     ISIGNR(L,M,7)= SIGN(M)*(-1)**(L+M)
*     ISIGNR(L,M,8)= (-1)**L
*     WHERE SIGN(M)= +1 FOR 0 <= M
*                    -1 FOR M <  0
*
*  2. ALL THESE ISIGNR VALUES ARE COMPRESSED TO BE USED
*     ACCORDING TO THE RECTANGULAR DIMENSION 'NDIM'. THE REMAINING
*     PLACE IS FILLED UP WITH ZEROS TO ENSURE THAT NO PROBLEM ARISE
*     IF ONE OVERRIDES THE DIMENSIONS.
*
*
      IMPLICIT NONE
      INTEGER  NDIM,NL,ISIGNR(8*(NL+1)*(NL+1)),NEWMOD(8,3),
     >         K,L,M,IND1,IND2,IND3,KNEW,INDNEW,NSELEC,
     >         KEYANI((NL+1)*(NL+1))
      LOGICAL  LROK
      DATA NEWMOD    / 1,0,0,0, 0,0,2,0,
     >                 1,2,3,4, 0,0,0,0,
     >                 1,2,3,4, 5,6,7,8 /
*
*     INDEX FOR SIGN 'ISIGNR' DEPENDING ON THE DIMENSION
      IND1(NSELEC,K)=      NSELEC + 1 + (K-1)* (NL+1)
      IND2(NSELEC,K)=      NSELEC + 1 + (K-1)*((NL+1)*(NL+2)/2)
      IND3(L,M,K)=    L*(L+1) + M + 1 + (K-1)*((NL+1)*(NL+1))
*
      DO 30 L= 0, NL
         DO 20 M= -L, L
            ISIGNR(IND3(L,M,1))= +1
            ISIGNR(IND3(L,M,2))= ISIGN(1,M)*(-1)**M
            ISIGNR(IND3(L,M,3))= ISIGN(1,M)
            ISIGNR(IND3(L,M,4))= (-1)**M
            ISIGNR(IND3(L,M,5))= (-1)**(L+M)
            ISIGNR(IND3(L,M,6))= ISIGN(1,M)*(-1)**L
            ISIGNR(IND3(L,M,7))= ISIGN(1,M)*(-1)**(L+M)
            ISIGNR(IND3(L,M,8))= (-1)**L
   20    CONTINUE
   30 CONTINUE
*
***** SELECTS THE GOOD SIGN ISIGNR(L,M) FUNCTIONS
*             FOR NDIM=1(SLAB),2(TWO-D RECT),3(THREE-D).
*
*     COMPRESSES 'ISIGNR' IN PLACE.
*
      DO 80 K= 1, 8
         NSELEC= 0
         KNEW= NEWMOD(K,NDIM)
         IF( KNEW.NE.0 )THEN
            DO 50 L= 0, NL
               DO 40 M= -L, L
                  IF( NDIM.EQ.1 )THEN
                     LROK= M.EQ.0
                     INDNEW= IND1(NSELEC,KNEW)
                  ELSEIF( NDIM.EQ.2 )THEN
                     LROK= MOD(L+M,2).EQ.0
                     INDNEW= IND2(NSELEC,KNEW)
                  ELSEIF( NDIM.EQ.3 )THEN
                     LROK= .TRUE.
                     INDNEW= IND3(L,M,KNEW)
                  ENDIF
                  IF( LROK )THEN
                     NSELEC= NSELEC+1
                     IF(INDNEW.NE.IND3(L,M,K))THEN
                        ISIGNR(INDNEW)= ISIGNR(IND3(L,M,K))
                     ENDIF
                     KEYANI(NSELEC) = L
                  ELSE
                     ISIGNR(IND3(L,M,K))= 0
                  ENDIF
   40          CONTINUE
   50       CONTINUE
         ELSE
            DO 70 L= 0, NL
               DO 60 M= -L, L
                  ISIGNR(IND3(L,M,K))= 0
   60          CONTINUE
   70       CONTINUE
         ENDIF
   80 CONTINUE
*
*     RETURN
      END
