*DECK MOCPIS
      SUBROUTINE MOCPIS(IPFLUX,DIRPIS,
     >                  NREG  ,NGRP  ,NL,NMAT,NANIS ,NSOUT ,NIFIS,NGSS,
     >                  NALB  ,MXGAUS,IFTRK2 ,IFTRAK,NTRK2 ,NTRK,MXSEG,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,XSTRC,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS  ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LOWTRK,IPRT,
     >                  VOL   ,VOLINV,MAXITR ,IGDBST,JGDBST,X1,Y1,Z1,S1)
*
* ---------------------------  MOCPIS  ---------------------------------
* 
*  1- PROGRAMME STATISTICS:
*      NAME     :  MOCPIS
*      USE      :  EVALUATE PIS MATRIX
*      MODIFIED BY :  MONCHAI A. (24/02/05)   
* 
*  2- ROUTINE PARAMETERS:
*    INPUT
*      IPFLUX   : POINTER TO THE FLUX                    I
*      IPTRK    : POINTER TO THE TRACKING                I
*      IPMACR   : POINTER TO THE MACROLIB                I
*      IFTRAK   : FILE UNIT NUMBER FOR TRACKS.           I
*      IPIS     : INDEX TO PIS MATRIX                    I
*      MATALB   : MATERIAL CODE IN REGIONS.              I(-NSOUT:
*                                                          NREG)
*      MXSEG    : MAXIMUM NUMBER OF SEGMENTS ON EACH     I
*                 TRACK (VARIABLE ACC. TO TRACKING LINE)
*      MXGAUS   : FIXED MAXIMUM NUMBER OF SEGMENTS(=64)  I
*      NGRP     : NUMBER OF GROUPS TREATED               I
*      NANIS    : NUMBER OF ANISOTROPY IN                I
*                 SCATTERING CROSS SECTIONS
*      NFUNL    : NUMBER OF LEGENDRE ORDER FOR FLUX      I
*      NGSS     : NUMBER OF GAUSS INTEGRATION POINTS     I  
*      NGSS2    : NUMBER OF LOW ORDER GAUSS INTEGRATION  I  
*                 POINTS
*      NIFIS    : NUMBER OF FISSILE ISOTOPES             I
*      NMAT     : NUMBER OF MIXTURES CONSIDERED          I
*      NREGIO   : NUMBER OF REGIONS CONSIDERED           I
*      NSOUT    : NUMBER OF SURFACES CONSIDERED          I
*      NTRK     : NUMBER OF TRACKING LINES               I 
*      NTRK2    : NUMBER OF LOW ORDER TRACKING LINES     I  
*      VOL      : VOLUME OF REGION                       R(NREG)
*      WGSS     : POLAR WEIGHT                           R(MXGAUS)
*      YGSS     : COSINE OF POLAR ANGLE                  R(MXGAUS)
*      XSDIA    : WITHIN GROUP SCATTERING                R(NMAT+1)
*                 CROSS SECTION
*      XSCAT    : GROUP SCATTERING CROSS SECTION         R(NMAT+1)
*      XSCHI    : FISSION NEUTRON DISTRIBUTION           R(0:NMAT,
*                                                          NIFIS,
*                                                          NGRP)
*      XSNUF    : GROUP FISSION NEUTRON PRODUCTION       R(0:NMAT,
*                 CROSS SECTION                            NIFIS,
*                                                          NGRP)
*      XSTRC    : GROUP TRANSPORT-CORRECTED TOTAL    R(NMAT+1)
*                 CROSS SECTION    
*   WORK VECTOR
*      FLM      : DUMMY VARIABLE                         D(NGSS,MXSEG)
*      FLP      : DUMMY VARIABLE                         D(NGSS,MXSEG) 
*      CYM      : DUMMY VARIABLE                         D(NGSS,MXSEG)
*      CYP      : DUMMY VARIABLE                         D(NGSS,MXSEG)
*      X1       : DUMMY VARIABLE                         D(NREG,NGRP)
*      Y1       : DUMMY VARIABLE                         D(NREG,NGRP)
*      S1       : DUMMY VARIABLE                         D(NREG,NGRP)
*      Z1       : DUMMY VARIABLE                         D(NREG,NGRP)
*
*--------------------------   MOCPIS    ------------------------------
*
      IMPLICIT NONE
      REAL        A
      COMMON      A(1)
      INTEGER     I(1)
      EQUIVALENCE (A(1),I(1))
      INTEGER      IPFLUX,MAXITR,IGDB1,JGDB1,IGDBST,JGDBST
      INTEGER      NREG,NGRP,NL,NMAT,NANIS,NSOUT,NIFIS,NGSS,NGSS2,
     >             NALB,MXGAUS,IFTRK2,IFTRAK,NTRK2,NTRK,MXSEG,
     >             MATALB(-NSOUT:NREG),NRSEG(MXSEG)
      INTEGER      IG,JG,IREG,JREG,IS,IOUT,IPRT,IFFIL,NTRACK 
      REAL         SIGANG(NGSS,-NALB:NMAT,NGRP),XSDIA(0:NMAT,NGRP),
     >             XSCAT(NGRP,0:NMAT,0:NANIS,NGRP),XTINV(0:NMAT,NGRP),
     >             SEGLEN(MXSEG),ALBEDO(NALB),XSTRC(0:NMAT,NGRP),
     >             EXPT(NGSS,MXSEG),EXP2(NGSS,MXSEG),WGSS(MXGAUS),
     >             YGSS(MXGAUS),VOLINV(NREG),VOL(NREG)
      DOUBLE PRECISION  FLM(NGSS,MXSEG),FLP(NGSS,MXSEG),
     >                  CYM(NGSS,MXSEG),CYP(NGSS,MXSEG)
      DOUBLE PRECISION X1(NREG,NGRP),Y1(NREG,NGRP),Z1(NREG,NGRP),
     >                 S1(NREG,NGRP)
      DOUBLE PRECISION  DZERO,DONE,DSUM
      REAL        ZERO
      LOGICAL  LEXTSO,LXSCAT,LOWTRK,LCWDR1,LCWAJ2
      INTEGER  ILCMLN,ILCMTY,ILCMUP,ILCMDN,IPIS 
      PARAMETER(IOUT=6,ILCMUP=1,ILCMDN=2)
      CHARACTER*12 NAMFLX,DIRPIS
      DZERO = 0.0000000000000000D+0
      DONE  = 1.0000000000000000D+0
      ZERO  = 0.00000000E+0
      LCWDR1 = .TRUE.
      LCWAJ2 = .FALSE.
      CALL LCMLEN(IPFLUX,DIRPIS,ILCMLN,ILCMTY)
      IF (ILCMLN .EQ. -1) THEN
      CALL LCMSIX(IPFLUX,DIRPIS,ILCMUP)
       DO 101 IG= IGDBST, NGRP-JGDBST
         WRITE(NAMFLX,'(4HPIS ,I3,1X)') IG 
         CALL LCMLEN(IPFLUX,NAMFLX,ILCMLN,ILCMTY)
         IF (ILCMLN.NE.NREG .OR. ILCMTY.NE.2) 
     >       GO TO 1000
101    CONTINUE
      CALL LCMSIX(IPFLUX,DIRPIS,ILCMDN) 
      IF(IPRT.GE.1000)  WRITE(IOUT,'(1X,A41)') 
     >  'PIS DIRECTORY EXIST, RETURN TO OUTER LOOP'
      RETURN
      ENDIF
1000  CONTINUE
      CALL SETARA(A,NREG,IPIS)
      CALL XDRSET(A(IPIS),NREG,0.0)
      CALL XDRSET(X1,2*NGRP*NREG,0.0)
      DO 100 IG= IGDBST, NGRP-JGDBST
        DO 200 IREG=1,NREG
         S1(IREG,IG)=XSTRC(MATALB(IREG),IG)
200    CONTINUE
        IGDB1=IG
        JGDB1=NGRP-IG
        LXSCAT=.FALSE. 
        LEXTSO=.TRUE.
            CALL MOCSWP(NREG  ,NGRP  ,NL,NMAT,NANIS ,NSOUT ,NIFIS,NGSS,
     >                  NALB  ,MXGAUS,IFTRK2 ,IFTRAK,NTRK2 ,NTRK,MXSEG,
     >                  MATALB,NRSEG ,SIGANG ,XSDIA ,XSCAT ,XTINV,
     >                  SEGLEN,ALBEDO,EXPT   ,EXP2  ,WGSS ,YGSS,NGSS2,
     >                  FLM   ,FLP   ,CYM    ,CYP   ,LXSCAT,LOWTRK,
     >                  VOLINV,LEXTSO,IGDB1  ,JGDB1 ,MAXITR,LCWAJ2,
     >                  X1,S1,Z1,S1)
      CALL LCMSIX(IPFLUX,DIRPIS,ILCMUP)
      DO 201 IREG=1,NREG
         A(IPIS-1+IREG)=REAL(DONE-Z1(IREG,IG))
201   CONTINUE
      WRITE(NAMFLX,'(4HPIS ,I3,1X)') IG 
      CALL LCMPUT(IPFLUX,NAMFLX,NREG,2,A(IPIS))
      CALL LCMSIX(IPFLUX,DIRPIS,ILCMDN) 
100   CONTINUE
      CALL RLSARA(A(IPIS))
      RETURN
      END
