*DECK MOCSOL
      SUBROUTINE MOCSOL(N,IS,B,IER,MAX)
*
*-----------------------------------------------------------------------
*
* SOLUTION OF A SYSTEM OF LINEAR EQUATIONS USING GAUSSIAN ELIMINATION
* WITH PARTIAL PIVOTING. DOUBLE PRECISION VERSION.
* THERE IS NO PIVOTING FOR TRIANGULAR UPPER PART OF THE MATRIX.
*
* INPUT PARAMETERS:
*  N       : ORDER OF THE COEFFICIENT MATRIX.
*  IS      : NUMBER OF RIGHT HAND VECTORS.
*  B       : COEFFICIENT MATRIX AUGMENTED WITH THE RIGHT HAND VECTORS.
*            DIMENSION B(MAX,N+IS)
*  MAX     : FIRST DIMENTION OF MATRIX B.
*
* OUTPUT PARAMETERS:
*  B       : SOLUTION VECTORS, STARTING AT B(1,N+1).
*  IER     : ERROR FLAG. EXECUTION FAILURE IF IER.NE.0.
*
*------------------------------------------------ AUTHOR: R. ROY    ----
*------------------------------ BASED ON *ALSBD*  AUTHOR: A. HEBERT ----
*
      IMPLICIT         NONE
      INTEGER          N, IS, IER, MAX
      INTEGER          M, JCOL, I, IN, J, K
      DOUBLE PRECISION B(MAX,*), ZERO, PER, PIVOT
      PARAMETER      ( ZERO=0.0D0 )
      M= N+IS
      IF( N.EQ.1 )THEN
         PIVOT= B(1,1)
         IF( PIVOT.EQ.ZERO )THEN
            IER= 1
            RETURN
         ENDIF
         DO  5 J=2,M
            B(1,J)= B(1,J)/PIVOT
    5    CONTINUE
      ELSE
*
*        SEARCH FOR MAXIMUM PIVOT ON COLUMN JCOL.
         DO 40 JCOL=1,N-1
*
*           CHECK IF ALL-ZERO ELEMENTS ON LINE, THEN NO P.P.
            PIVOT= B(JCOL,JCOL)
            IN= JCOL
            DO 15 I= JCOL+1,N
               IF( B(JCOL,I).NE.ZERO )THEN
                  IN= I
               ENDIF
   15       CONTINUE
            IF( IN.NE.JCOL.OR.PIVOT.EQ.ZERO )THEN
*
*              SEARCH FOR MAXIMUM PIVOT ON COLUMN JCOL.
               IN= JCOL
               DO 10 I=JCOL+1,N
                  IF( ABS(B(I,JCOL)).GT.ABS(PIVOT) )THEN
                     IN= I
                     PIVOT= B(I,JCOL)
                  ENDIF
   10          CONTINUE
            ENDIF
            IF( PIVOT.EQ.ZERO )THEN
               IER=1
               RETURN
            ENDIF
*
*           TRIANGULARIZATION.
            PIVOT= B(IN,JCOL)
            B(IN,JCOL)= B(JCOL,JCOL)
            DO 30 J=JCOL+1,M
               PER= B(IN,J)/PIVOT
               B(IN,J)= B(JCOL,J)
               B(JCOL,J)= PER
               DO 20 I=JCOL+1,N
                  B(I,J)= B(I,J)-B(I,JCOL)*PER
   20          CONTINUE
   30       CONTINUE
   40    CONTINUE
*
         PIVOT= B(N,N)
         IF( PIVOT.EQ.ZERO )THEN
            IER=1
            RETURN
         ENDIF
         DO 50 J=N+1,M
            B(N,J)= B(N,J)/PIVOT
   50    CONTINUE
*
*        BACK SUBSTITUTION.
         DO 80 IN=2,N
            I=N-IN+1
            DO 70 J=N+1,M
               PER=B(I,J)
               DO 60 K=I+1,N
                  PER=PER-B(I,K)*B(K,J)
   60          CONTINUE
               B(I,J)=PER
   70       CONTINUE
   80    CONTINUE
      ENDIF
      IER=0
      RETURN
      END
