*DECK MRGVST
      SUBROUTINE MRGVST(IPRINT,IUPD,NSOUTO,NVOUTO,NSOUTN,NVOUTN,
     >                  IMERGE,MIXN,MATO,VOLO,MATN,VOLN)
C
C---------------------------  MRGVST ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : MRGVST
C      USE      : MERGE VOLUME SURFACE INFORMATION ON TRACK FILE
C      MODIFIED : 97-11-06 (G.M)
C      AUTHOR   : G.MARLEAU
C
C  2- ROUTINE PARAMETERS:
C    INPUT
C      IPRINT   : PRINT LEVEL                      I
C      IUPD     : TYPE OF MERGE REQUIRED           I(4)
C                 IUPD(1) FOR REGION MERGE
C                 IUPD(2) FOR SURFACE MERGE
C                 IUPD(3) FOR MATERIAL MERGE
C                 IUPD(4) FOR ALBEDO MERGE
C      NSOUTO   : OLD NUMBER OF SURFACES           I
C      NVOUTO   : OLD NUMBER OF REGIONS            I
C      NSOUTN   : NEW NUMBER OF SURFACES           I
C      NVOUTN   : NEW NUMBER OF REGIONS            I
C      IMERGE   : MERGED POSITION                  I(-NSOUTO:NVOUTO)
C      MIXN     : NEW MATERIAL FOR OLD REGIONS     I(NVOUTO)
C      MATO     : OLD MATERIAL PER REGION          I(-NSOUTO:NVOUTO)
C      VOLO     : OLD VOLUMES                      I(-NSOUTO:NVOUTO)
C    OUTPUT
C      MATN     : NEW MATERIAL PER REGION          I(-NSOUTN:NVOUTN)
C      VOLN     : NEW VOLUMES                      I(-NSOUTN:NVOUTN)
C
C
C---------------------------   MRGVST --------------------------------
C
      IMPLICIT         NONE
      INTEGER          IOUT
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NAMSBR='MRGVST')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER          IPRINT,IUPD(4),NSOUTO,NVOUTO,NSOUTN,NVOUTN,
     >                 IMERGE(-NSOUTO:NVOUTO),MIXN(NVOUTO),
     >                 MATO(-NSOUTO:NVOUTO),MATN(-NSOUTN:NVOUTN)
      REAL             VOLO(-NSOUTO:NVOUTO),VOLN(-NSOUTN:NVOUTN)
C----
C  LOCAL VARIABLES
C----
      INTEGER          IVSN,IVSO,IVST
      DOUBLE PRECISION DVOL
C----
C  VERIFY IF BOUNDARY CONDITIONS ARE ADEQUATE FOR MERGE VECTOR
C----
      IF(IPRINT.GE.10) THEN
        WRITE(IOUT,6000)
        WRITE(IOUT,6010) (MATO(IVSO),VOLO(IVSO),IVSO=-1,-NSOUTO,-1)
      ENDIF
      IF(IUPD(2). LT. 0) THEN
        DO 100 IVSN=-NSOUTN,-1
          MATN(IVSN)=0
          DVOL=0.0D0
          DO 101 IVSO=-NSOUTO,-1
           IF(IMERGE(IVSO) .EQ. IVSN) THEN
             IF(MATN(IVSN) .EQ. 0) THEN
               MATN(IVSN)=MATO(IVSO)
             ELSE IF(MATN(IVSN) .NE. MATO(IVSO))THEN
               WRITE(IOUT,6100) NAMSBR,IVSN,MATN(IVSN),IVSO,MATO(IVSO)
               WRITE(IOUT,6101)
               WRITE(IOUT,6102) (IVST,MATO(IVST),IVST=-NSOUTO,-1)
               CALL XABORT(NAMSBR//
     >           ': BOUNDARY CONDITIONS INCOMPATIBLE FOR MERGE')
             ENDIF
             DVOL=DVOL+DBLE(VOLO(IVSO))
           ENDIF
 101      CONTINUE
          VOLN(IVSN)=REAL(DVOL)
 100    CONTINUE
      ELSE
        DO 110 IVSO=-NSOUTO,-1
          MATN(IVSO)=MATO(IVSO)
          VOLN(IVSO)=VOLO(IVSO)
 110    CONTINUE
      ENDIF
      IF(IPRINT.GE.10) THEN
        WRITE(IOUT,6001)
        WRITE(IOUT,6010) (MATN(IVSN),VOLN(IVSN),IVSN=-1,-NSOUTN,-1)
        WRITE(IOUT,6002)
        WRITE(IOUT,6010) (MATO(IVSO),VOLO(IVSO),IVSO=1,NVOUTO)
      ENDIF
C----
C  CHANGE ORIGINAL MATERIAL IF REQUESTED
C----
      IF(IUPD(3) .GT. 0) THEN
        DO 120 IVSO=1,IUPD(3)
          MATO(IVSO)=MIXN(IVSO)
 120    CONTINUE
      ENDIF
C----
C  VERIFY IF MATERIALS ARE ADEQUATE FOR MERGE VECTOR
C----
      IF(IUPD(1) .GT. 0) THEN
        MATN(0)=0
        VOLN(0)=0.0
        DO 130 IVSN=1,NVOUTN
          MATN(IVSN)=0
          DVOL=0.0D0
          DO 131 IVSO=1,NVOUTO
           IF(IMERGE(IVSO) .EQ. IVSN) THEN
             IF(MATN(IVSN) .EQ. 0) THEN
               MATN(IVSN)=MATO(IVSO)
             ELSE IF(MATN(IVSN) .NE. MATO(IVSO))THEN
               WRITE(IOUT,6200) NAMSBR,IVSN,MATN(IVSN),IVSO,MATO(IVSO)
               WRITE(IOUT,6201)
               WRITE(IOUT,6202) (IVST,MATO(IVST),IVST=1,NVOUTO)
               CALL XABORT(NAMSBR//
     >           ': MATERIALS INCOMPATIBLE FOR MERGE')
             ENDIF
             DVOL=DVOL+DBLE(VOLO(IVSO))
           ENDIF
 131      CONTINUE
          VOLN(IVSN)=REAL(DVOL)
 130    CONTINUE
      ELSE
        DO 140 IVSO=1,NVOUTO
          MATN(IVSO)=MATO(IVSO)
          VOLN(IVSO)=VOLO(IVSO)
 140    CONTINUE
      ENDIF
C----
C  CHANGE FINAL MATERIAL IF REQUESTED
C----
      IF(IUPD(3) .LT. 0) THEN
        DO 150 IVSN=1,-IUPD(3)
          MATN(IVSN)=MIXN(IVSN)
 150    CONTINUE
      ENDIF
      IF(IPRINT.GE.10) THEN
        WRITE(IOUT,6003)
        WRITE(IOUT,6010) (MATN(IVSN),VOLN(IVSN),IVSN=1,NVOUTN)
      ENDIF
C----
C  PRINT FORMATS
C----
 6000 FORMAT(' INITIAL SURFACE AREA AND ALBEDO NUMBER')
 6001 FORMAT(' FINAL SURFACE AREA AND ALBEDO NUMBER')
 6002 FORMAT(' INITIAL REGION VOLUME AND MATERIAL')
 6003 FORMAT(' FINAL REGION VOLUME AND MATERIAL')
 6010 FORMAT(1P,5(I10,E15.7))
C----
C  ABORT FORMATS
C----
 6100 FORMAT(' ------ ABORT IN ROUTINE ',A6,'  ------'/
     >       ' BOUNDARY CONDITIONS INCOMPATIBLE FOR MERGE '/
     >       ' NEW REGION = ',I10,5X,'SURFACE =',I10/
     >       ' OLD REGION = ',I10,5X,'SURFACE =',I10/
     >       ' ----------------------------------------')
 6101 FORMAT(' SURFACE DESCRIPTION '/
     >       4(' SURFACE  ->  ALBEDO'))
 6102 FORMAT(4(1X,I7,6X,I6))
 6200 FORMAT(' ------ ABORT IN ROUTINE ',A6,'  ------'/
     >       ' MATERIAL INCOMPATIBLE FOR MERGE '/
     >       ' NEW REGION = ',I10,5X,'MATERIAL =',I10/
     >       ' OLD REGION = ',I10,5X,'MATERIAL =',I10/
     >       ' ----------------------------------------')
 6201 FORMAT(' REGION DESCRIPTION '/
     >       4(' REGION  -> MATERIAL'))
 6202 FORMAT(4(1X,I6,5X,I8))
C----
C  RETURN
C----
      RETURN
      END
