*DECK QIJABC
      SUBROUTINE QIJABC(NREG,NSOUT,NPRB,NGRP,SIGTAL,MATRT,PROB,IDL,
     >                  PSST,PSVT)
C
C-------------------------    QIJABC    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJABC
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : THIS ROUTINE RECONSTRUCT COLLISION PROBABILITIES (CP)
C                FOR  ALL ZONES ELIMINATING SURFACES FROM THE SYSTEM.
C     MODIFIED : 98-02-11 (G.M.) ( PERIODIC B.C. )
C     AUTHOR   : R. ROY (96-09-18)
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSOUT   : # OF SURFACES FOR GEOMETRY.          I
C     NPRB    : NUMBER OF PROBABILITIES IN PROB      I
C     NGRP    : NUMBER OF GROUPS                     I
C     SIGTAL  : ALBEDO-SIGT VECTOR                   R(-NSOUT:NREG)
C     MATRT   : REFLECTION/TRANSMISSION VECTOR       I(NSOUT)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           D(NGRP,NPRB)
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,NPRB)
C  WORK
C     IDL     : PSS MATRIX DIAGONAL LOCATION         I(NSOUT)
C     PSST    : WORKING AREA FOR PSST                D(NSOUT,NSOUT)
C               PSST=(A**(-1)-PSS)**(-1)
C     PSVT    : WORKING AREA FOR PSVT                D(NGRP,
C                                                      NSOUT,NREG)
C               PSVT=PSST*PSV
C
C  3-EXTERNAL ROUTINE CALLED
C     ALINVD  : INVERSE A DOUBLE PRECISION MATRIX
C     XABORT  : DRAGON ABORT ROUTINE
C
C
      IMPLICIT         NONE
      INTEGER          NREG,NSOUT,NGRP,NPRB,MATRT(NSOUT),IDL(NSOUT)
      REAL             SIGTAL(NGRP,-NSOUT:NREG)
      DOUBLE PRECISION PROB(NGRP,NPRB),PSST(NSOUT,NSOUT),
     >                 PSVT(NGRP,NSOUT,NREG)
      INTEGER          NSP1,IG,IPSS,ISUR,ISX,IS,JSX,JS,IER,
     >                 IV,JV,IVSI,IVVI,ISV,IVS,IVV
      NSP1=NSOUT+1
C----
C  SCAN OVER EACH GROUP
C----
      DO 200 IG=1,NGRP
        IPSS=0
        ISUR=(NSOUT*NSP1)/2
        ISX=0
C----
C  EVALUATE MATRIX (A**(-1)-PSS)
C----
        DO 100 IS=-NSOUT,-1,1
          ISX=ISX+1
          JSX=0
          ISUR=ISUR+1
          DO 101 JS=-NSOUT,IS,1
            JSX=JSX+1
            IPSS=IPSS+1
            IF((SIGTAL(IG,IS).EQ.0.0).OR.
     >         (SIGTAL(IG,JS).EQ.0.0)) THEN
              PSST(ISX,JSX)= 0.0D0
            ELSE
              PSST(ISX,JSX)=-PROB(IG,IPSS)
            ENDIF
            IF(JS.NE.IS) THEN
              PSST(JSX,ISX)=DBLE(PSST(ISX,JSX))
            ENDIF
 101      CONTINUE
          IF(SIGTAL(IG,IS) .EQ. 0.0)THEN
            PSST(ISX,ISX)=PROB(IG,ISUR)
          ELSE
            JS=-MATRT(-IS)
            IF(JS .EQ. IS) THEN
              PSST(ISX,ISX)=PSST(ISX,ISX)
     >                     +PROB(IG,ISUR)/DBLE(SIGTAL(IG,IS))
            ELSE IF(JS .LT. IS) THEN
              JSX=NSOUT+JS+1
              PSST(ISX,JSX)=PSST(ISX,JSX)
     >                     +PROB(IG,ISUR)/DBLE(SIGTAL(IG,IS))
              PSST(JSX,ISX)=PSST(ISX,JSX)
            ENDIF
          ENDIF
 100    CONTINUE
C----
C  INVERSE MATRIX PSST=(A**(-1)-PSS)
C----
        CALL ALINVD(NSOUT,PSST,NSOUT,IER,IDL)
C----
C  CHECK IF INVERSE IS VALID
C----
        IF(IER .NE. 0 ) CALL XABORT
     >    ('QIJABC: IMPOSSIBLE TO INVERT PSS COUPLING MATRIX')
        IVSI=(NSP1*(NSP1+1))/2
        IVVI=IVSI+NSP1
        DO 110 IV=1,NREG
C----
C    PSVT(IS,IV)=SUM(JSS) PSST(ISS,JSS)*PSV(JSS,IV)
C----
          DO 111 IS=1,NSOUT
            PSVT(IG,IS,IV)=0.0D0
 111      CONTINUE
          DO 120 IS=1,NSOUT
            DO 121 JS=1,NSOUT
              ISV=IVSI+JS
              PSVT(IG,IS,IV)=PSVT(IG,IS,IV)+PSST(IS,JS)*PROB(IG,ISV)
 121        CONTINUE
 120      CONTINUE
          IVV=IVVI
          DO 130 JV=1,IV
            IVV=IVV+1
            ISV=0
            IVS=IVSI
            DO 131 IS=-NSOUT,-1,1
              ISV=ISV+1
              IVS=IVS+1
              IF(SIGTAL(IG,IS).NE.0.0) THEN
                PROB(IG,IVV)=PROB(IG,IVV)
     >                      +PROB(IG,IVS)*PSVT(IG,ISV,JV)
              ENDIF
 131        CONTINUE
 130      CONTINUE
          IVSI=IVSI+NSP1+IV
          IVVI=IVVI+NSP1+IV
 110    CONTINUE
 200  CONTINUE
      RETURN
      END
