*DECK QIJNOS
      SUBROUTINE QIJNOS(NREG,NSNEG,NPRB,NGRP,SIGINV,PROB,VOLUME,ARRAY)
C
C-------------------------    QIJNOS    -------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : QIJNOS
C     LEVEL    : 4 (CALLED BY 'XL3TRK')
C     USE      : THIS ROUTINE RECONSTRUCT COLLISION PROBABILITIES (CP)
C                WITH NO SELF-SCATTERING REDUCTION.
C     AUTHOR   : R. ROY (96-09-24)
C
C 2- PARAMETERS:
C  INPUT
C     NREG    : # OF ZONES FOR GEOMETRY.             I
C     NSNEG   : # OF SURFACES FOR GEOMETRY.          I
C     NPRB    : NUMBER OF PROBABILITIES IN PROB I
C     NGRP    : # OF GROUPS.                         I
C     SIGINV  : SIGT INVERSE VECTOR                  R(NREG)
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,*)
C     VOLUME  : VOLUME VECTOR.                       R(NREG)
C     ARRAY   : WORKING VECTOR.                      D(NREG)
C  OUTPUT
C     PROB    : -CP- MATRIX FOR ALL TYPES.           R(NGRP,*)
C----------------------------------------------------------------------
      IMPLICIT NONE
      INTEGER NREG,NSNEG,NGRP,NPRB
      INTEGER NSOUT,IPRB,INEW,IPAR,IG,IV,JV,NDEBV,I,J,IND,INDIJ
      REAL    SIGINV(NGRP,NREG),VOLUME(NREG)
      DOUBLE PRECISION PROB(NGRP,*),ARRAY(NREG)
      IND(I,J) = MAX(I,J)*(MAX(I,J)-1)/2 + MIN(I,J)
C----
C  STORE SYMMETRIC PVV AT BEGINNING OF WORK ARRAY
C----
      NSOUT=-NSNEG
      NDEBV=(NSOUT+1)*(NSOUT+2)/2+1
      IPAR= NDEBV
      INEW= 1
      DO 102 IV=1,NREG
        IPRB= IPAR+NSOUT+1
        DO 101 JV=1,IV
          DO 100 IG=1,NGRP
            PROB(IG,INEW)= PROB(IG,IPRB)*SIGINV(IG,IV)*SIGINV(IG,JV)
 100      CONTINUE
          IPRB= IPRB+1
          INEW= INEW+1
 101    CONTINUE
        IPAR= IPRB
 102  CONTINUE
C----
C  INVERT VOLUMES
C----
      DO 60 IV=1,NREG
        ARRAY(IV)= 1.0D0/DBLE(VOLUME(IV))
  60  CONTINUE
C----
C  STORE COMPLETE PVV AFTER UNFOLDING AND VOLUME DIVISION
C----
      DO 105 IV=1,NREG
        DO 104 JV=1,NREG
          INDIJ= IND(IV,JV)
          DO 103 IG=1,NGRP
            PROB(IG,INEW)= ARRAY(JV) * PROB(IG,INDIJ)
 103      CONTINUE
          INEW= INEW+1
 104    CONTINUE
 105  CONTINUE
C
      RETURN
      END
