*DECK SAD
      SUBROUTINE SAD(NENTRY,HENTRY,IENTRY,JENTRY,KENTRY)
*
*----------------------------SAD----------------------------------------
*
* 1- PROGRAMME STATISTICS:
*       NAME     : SAD
*       USE      : COMPUTATION OF GENERALIZED ADJOINT SOURCES & FLUXES
*       NOTES    : WHEN CORRECTION IS REQUIRED USING MOCC METHOD, 
*                  THE PERTURBED MACROLIB (IPLIBP) MUST BE INSERTED
*                  IN THE INPUT FILE (PUT AFTER THE UNPERTURBED LIB)
*       AUTHOR   : T. Courau 2000/08/01
*                  M. Assawaroongruengchot 2007/01/01
*       REMARK   : THIS ROUTINE IS INSPIRED FROM FLU AND EDI
* 2- ROUTINE PARAMETERS:
*     INPUT/OUTPUT PARAMETERS:
*      NENTRY : NUMBER OF LINKED LISTS OR FILES USED BY THE MODULE.
*      HENTRY : CHARACTER*12 NAME OF EACH LINKED LIST OR FILE.
*      IENTRY : 1 LINKED LIST; 2 XSM FILE;
*               3 SEQUENTIAL BINARY FILE; 4 SEQUENTIAL ASCII FILE.
*      JENTRY : 0 LINKED LIST OR FILE CREATED.
*               1 LINKED LIST OR FILE OPENED FOR MODIFICATIONS;
*               2 LINKED LIST OR FILE OPENED IN READ-ONLY MODE.
*      KENTRY : FILE UNIT NUMBER OR LINKED LIST ADDRESS
*               DIMENSION HENTRY(NENTRY),IENTRY(NENTRY),
*               JENTRY(NENTRY),KENTRY(NENTRY)
* 3- LINKED LIST / XSM FILE:  (ORDER IS ARBITRARY FOR 3,4,5)
*     HENTRY(1) : CREATE OR MODIFICATION TYPE(L_FLUX).
*     HENTRY(2) : CREATE OR MODIFICATION TYPE(L_EDIT)
*     HENTRY(3) : READ-ONLY TYPE(L_PIJ).
*     HENTRY(4) : READ-ONLY TYPE(L_MACROLIB OR L_LIBRARY).
*     HENTRY(5) : READ-ONLY TYPE(L_TRACK).
*     HENTRY(6) : READ-ONLY OPTIONAL BINARY TRACKING FILE.
*
*----------------------------SAD----------------------------------------
*
      IMPLICIT     NONE
      INTEGER      IOUT,NSTATE,NBEPS,ILCMUP,ILCMDN,MAXED
      CHARACTER    NAMSBR*6
      REAL         ROUMIN,RINMIN
      PARAMETER   (IOUT=6,NSTATE=40,NBEPS=5,ROUMIN=1.0E-5,RINMIN=5.0E-5,
     >             NAMSBR='SAD   ',ILCMUP=1,ILCMDN=2,MAXED=50)
C----
C  ROUTINE PARAMETERS
C----
      INTEGER      NENTRY
      CHARACTER    HENTRY(NENTRY)*12
      INTEGER      IENTRY(NENTRY),JENTRY(NENTRY),KENTRY(NENTRY)
C----
C  MEMORY ALLOCATION
C----
      INTEGER     IBASE(1)
      REAL        RBASE
      COMMON      RBASE(1)
      EQUIVALENCE(RBASE(1),IBASE(1))
      INTEGER     IMAT,IVOL,IKFLX,JFUNKN,JSUNKN,ISIGMA,JSOURC,
     >            JCORMA,JSFUNK,NFUNL,IXSFL,NAMXSF,INAMEA,INDXRA
C----
C  LOCAL PARAMETERS
C----
      INTEGER    IKSYS,IKTRK,IKLIB,IKBTF,IPSYS,IPTRK,IPLIB,IPBTF,
     >           IPFLUX,IPTMP,IPEDIT,ISTATE(NSTATE),KCHAR(18),
     >           ICOPAR(6),NREG,IPRINT,NGROUP,ILEAK,ITYPEC,ISADJ,
     >           NUNKNO,ITRANC,NIFISS,MXNL,NBMIX,IPHASE,ISCATR,
     >           ITPIJ,ITC,IEN,ILCMTY,ILCMLN,INITFL,KNORM,ITYLCM,
     >           ILNLCM
      CHARACTER  TEXT12*12,TITLE*72,CMODUL*12,HSIGN*12,
     >           CXDOOR*12,COPTIO*4,CERROR*131,CNAME*12,NAMREA*8
      LOGICAL    LEAKSW,LSAV,LCOR,LNORM,LALBS,LR2,LMODE(20)
      REAL       B2(4),EPSPAR(NBEPS)
      DOUBLE PRECISION REFKEF
      INTEGER    NALBP
      INTEGER    NNAM
C----
C CONDENSATION & HOMOGENIZATION  PARAMETERS
C----
      INTEGER     IENERG,IENERV,NTENER,NMERGE,NGCOND,IGCOND,
     >            IMERGE,IMIXME,NGCR,IGROUP,IREG,IECR,JPRINT,
     >            IGCR,JFLUXE,NEDMAC,FLUX
C----
C  DATA
C----
      CHARACTER   NAMGPA(2)*12,NAMGAS*12
      SAVE        NAMGPA
      DATA        NAMGPA
     >           /'PSEUDOGENADJ','FLUXGENADJ  '/
      DATA        NAMGAS
     >           /'FIXG      '/
      LOGICAL     LMOC,LEXIN,ILK,LSDINI,LOADFX
      INTEGER     IGP4,KTOPT,NGSS,NALB,
     >            ITRB,NL,NBN,NANIS,LCACA
      INTEGER  MAXM1,NWORDS
      INTEGER  IAJFLO,IFLUXO,VOLINV,
     >         DIFFB1,XSOLD,XSSIGT,XTINV,XSSIGW,XSCHI,XSNUF,XSTK,
     >         NJJ,IJJ,IPOS,XSCAT,VSCAT,ALBEDO
      REAL     RKEFF
C---  INITIALIZE MOC RELATED VARIABLES ---
      ITRB= 0
      NL=  -1
      NBN= -1
      LOADFX =.FALSE.
C----
C  PARAMETER VALIDATION
C----
      IF(NENTRY .LT. 5) CALL XABORT(NAMSBR//
     >  ': AT LEAST FIVE DATA STRUCTURES REQUIRED')
      IF(IENTRY(1) .NE. 1 .AND. IENTRY(1) .NE. 2) CALL XABORT(NAMSBR//
     >  ': FIRST DATA STRUCTURE NOT A LINKED LIST OR XSM FILE')
      IPFLUX=KENTRY(1)
      IF(JENTRY(1) .EQ. 0) THEN
        HSIGN='L_FLUX'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPFLUX,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(1) .EQ. 1) THEN
        LOADFX = .TRUE.
        CALL LCMGET(IPFLUX,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_FLUX') THEN
          CALL XABORT(NAMSBR//
     >      ': SIGNATURE OF '//HENTRY(1)//' IS '//HSIGN//
     >      '. L_FLUX EXPECTED.')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >    ': FLUX STRUCTURE IN READ-ONLY MODE FORBIDDEN')
      ENDIF
      IF(IENTRY(2) .NE. 1 .AND. IENTRY(2) .NE. 2) CALL XABORT(NAMSBR//
     >': LINKED LIST OR XSM FILE EXPECTED AT LHS.')
      IPEDIT=KENTRY(2)
      IF(JENTRY(2) .EQ. 0) THEN
        HSIGN='L_EDIT'
        READ(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        CALL LCMPUT(IPEDIT,'SIGNATURE',3,3,KCHAR)
      ELSE IF(JENTRY(2) .EQ. 1) THEN
        CALL LCMGET(IPEDIT,'SIGNATURE',KCHAR)
        WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
        IF(HSIGN .NE. 'L_EDIT') THEN
          CALL XABORT(NAMSBR//
     >      ': SIGNATURE OF '//HENTRY(2)//' IS '//HSIGN//
     >      '. L_EDIT EXPECTED.')
        ENDIF
      ELSE
        CALL XABORT(NAMSBR//
     >  ': EDIT STRUCTURE IN READ-ONLY MODE FORBIDDEN')
      ENDIF
      CALL LCMOP(IPTMP,'SOURCES-ADJ',0,1,0)
C----
C  SCAN READ-ONLY MODE DATA STRUCTURE
C  FOR PIJ, TRACK AND LIB/MAC AND BINARY TRACKING FILE
C----
      IKSYS=0
      IKTRK=0
      IKLIB=0
      IKBTF=0
      IPSYS=0
      IPTRK=0
      IPLIB=0
      IPBTF=0
      DO 100 IEN=3,NENTRY
        TEXT12=HENTRY(IEN)
        IF(JENTRY(IEN) .NE. 2) CALL XABORT(NAMSBR//
     >    ': DATA STRUCTURE '//TEXT12//' NOT IN READ-ONLY MODE')
        IF(IENTRY(IEN) .EQ. 1 .OR. IENTRY(IEN) .EQ. 2) THEN
          CALL LCMGET(KENTRY(IEN),'SIGNATURE',KCHAR)
          WRITE(HSIGN,'(3A4)') (KCHAR(ITC),ITC=1,3)
          IF(HSIGN .EQ. 'L_PIJ' .AND. IKSYS .EQ. 0) THEN
            IPSYS=KENTRY(IEN)
            IKSYS=IEN
          ELSE IF(HSIGN .EQ. 'L_TRACK' .AND. IKTRK .EQ. 0) THEN
            IPTRK=KENTRY(IEN)
            IKTRK=IEN
C         --- VERIFY THE UNPERTURBED MACRO/LIBRARY ---  
          ELSE IF(HSIGN .EQ. 'L_LIBRARY' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=IEN
          ELSE IF(HSIGN .EQ. 'L_MACROLIB' .AND. IKLIB .EQ. 0) THEN
            IPLIB=KENTRY(IEN)
            IKLIB=-IEN
          ENDIF
        ELSE IF(IENTRY(IEN) .EQ. 3 .AND. IKBTF .EQ. 0) THEN
C         --- VERIFY IFTRAK BINARY TRACKING FILE ---
          IPBTF=KENTRY(IEN)
          IKBTF=IEN
        ENDIF
 100  CONTINUE
C----
C  TEST FOR MISSING STRUCTURE (STRUCTURE ABSOLUTELY REQUIRED)
C----
C   DISABLE TO TAKE ACC OF MOCC  
C   BUT CHECK IKSYS=0 WITH LMOC BELOW ******   
C      IF(IKSYS .EQ. 0) CALL XABORT(NAMSBR//
C     >  ': NO DATA STRUCTURE WITH SIGNATURE L_PIJ FOUND.')
C
      IF(IKTRK .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE L_TRACK FOUND.')
      IF(IKLIB .EQ. 0) CALL XABORT(NAMSBR//
     >  ': NO DATA STRUCTURE WITH SIGNATURE '//
     >   'L_LIBRARY OR L_MACROLIB FOUND.')
C----
C  RECOVER IPTRK INFORMATION.
C----
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPTRK,'STATE-VECTOR',ISTATE)
      CALL LCMGET(IPTRK,'TRACK-TYPE  ',KCHAR)
      WRITE(CMODUL,'(3A4)') (KCHAR(ITC),ITC=1,3)
      NREG=ISTATE(1)
      IF(NREG .EQ. 0) CALL XABORT(NAMSBR//': NREG = 0')
      NUNKNO=ISTATE(2)
      LEAKSW=ISTATE(3).EQ.0
      IGP4=   ISTATE(4)
      KTOPT=  ISTATE(9)
      LCACA=  ISTATE(13)
      NGSS=   ISTATE(14)
      CALL SETARA(IBASE,NREG,IMAT)
      CALL SETARA(RBASE,NREG,IVOL)
      CALL SETARA(IBASE,NREG,IKFLX)
      CALL LCMGET(IPTRK,'MATCOD      ',IBASE(IMAT))
      CALL LCMGET(IPTRK,'VOLUME      ',RBASE(IVOL))
      CALL LCMGET(IPTRK,'KEYFLX      ',IBASE(IKFLX))
      CALL LCMLEN(IPTRK,'TITLE       ',ILCMLN,ILCMTY)
      IF(ILCMLN .GT. 0) THEN
        CALL LCMGET(IPTRK,'TITLE',KCHAR)
        WRITE(TITLE,'(18A4)') (KCHAR(ITC),ITC=1,18)
      ELSE
        TITLE='*** NO TITLE PROVIDED ***'
      ENDIF
C---  INITIALIZE VARIABLES FOR IPSYS
      LNORM=.FALSE.
      LALBS=.FALSE.
      LR2=.TRUE.
      IF (IKSYS .GT. 0 ) THEN
        CALL XDISET(ISTATE,NSTATE,0)
        CALL LCMGET(IPSYS,'STATE-VECTOR',ISTATE)
        IPHASE =ISTATE(1)
        ISCATR =ISTATE(2)
        KNORM  =ISTATE(5)
        NGROUP =ISTATE(7)
        IF(NGROUP .EQ. 0) CALL XABORT(NAMSBR//': NGROUP = 0.')
        NBMIX=ISTATE(9)
        IF(LEAKSW) THEN
          IF(ISTATE(4) .EQ. 0) LEAKSW=.FALSE.
        ENDIF
        IF(ISTATE(3) .EQ. 1) THEN
            LR2=.FALSE.
        ENDIF
        IF(ISTATE(4) .EQ. 1) THEN
            LNORM=.TRUE.
        ELSE IF(ISTATE(4) .EQ. 2) THEN
            LALBS=.TRUE.
        ENDIF
      ENDIF
C----
C  RECOVER MACROLIB INFORMATION.
C----
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB ,'MACROLIB',ILCMUP)
      CALL XDISET(ISTATE,NSTATE,0)
      CALL LCMGET(IPLIB,'STATE-VECTOR',ISTATE)
      IF(IKSYS.GT.0) THEN
         IF(NGROUP .NE. ISTATE(1)) CALL XABORT(NAMSBR//
     >     ': THE NUMBER OF GROUPS IN THE ASMPIJ'// 
     >     ' IS NOT EQUAL TO THE NUMBER OF GROUPS IN THE MACROLIB.')
         IF(NBMIX .NE. ISTATE(2)) CALL XABORT(NAMSBR//
     >     ': THE NUMBER OF MIXTURES IN THE ASMPIJ'// 
     >     ' IS NOT EQUAL TO THE NUMBER OF MIXTURES IN THE MACROLIB.')
      ENDIF
      NGROUP=ISTATE(1)
      NBMIX=ISTATE(2)
      MXNL=ISTATE(3)
      NIFISS=ISTATE(4)
      NEDMAC=ISTATE(5)
      ITRANC=ISTATE(6)
      NALBP =ISTATE(8)
      IF(IGP4 .GT. NBMIX) CALL XABORT(NAMSBR//
     >   ': THE NUMBER OF MIXTURES IN THE TRACKING'// 
     >   ' IS GREATER THAN THE NUMBER OF MIXTURES IN THE MACROLIB.')
C---  INITIALIZE PARAMETERS ---      
      NANIS=  ISTATE(3)-1
      CALL SETARA(RBASE,2*NGROUP+1,IENERG)
      CALL SETARA(RBASE,NGROUP,IENERV)
C----
C  READ GROUP STRUCTURE
C----
      CALL LCMLEN(IPLIB,'ENERGY',ILCMLN,ILCMTY)
      NTENER=ILCMLN
      IF(NTENER .EQ. NGROUP+1) THEN
        CALL LCMGET(IPLIB,'ENERGY',RBASE(IENERG))
      ELSE IF(NTENER .NE. 0) THEN
        CALL XABORT(NAMSBR//': INVALID NUMBER OF GROUP ON MACROLIB')
      ENDIF
C----
C  INITIALIZE VECTORS FOR READ
C----
      NMERGE=NREG
      NGCOND=NGROUP
      CALL SETARA(IBASE,NGROUP,IGCOND)
      CALL SETARA(IBASE,NREG,IMERGE)
      DO 110 IGROUP=1,NGROUP
        IBASE(IGCOND+IGROUP-1)=IGROUP
 110  CONTINUE
      DO 111 IREG=1,NREG
        IBASE(IMERGE+IREG-1)=IREG
 111  CONTINUE
      NGCR=0
C----
C  RECOVER FLUX INFORMATION
C----
      CXDOOR='TRAFIC'
      IF(IPHASE .EQ. -1) THEN
        CXDOOR=CMODUL
        ITPIJ=ISCATR+1
      ELSE
        ITPIJ=2*IPHASE+ISCATR+1
      ENDIF
C----
C  READ FLUX STATE VECTOR IF REQUIRED AND TEST FOR COMPATIBILITY
C----
      CALL XDISET(ISTATE,NSTATE,0)
      IF(JENTRY(1) .EQ. 1) THEN
        CALL LCMGET(IPFLUX,'STATE-VECTOR',ISTATE)
        ITYPEC=ISTATE(1)
        ILEAK=ISTATE(2)
        ICOPAR(1)=ISTATE(3)
        ICOPAR(2)=ISTATE(4)
        ICOPAR(3)=ISTATE(5)
        ICOPAR(4)=ISTATE(6)
        ICOPAR(5)=ISTATE(7)
        ICOPAR(6)=ISTATE(11)
        ISADJ=ISTATE(8)
        CALL LCMGET(IPFLUX,'EPS-CONVERGE',EPSPAR)
C----
C  USE FLUX FROM IPFLUX FOR INITIALIZATION OPTION ONLY IF
C  CURRENT FLUX DIMENSIONS COMPATIBLE WITH
C  DIMENSIONS PROVIDED BY SYSTEM MATRIX
C  OTHERWISE USE DEFAULT INITIALIZATION PROCEDURE
C----
        IF(NGROUP .EQ. ISTATE(9) .AND.
     >     NUNKNO .EQ. ISTATE(10) ) THEN
          INITFL=2
        ELSE
          INITFL=0
        ENDIF
      ELSE
        ITYPEC=-99
        ILEAK=0
        IF(ITPIJ .GE. 3) ILEAK=5
        ICOPAR(1)=3
        ICOPAR(2)=3
        ICOPAR(3)=1
        ICOPAR(4)=0
        ICOPAR(5)=0
        ICOPAR(6)=0
        ISADJ=0
        EPSPAR(1)=RINMIN
        EPSPAR(2)=ROUMIN
        EPSPAR(3)=0.0
        EPSPAR(4)=10.0
        EPSPAR(5)=-1.0
        INITFL=0
      ENDIF
C----
C  READ SAD EXECUTION OPTIONS
C----
      LSAV=.FALSE.
      LCOR=.TRUE.
      NNAM=NEDMAC+5
      CALL SETARA(IBASE,2*NNAM,INAMEA)
      CALL SETARA(IBASE,NNAM,IXSFL)
      CALL SETARA(IBASE,NNAM,INDXRA)
      CALL SETARA(IBASE,NGROUP+1,IGCR)
      CALL SETARA(RBASE,NGROUP+1,IECR)
      CALL XDISET(IBASE(IGCR),NGROUP+1,NGROUP)
      CALL XDRSET(RBASE(IECR),NGROUP+1,0.0)
      CALL SETARA(RBASE,NUNKNO*NGROUP,JFUNKN)
      CALL SETARA(RBASE,NUNKNO*NGROUP,JSUNKN)
      CALL SETARA(IBASE,NBMIX+1,IMIXME)
*----
*  INITIALIZE INAMEA
*----
      NAMREA='TOTAL   '
      READ(NAMREA,'(2A4)') IBASE(INAMEA),IBASE(INAMEA+1)
      NAMREA='NUSIGF  '
      READ(NAMREA,'(2A4)') IBASE(INAMEA+2),IBASE(INAMEA+3)
      NAMREA='CHI     '
      READ(NAMREA,'(2A4)') IBASE(INAMEA+4),IBASE(INAMEA+5)
      NAMREA='ABS     '
      READ(NAMREA,'(2A4)') IBASE(INAMEA+6),IBASE(INAMEA+7)
      NAMREA='SCAD    '
      READ(NAMREA,'(2A4)') IBASE(INAMEA+8),IBASE(INAMEA+9)
      IF(NEDMAC .GT. 0) THEN
        CALL LCMGET(IPLIB,'ADDXSNAME',IBASE(INAMEA+10))
      ENDIF
      CALL SADGET(IPFLUX,LEAKSW,NGROUP,NREG  ,NUNKNO,ITPIJ ,
     >            ITYPEC,ISADJ ,ICOPAR,EPSPAR,COPTIO,ILEAK ,
     >            IPRINT,INITFL,B2    ,REFKEF,NBMIX ,
     >            NNAM ,IBASE(IXSFL) ,IBASE(INAMEA),
     >            LSAV  ,LCOR  ,NGCR  ,NMERGE,
     >            IBASE(IKFLX) ,IBASE(IMAT)  ,RBASE(JFUNKN),
     >            IBASE(IMIXME),IBASE(IGCR)  ,RBASE(IECR)  ,
     >            IBASE(IMERGE),
     >            LMOC,KTOPT,ITRANC,LMODE,
     >            ITRB,NL,NBN,NANIS,LEXIN,ILK,LSDINI)
C---  VERIFY BINARY TRACKING FILE IF GENERALIZED ADJ MOCC SELECTED ---
      IF( IKBTF.EQ.0 .AND. LMOC) CALL XABORT(NAMSBR//
     > ': MISSING BINARY TRACKING FILE FOR MOC MODULE')
C--- VERIFY IPSYS IF GENERALIZED ADJOINT CP MODE SELECTED ---
      IF( IKSYS.EQ.0 .AND. (.NOT.LMOC)) CALL XABORT(NAMSBR//
     > ': MISSING ASMPIJ FILE FOR GENERALIZED ADJOINT CP CALCULATION')
      CALL RLSARA(IBASE(IMIXME))
      IF ( EPSPAR(5) .EQ. -1.0 ) THEN
        IF (ITYPEC .GE. 2  .OR. ITYPEC .LE. 4) THEN
              EPSPAR(5)=1.0E+2
        ELSE
              EPSPAR(5)=1.0E+5
        ENDIF
      ENDIF
C------
C CHECK PARAMETERS PREVIOUSLY READ
C------
      IF (      ILEAK  .GE. 5  .AND. ITYPEC .GE. 2
     >    .AND. ITYPEC .LE. 4  .AND. ITRANC .EQ. 0
     >    .AND. (COPTIO .EQ. 'B0TR' .OR. COPTIO .EQ. 'LKRD')) THEN
        CERROR=' '
        WRITE(CERROR,9000) NAMSBR,COPTIO
        CALL XABORT(CERROR)
      ENDIF
      IF( ITYPEC .GE. 2 .AND. ITYPEC .LE. 4 .AND.
     >    ILEAK  .GE. 5 .AND. NUNKNO .LT. 4*NREG ) CALL XABORT(NAMSBR//
     >                ': SET ANIS = 2 IN EXCELT: WITH PIJK CALCULATION')
C-----
C ITYPEC=1,2,3,4 (TYPE K OR B ONLY)
C-----
      IF (ITYPEC .LT. 1 .OR. ITYPEC .GT. 4) CALL XABORT(NAMSBR//
     >   ': TYPE K OR TYPE B ARE AVAILABLE')
*----
* ILEAK<=0,1,2,3 (NO ALBEDO OR HETEROGENEOUS COMPUTATION)
*----
      IF (ILEAK .LT. 0 .OR. ILEAK  .GT. 3 ) CALL XABORT(NAMSBR//
     >   ': THIS LEAKAGE CASE NOT AUTHORIZED')
      IF (ILEAK .GT. 1  .AND. ILEAK .NE. 3) THEN
       WRITE(IOUT,8000)
        ILEAK=3
      ENDIF
*----
*  INITIALIZE MOC VARIABLES 
*----
      NALB=   6
C---  UPLOAD ALBEDO PROPERTIES ----
      CALL SETARA(RBASE, NALB, ALBEDO )
      CALL LCMLEN(IPTRK,'ALBEDO',ILNLCM,ITYLCM)
      IF( ILNLCM.NE.NALB )THEN
        CALL XABORT(NAMSBR//
     >       'SAD : INCOMPATIBLE NUMBER OF ALBEDOS')
      ENDIF
      CALL LCMGET(IPTRK,'ALBEDO',RBASE(ALBEDO))
      MAXM1= NBMIX+1
C---  FOR REAL VARIABLE NWORDS=1 ----
      NWORDS= 1
      CALL SETARA(RBASE, NGROUP, DIFFB1 )
      CALL SETARA(RBASE, NREG, VOLINV )
C     --- CAL 1/VOL(IREGIO) ----
      DO 1000 IREG=1,NREG
         RBASE(VOLINV+IREG-1)= 1.0/RBASE(IVOL+IREG-1)
           IF(RBASE(IVOL+IREG-1).EQ. 0.0)  THEN
           CALL XABORT(NAMSBR//'SAD: VOL=0')
           ENDIF
 1000 CONTINUE
C     --- MEMORY ALLOCATION FOR DIRECT FLUX ARRAY ---
      CALL SETARA(RBASE,NUNKNO*NGROUP*NWORDS  , IFLUXO )
C     --- MEMORY ALLOCATION FOR ADJOINT FLUX ARRAY ---
      CALL SETARA(RBASE,1*NUNKNO*NGROUP*NWORDS, IAJFLO )
      CALL SETARA(RBASE, NGROUP*MAXM1, XSOLD  )
      CALL SETARA(RBASE, NGROUP*MAXM1, XSSIGT )
      CALL SETARA(RBASE, NGROUP*MAXM1, XTINV  )
      CALL SETARA(RBASE, NGROUP*MAXM1, XSSIGW )
      CALL SETARA(RBASE, MAXM1*NGROUP*NIFISS, XSCHI )
      CALL SETARA(RBASE, MAXM1*NGROUP*NIFISS, XSNUF )
      CALL SETARA(RBASE, NBMIX*NIFISS, XSTK  )
      CALL SETARA(IBASE, NBMIX, NJJ    )
      CALL SETARA(IBASE, NBMIX, IJJ    )
      CALL SETARA(IBASE, NBMIX, IPOS   )
      CALL SETARA(RBASE, MAXM1*NGROUP*NGROUP*(NANIS+1), XSCAT  )
      CALL SETARA(RBASE, NBMIX*NGROUP*NGROUP, VSCAT  )
      NFUNL= ((NL+1)*(NL+2))/2
      IF( NFUNL*NREG.GT.NUNKNO )THEN
         WRITE(IOUT,7001) NL,NFUNL,NREG,NUNKNO
         CALL XABORT(' SAD: INCOMPATIBLE DIMENSION FOR FLUX')
      ENDIF
C     -- CHANGE TO DOUBLE PRECISION ---
      CALL SETARA(RBASE,2* 8*NUNKNO*NGROUP, FLUX )
C----
C COMPUTE DIRECT AND ADJOINT FLUX
C----
      JPRINT=IPRINT
      IF (LCOR) THEN
       ISADJ=2
      ELSE
       ISADJ=1
      ENDIF
      IF (LSDINI) THEN
      CALL SADINI(IPFLUX,IPSYS ,IPLIB ,IPTRK ,IPBTF ,ITPIJ ,
     >            CXDOOR,COPTIO,JPRINT,INITFL,LEAKSW,ITYPEC,
     >            ISADJ ,ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX ,
     >            NREG  ,NUNKNO,NIFISS,MXNL  ,B2    ,REFKEF,
     >            TITLE ,NAMGPA,NAMGAS,
     >            RBASE(JFUNKN),RBASE(JSUNKN),IBASE(IMAT)  ,
     >            RBASE(IVOL)  ,IBASE(IKFLX)  ,
     >            LMOC  ,NGSS  ,ITRANC,ITRB  ,NL    ,NBN   ,NANIS ,
     >            LCACA ,LEXIN ,ILK   ,NALB  ,
     >            RBASE(IFLUXO),RBASE(IAJFLO),RBASE(DIFFB1),
     >            RBASE(VOLINV),RBASE(XSOLD) ,RBASE(XSSIGT),
     >            RBASE(XTINV) ,RBASE(XSSIGW),RBASE(XSCHI),
     >            RBASE(XSNUF) ,RBASE(XSTK)  ,RBASE(XSCAT),
     >            RBASE(VSCAT) ,RBASE(ALBEDO),IBASE(NJJ),
     >            IBASE(IJJ)   ,IBASE(IPOS)  ,RKEFF,
     >            LOADFX  ,LMODE    ,RBASE(FLUX) )
      ENDIF
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB ,'MACROLIB',ILCMDN)
C----
C  TEST ENERGY CONDENSATION INPUT
C----
      CALL EDIENE(NGROUP,NGCR  ,NGCOND,NTENER,
     >            IBASE(IGCR)  ,RBASE(IECR)  ,IBASE(IGCOND),
     >            RBASE(IENERG),RBASE(IENERV))
      CALL RLSARA(RBASE(IECR))
      CALL RLSARA(IBASE(IGCR))
C------
C STATE VECTOR PRINTING
C------
      CALL XDISET(ISTATE,NSTATE,0)
      ISTATE(1)=NMERGE
      ISTATE(2)=NGCOND
      ISTATE(10)=IPRINT
      ISTATE(11)=2
      ISTATE(14)=NGCOND
      ISTATE(15)=NMERGE
      IF (LCOR) THEN
       ISTATE(17)=1
      ELSE
       ISTATE(17)=0
      ENDIF
      CALL LCMPUT(IPEDIT,'STATE-VECTOR',NSTATE,1,ISTATE)
      CALL LCMPUT(IPEDIT,'REF:IMERGE',NREG,1,IBASE(IMERGE))
      CALL LCMPUT(IPEDIT,'REF:IGCOND',NGCOND,1,IBASE(IGCOND))
C----
C  Save SAD processing parameters on flux
C----
      CALL SADIDX(IPFLUX,IPLIB ,NGROUP,NIFISS,NNAM,
     >            IBASE(INAMEA),IBASE(IXSFL),IBASE(INDXRA))
       
C----
C COMPUTE GENERALIZED ADJOINT SOURCES AND FLUX NORMALISATION
C----
      CALL SETARA(RBASE,NREG*NGROUP,JFLUXE)
      CALL SADGAS(IPEDIT,IPTMP ,IPFLUX,IPLIB ,NGROUP,NBMIX ,
     >            NREG  ,NIFISS,MXNL  ,ITRANC,NGCOND,NMERGE,
     >            NTENER,NEDMAC,
     >            NNAM,IBASE(INAMEA),IBASE(IXSFL),IBASE(INDXRA),
     >            IBASE(IMAT)  ,RBASE(IVOL)  ,IBASE(IKFLX) ,
     >            IBASE(IGCOND),IBASE(IMERGE),RBASE(JFLUXE))
      CALL RLSARA(RBASE(JFLUXE))
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB ,'MACROLIB',ILCMUP)
C----
C COMPUTE CORRECTION DUE TO HIGH ORDER FLUX COMPONENT
C----
      IF ( LCOR ) THEN
       CNAME=' '
       CALL SETARA(RBASE,2*NBMIX,ISIGMA)
       CALL SETARA(RBASE,NUNKNO,JSOURC)
       CALL SETARA(RBASE,NREG*NBMIX*NGROUP,JCORMA)
       CALL SETARA(RBASE,NUNKNO*NGROUP,JSFUNK)
       IF (LMOC) THEN
      CALL SADMCL(IPFLUX,IPLIB ,IPTRK ,IPBTF,NUNKNO,IPRINT,NFUNL ,
     >            NGROUP,NIFISS,NBMIX ,NREG ,MXNL  ,LEAKSW,ITRANC,
     >            TITLE ,ITPIJ ,IBASE(IMAT),RBASE(IVOL),
     >            IBASE(IKFLX),RBASE(JFUNKN),RBASE(JSUNKN),
     >            RBASE(ISIGMA),RBASE(JCORMA),
     >            ITYPEC,ILEAK ,ICOPAR,EPSPAR,
     >            LMOC  ,NGSS  ,ITRB  ,NL    ,NBN   ,
     >            NANIS ,LCACA ,LEXIN ,ILK   ,NALB  ,RBASE(IFLUXO),
     >            RBASE(IAJFLO),RBASE(DIFFB1),RBASE(VOLINV),
     >            RBASE(XSOLD) ,RBASE(XSSIGT),RBASE(XTINV),
     >            RBASE(XSSIGW),RBASE(XSCHI) ,RBASE(XSNUF),
     >            RBASE(XSTK)  ,RBASE(XSCAT) ,RBASE(VSCAT),
     >            RBASE(ALBEDO),IBASE(NJJ)   ,IBASE(IJJ), 
     >            IBASE(IPOS)  ,LMODE ,RBASE(FLUX))
       ELSE
       CALL SADCOR(IPFLUX,IPSYS ,IPLIB ,IPTRK ,IPBTF,
     >             NUNKNO,CMODUL,IPRINT,NGROUP,NIFISS,
     >             NBMIX ,NREG  ,MXNL  ,LEAKSW,ITRANC,
     >             TITLE ,LNORM ,LALBS ,LR2   ,IPHASE,
     >             ISCATR,KNORM ,CNAME ,ITPIJ ,IBASE(IMAT)  ,
     >             RBASE(IVOL)  ,IBASE(IKFLX) ,RBASE(JFUNKN),
     >             RBASE(JSUNKN),RBASE(JSOURC),RBASE(ISIGMA),
     >             RBASE(JCORMA),RBASE(JSFUNK),
     >             NALBP)
       ENDIF
       CALL RLSARA(RBASE(JSFUNK))
       CALL RLSARA(RBASE(JCORMA))
       CALL RLSARA(RBASE(JSOURC))
       CALL RLSARA(RBASE(ISIGMA))
       ISADJ=4
      ELSE
       ISADJ=4
      ENDIF
C----
C COMPUTE GENERALIZED ADJOINT FLUXES
C----
      CALL SADFLU(IPFLUX,IPSYS ,IPTMP,IPLIB,IPTRK,IPBTF,ITPIJ,
     >            CXDOOR,COPTIO,IPRINT,LEAKSW,ITYPEC,ISADJ,
     >            ILEAK ,ICOPAR,EPSPAR,NGROUP,NBMIX,NREG,NMERGE,
     >            NGCOND,NUNKNO,NIFISS,MXNL,INITFL,
     >            B2,REFKEF,TITLE,NAMGPA,NAMGAS,
     >            NNAM,IBASE(INAMEA),IBASE(IXSFL),
     >            RBASE(JFUNKN),RBASE(JSUNKN),IBASE(IMAT) ,
     >            RBASE(IVOL)  ,IBASE(IKFLX),IBASE(IMERGE),
     >            IBASE(IGCOND),
     >            LMOC  ,NGSS  ,ITRANC,ITRB  ,NL    ,NBN   ,NANIS ,
     >            LCACA ,LEXIN ,ILK   ,NALB  ,
     >            RBASE(IFLUXO),RBASE(IAJFLO),RBASE(DIFFB1),
     >            RBASE(VOLINV),RBASE(XSOLD) ,RBASE(XSSIGT),
     >            RBASE(XTINV) ,RBASE(XSSIGW),RBASE(XSCHI),
     >            RBASE(XSNUF) ,RBASE(XSTK)  ,RBASE(XSCAT),
     >            RBASE(VSCAT) ,RBASE(ALBEDO),IBASE(NJJ),
     >            IBASE(IJJ)   ,IBASE(IPOS),RKEFF,
     >            IKLIB ,LOADFX,LMODE,RBASE(FLUX)  )
      IF(IKLIB .GT. 0) CALL LCMSIX(IPLIB ,'MACROLIB',ILCMDN)
C----
C  RELEASE MOC VARIABLES
C----
      CALL RLSARA( RBASE(FLUX)  )
      CALL RLSARA( RBASE(VSCAT ) )
      CALL RLSARA( RBASE(XSCAT ) )
      CALL RLSARA( IBASE(IPOS  ) )
      CALL RLSARA( IBASE(IJJ   ) )
      CALL RLSARA( IBASE(NJJ   ) )
      CALL RLSARA( RBASE(XSTK  ) )
      CALL RLSARA( RBASE(XSNUF ) )
      CALL RLSARA( RBASE(XSCHI ) )
      CALL RLSARA( RBASE(XSSIGW) )
      CALL RLSARA( RBASE(XTINV ) )
      CALL RLSARA( RBASE(XSSIGT) )
      CALL RLSARA( RBASE(XSOLD ) )
      CALL RLSARA( RBASE(IAJFLO) )
      CALL RLSARA( RBASE(IFLUXO) )
      CALL RLSARA( RBASE(VOLINV) )
      CALL RLSARA( RBASE(DIFFB1) )
      CALL RLSARA( RBASE(ALBEDO) )
*
      CALL RLSARA(RBASE(JFUNKN))
      CALL RLSARA(RBASE(JSUNKN))
C----
C  RELEASE CONDENSATION/MERGE VECTOR
C----
      CALL RLSARA(IBASE(INDXRA))
      CALL RLSARA(IBASE(IXSFL))
      CALL RLSARA(IBASE(INAMEA))
      CALL RLSARA(IBASE(IMERGE))
      CALL RLSARA(IBASE(IGCOND))
      CALL RLSARA(RBASE(IENERV))
      CALL RLSARA(RBASE(IENERG))
C----
C  RELEASE GENERAL TRACKING INFORMATION.
C----
      CALL RLSARA(IBASE(IKFLX))
      CALL RLSARA(RBASE(IVOL))
      CALL RLSARA(IBASE(IMAT))
C----
C SAVE COMPRESSED GENERALIZED ADJOINT
C SOURCES IF REQUIRED
C---
      IF (LSAV) THEN
       CALL LCMSIX(IPEDIT,'ADJOINT-SCES',ILCMUP)
       CALL LCMEQU(IPTMP,IPEDIT)
       CALL LCMSIX(IPEDIT,'ADJOINT-SCES',ILCMDN)
      ENDIF
      CALL LCMCL(IPTMP,2)
      RETURN
C----
C  ERROR FORMAT
C----
 7001 FORMAT(1X,'SAD: NL   =',I10/
     >       1X,'SAD: NFUNL=',I10/
     >       1X,'SAD: NREG =',I10/
     >       1X,'SAD: NUN  =',I10)
 8000 FORMAT(1X,'*** WARNING *** BAD BUCKLING TYPE ',
     >       /1X,'AUTOMATICALLY SET TO TYPE B SIGS')
 9000 FORMAT(A6,': ILLEGAL OPTION ',A4,
     >' FOR HETEROGENEOUS LEAKAGE WITHOUT TRANSPORT CORRECTION')
      END
