*DECK SADPSA
      SUBROUTINE  SADPSA(IPTMP ,NGCOND,NMERGE,NL    ,NREGIO,
     >                   NGROUP,ITRANC,NBMIX ,NIFISS,NEDMAC,
     >                   NTAUXT,NNAM  ,INAMEA,IXSFL ,INDXRA,
     >                   IMERGE,IGCOND,MATCOD,RATECM,SCATTD,SADXS ,
     >                   XSSCAT,FLAGMX,PCHI  ,PFIS  ,CHISUM,SASCAT,
     >                   DASCAT)
C
C
C---------------------------  SADPSA  ---------------------------------
C
C  1- PROGRAMME STATISTICS:
C      NAME     : SADPSA
C      USE      : EVALUATE & PRINT GENERALIZED ADJOINT SOURCES
C      AUTHOR   : T. COURAU 2000/08/01
C      REMARK   : THIS ROUTINE IS INSPIRED FROM EDIPXS
C
C  2- ROUTINE PARAMETERS:
C    INPUT / OUTPUT
C      IPTMP   : TEMPORARY POINTER TO THE GENERALIZED   I
C                ADJOINT SOURCES
C      NGCOND   : NUMBER OF GROUPS CONDENSED            I
C      NMERGE   : NUMBER OF REGIONS MERGED              I
C      NL       : NUMBER OF LEGENDRE ORDERS             I
C      NREGIO   : NUMBER OF REGIONS                     I
C      NGROUP   : NUMBER OF GROUPS                      I
C      ITRANC   : TYPE OF TRANSPORT CORRECTION          I
C      NBMIX    : NUMBER OF MIXTURES                    I
C      NIFISS   : NUMBER OF FISSILE ISOTOPES            I
C      NEDMAC   : NUMBER OF ADITIONAL EDITION X-S       I
C      NTAUXT   : NUMBER OF REACTION RATE EDITS         I
C      NNAM     : NUMBER OF STANDARD XS TREATED         I
C      INAMEA   : NAME OF THE XS TO BE TREATED          I(2,NNAM)
C      IXSFL    : FLAG FOR THE XS TO BE TREATED         I(NNAM)
C      INDXRA   : Index for generalized adjoint and     I(NNAM)
C                 and reaction rates 
C      IMERGE   : INDEX OF MERGED REGION                I(NREGIO)
C      IGCOND   : LIMIT CONDENSED GROUPS                I(NGCOND)
C      MATCOD   : MATERIAL PER REGION                   I(NREGIO)
C      RATECM   : AVERAGED REGION/GROUP X-S             D(0:NMERGE,
C                 = RATECM(*,1) = FLUXES                  NGCOND,NTAUXT)
C                 = RATECM(*,2) = TOTAL
C                 = RATECM(*,3) = ABS
C                 = RATECM(*,4) = FISSIONS
C                 = RATECM(*,5) = PRODUCTIONS
C                 = RATECM(*,6) = TOTAL LEAKAGES
C                 = RATECM(*,7) = TOTAL OUT OF GROUP SCATTERING
C                 = RATECM(*,8) = DIAGONAL SCATTERING X-S
C                 = RATECM(*,9) = CHI
C                 = RATECM(*,10) = WIMS TYPE TRANSPORT CORRECTION
C                 = RATECM(*,11) = X-DIRECTED LEAKAGES
C                 = RATECM(*,12) = Y-DIRECTED LEAKAGES
C                 = RATECM(*,13) = Z-DIRECTED LEAKAGES
C                 = RATECM(*,*,13+JJ) NUSIGF FOR JJ
C                 = RATECM(*,*,13+NIFISS+JJ) CHI FOR JJ
C                 = RATECM(*,*,13+2*NIFISS+IL) SIGS FOR IL
C                 = RATECM(*,*,13+2*NIFISS+NL+IE) EDIT IE
C                 = RATECM(*,*,NTAUXT)= 1/V MERGE CONDENSED
C      SCATTD   : DOUBLE PRECISION SCATTERING RATES D(NMERGE,NGCOND,
C                                                     NGCOND,NL)
C      SADXS    : GENERALIZED ADJOINT SOURCE        R(0:NBMIX,
C                                                     NGROUP,NNAM)
C                            FOR STANDARD XS
C      XSSCAT   : SCATTERING MATRIX                 R(NGROUP,NGROUP,
C                                                     NBMIX,NL)
C    WORK
C      FLAGMX   : CHECK IF A MIX HAS BEEN TREATED   I(0:NBMIX)
C      PCHI     : STORE SPECTRUM                    R(NBMIX,NIFISS,
C                                                     NGROUP)
C      PFIS     : STORE FISSION XS                  R(NBMIX,NIFISS,
C                                                     NGROUP)
C      CHISUM   : STORE CONDENSED SPECTRUM          D(NIFISS)
C      SASCAT   : GENERALIZED ADJOINT SOURCE        R(0:NBMIX,NGCOND,
C                 FOR SCATTERING XS                   NGROUP,NL)
C      DASCAT   : SCATTERING XS                     D(0:NBMIX,NGCOND,
C                                                   NGROUP,NL)
C
C  3- INTERNAL PARAMETERS:
C
C      IOUT    : OUTPUT UNIT = 6                    I
C      CGRPNM  : GROUP INDEX NAME FOR LCM           C*12
C
C---------------------------   SADPSA  --------------------------------
C
      IMPLICIT          NONE
      INTEGER           IOUT,ILCMUP,ILCMDN
      CHARACTER         NAMSBR
      PARAMETER        (IOUT=6,ILCMUP=1,ILCMDN=2,NAMSBR='SADPSA')
C----
C  ROUTINE PARAMETERS
C----
      INTEGER           IPTMP,NGCOND,NMERGE,NL,NREGIO,NGROUP,ITRANC,
     >                  NBMIX,NIFISS,NEDMAC,NTAUXT,NNAM
      INTEGER           IXSFL(NNAM),INAMEA(2,NNAM),INDXRA(NNAM)
      INTEGER           IMERGE(NREGIO),IGCOND(NGCOND),
     >                  MATCOD(NREGIO),FLAGMX(0:NBMIX)
      REAL              XSSCAT(NBMIX,NGROUP,NGROUP,NL),
     >                  SADXS(0:NBMIX,NGROUP,NNAM),
     >                  PCHI(NBMIX,NIFISS,NGROUP),
     >                  PFIS(NBMIX,NIFISS,NGROUP),
     >                  SASCAT(0:NBMIX,NGCOND,NGROUP,NL)
      DOUBLE PRECISION  RATECM(0:NMERGE,NGCOND,NTAUXT),
     >                  SCATTD(NMERGE,NGCOND,NGCOND,NL),CHISUM(NIFISS),
     >                  DASCAT(NBMIX,NGCOND,NGROUP,NL)
C----
C  LOCAL PARAMETERS
C----
      CHARACTER         CM*2,CGRPNM*12,IGCAR*2
      INTEGER           INAM,IGROUP,
     >                  IMIX,IFIS,IMRG,IREG,IGRC,IGRFIN,IGRDEB,
     >                  FL,IGR,IL,JGRC,JGROUP,JGRFIN,JGRDEB
      DOUBLE PRECISION  DFACT,ACCUMU
      CHARACTER         NAMREA*8
      INTEGER           IDRAT
C-----
C ADJOINT SOURCE COMPUTING
C-----
      DO  INAM=1,NNAM
        IF(IXSFL(INAM).EQ.2) THEN
          IDRAT=INDXRA(INAM)
          IGRFIN=0
          DO IGRC=1,NGCOND
            IGRDEB=IGRFIN+1
            IGRFIN=IGCOND(IGRC)
            DO IGROUP=IGRDEB,IGRFIN
              CALL XDISET(FLAGMX,NBMIX+1,0)
              DO IMRG=1,NMERGE
                DO IREG=1,NREGIO
                  IMIX=MATCOD(IREG)
                  FL=FLAGMX(IMIX)
                  IF(IMERGE(IREG).EQ.IMRG.AND.FL.EQ.0) THEN
                    FLAGMX(IMIX)=1
                    IF (INAM.EQ.3) THEN
C----
C FISSION SPECTRUM
C----
                      IF(RATECM(IMRG,IGRC,IDRAT).GT.0.0) THEN
                        DFACT=1.0/RATECM(IMRG,IGRC,4)
                        DO IFIS=1,NIFISS
                          CHISUM(IFIS)=0.D0
                          DO IGR=IGRDEB,IGRFIN
                            CHISUM(IFIS)=CHISUM(IFIS)
     >                      +PCHI(IMIX,IFIS,IGR)
                          ENDDO
                        ENDDO
                        ACCUMU=0.D0
                        DO IFIS=1,NIFISS
                          ACCUMU=ACCUMU
     >                    +CHISUM(IFIS)*PFIS(IMIX,IFIS,IGROUP)
                        ENDDO
                        SADXS(IMIX,IGROUP,INAM)=DFACT*(ACCUMU-
     >                  RATECM(IMRG,IGRC,9)*SADXS(IMIX,IGROUP,3))
                      ENDIF
                    ELSE IF(INAM.EQ.2) THEN
C----
C NUSIGF XS
C----
                      ACCUMU=0.D0
                      DO IFIS=1,NIFISS
                        ACCUMU=ACCUMU+RATECM(IMRG,IGRC,IDRAT-1+IFIS)
                      ENDDO
                      DFACT=1.0/RATECM(IMRG,IGRC,1)
                      SADXS(IMIX,IGROUP,INAM)=
     >                         DFACT*SADXS(IMIX,IGROUP,INAM)
     >                        -DFACT**2*ACCUMU
                    ELSE
C----
C STANDARD XS
C----
                      DFACT=1.0/RATECM(IMRG,IGRC,1)
                      SADXS(IMIX,IGROUP,INAM)=
     >                         DFACT*SADXS(IMIX,IGROUP,INAM)
     >                        -DFACT**2*RATECM(IMRG,IGRC,IDRAT)
                    ENDIF
                  ENDIF
                ENDDO
              ENDDO
            ENDDO
          ENDDO
        ENDIF
      ENDDO
C----
C Default adjoint sources
C----
      DO INAM=1,4
        IF(IXSFL(INAM).EQ.2) THEN
          WRITE(NAMREA,'(2A4)') INAMEA(1,INAM),INAMEA(2,INAM)
          DO IGROUP=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
            CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
            CALL LCMPUT(IPTMP,NAMREA,NBMIX+1,2,SADXS(0,IGROUP,INAM))
            CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
          ENDDO
        ENDIF
      ENDDO
C---
C Scattering adjoint sources
C---
      IF(IXSFL(5).EQ.2) THEN
        IL=1
        CALL XDRSET(SASCAT,(NBMIX+1)*NGCOND*NGROUP*NL,0.0)
        CALL XDDSET(DASCAT,NBMIX*NGCOND*NGROUP*NL,0.0D0)
        DO JGROUP=1,NGROUP
          DO IMIX=1,NBMIX
            IGRFIN=0
            DO IGRC=1,NGCOND
              IGRDEB=IGRFIN+1
              IGRFIN=IGCOND(IGRC)
              ACCUMU=0.D0
              DO IGROUP=IGRDEB,IGRFIN
                ACCUMU=ACCUMU+XSSCAT(IMIX,IGROUP,JGROUP,IL)
              ENDDO
              DASCAT(IMIX,IGRC,JGROUP,IL)=ACCUMU
            ENDDO
          ENDDO
        ENDDO
C---
C SCATTERING ADJOINT SOURCE COMPUTATION LOOP
C---
        DO IGRC=1,NGCOND
          CALL XDISET(FLAGMX,NBMIX+1,0)
          DO IMRG=1,NMERGE
            DO IREG=1,NREGIO
              IMIX=MATCOD(IREG)
              FL=FLAGMX(IMIX)
              IF(IMERGE(IREG).EQ.IMRG.AND.FL.EQ.0) THEN
                FLAGMX(IMIX)=1
                JGRFIN=0
                IF(IMIX.GT.0) THEN
                  DO JGRC=1,NGCOND
                    JGRDEB=JGRFIN+1
                    JGRFIN=IGCOND(JGRC)
                    DFACT=1.0/RATECM(IMRG,JGRC,1)
                    DO JGROUP=JGRDEB,JGRFIN
                      SASCAT(IMIX,IGRC,JGROUP,IL)=
     >                DFACT*DASCAT(IMIX,IGRC,JGROUP,IL)
     >               -DFACT**2*SCATTD(IMRG,IGRC,JGRC,IL)
                    ENDDO
                  ENDDO
                ELSEIF(IMIX.EQ.0) THEN
                  DO JGRC=1,NGCOND
                    JGRDEB=JGRFIN+1
                    JGRFIN=IGCOND(JGRC)
                    DFACT=1.0/RATECM(IMRG,JGRC,1)
                    DO JGROUP=JGRDEB,JGRFIN
                      SASCAT(IMIX,IGRC,JGROUP,IL)=
     >               -DFACT**2*SCATTD(IMRG,IGRC,JGRC,IL)
                    ENDDO
                  ENDDO
                ENDIF
              ENDIF
            ENDDO
          ENDDO
        ENDDO
C------
C COMPRESSED SCATTERING ADJOINT SOURCE PRINTING
C------
        DO  IGRC=1,NGCOND
          WRITE (IGCAR,'(I2)') IGRC
          WRITE (CM,'(I2)') IL-1
          DO JGROUP=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') JGROUP,NGROUP
            CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
            CALL LCMPUT(IPTMP,'SCAD'//CM//IGCAR,NBMIX+1,2,
     >                  SASCAT(0,IGRC,JGROUP,IL))
            CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
          ENDDO
        ENDDO
      ENDIF
C----
C Additional adjoint sources
C----
      DO INAM=6,NNAM
        IF(IXSFL(INAM).EQ.2) THEN
          WRITE(NAMREA,'(2A4)') INAMEA(1,INAM),INAMEA(2,INAM)
          DO IGROUP=1,NGROUP
            WRITE(CGRPNM,'(5HGROUP,I3,1H/,I3)') IGROUP,NGROUP
            CALL LCMSIX(IPTMP,CGRPNM,ILCMUP)
            CALL LCMPUT(IPTMP,NAMREA,NBMIX+1,2,SADXS(0,IGROUP,INAM))
            CALL LCMSIX(IPTMP,CGRPNM,ILCMDN)
          ENDDO
        ENDIF
      ENDDO
      RETURN
      END
