*DECK SHELL2
      SUBROUTINE SHELL2 (NGPT,RAYRE,SIGT,PETIT,PVV,PVS,PSS)
*
*-----------------------------------------------------------------------
*
*   INTEGRATION OF THE COLLISION PROBABILITIES OF AN HOMOGENEOUS 1-D
*   SPHERE OR SPHERICAL SHELL (DP-1 SURFACE ANGULAR FLUX APPROXIMATION)
*
*   INPUT PARAMETERS.
*   NGPT    : NUMBER OF GAUSS-JACOBI INTEGRATION POINTS.
*   RAYRE   : RADIUS OF SHELL.DIMENSION RAYRE(2)
*   SIGT    : CROSS SECTION.
*   PETIT   : VOIDED BLOCK CRITERION.
*
*   OUTPUT PARAMETERS.
*   PVV     : VOLUME TO VOLUME REDUCED PROBABILITY.
*   PVS     : VOLUME TO SURFACE PROBABILITIES.
*             DIMENSION PVS(2) (SPHERE) OR PVS(4) (SHELL)
*   PSS     : SURFACE TO SURFACE PROBABILITIES IN THE FOLLOWING ORDER:
*             1 TO 1, 2 TO 1, 3 TO 1, 4 TO 1, 1 TO 2, 2 TO 2, ETC.
*             WHERE 1 AND 2 ARE THE INTERNAL SURFACES;
*                   3 AND 4 ARE THE EXTERNAL SURFACES.
*             DIMENSION PSS(4) (SPHERE) OR PSS(16) (SHELL).
*
*----------------------------------------------------- M-L VERGAIN -----
*
      PARAMETER (PI=3.1415926535,MGPT=64)
      LOGICAL FILLED
      DIMENSION RAYRE(2),PVS(*),PSS(*),ZJKSI(MGPT),WJKSI(MGPT),
     >          SEGL(6*MGPT),AEGL(2*MGPT),POSE(7)
*
      CALL ALGJP(NGPT,ZJKSI,WJKSI)
      RAI=RAYRE(1)
      RAA=RAYRE(2)
      FILLED=(RAI.EQ.0.0)
      SS1=RAI/RAA
      SS2=SS1*SS1
      SS3=SS2*SS1
      FP2=1.0-SS2
      SP2=SQRT(FP2)
      SQ=SQRT(2.0)
      IOS=1
      IL=1
      IF (FILLED) THEN
         DO 10 I=1,2
   10    PVS(I)=0.0
         DO 20 I=1,4
   20    PSS(I)=0.0
         DO 30 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         AEGL(IL)=2.0*RAA*ZJKSI(J)
         SEGL(IOS)=2.0*WJKSI(J)
         SEGL(IOS+1)=2.0*WJKSI(J)*ZJKSI(J)
         SEGL(IOS+2)=2.0*WJKSI(J)*XZ2
         IL=IL+1
   30    IOS=IOS+3
      ELSE
         DO 40 I=1,4
   40    PVS(I)=0.0
         DO 50 I=1,16
   50    PSS(I)=0.0
         DO 120 J=1,NGPT
         XZ2=ZJKSI(J)*ZJKSI(J)
         AEGL(IL)=RAA*(SQRT(1.0-SS2*XZ2)-SS1*SQRT(1.0-XZ2))
         AEGL(IL+1)=2.0*RAA*SP2*ZJKSI(J)
         SEGL(IOS)=2.0*WJKSI(J)
         SEGL(IOS+1)=2.0*WJKSI(J)*SQRT(1.0-SS2*XZ2)
         SEGL(IOS+2)=2.0*WJKSI(J)*SQRT(1.0-XZ2)
         SEGL(IOS+3)=2.0*WJKSI(J)*SQRT(1.0-XZ2)*SQRT(1.0-SS2*XZ2)
         SEGL(IOS+4)=2.0*WJKSI(J)*FP2*SP2*ZJKSI(J)
         SEGL(IOS+5)=2.0*WJKSI(J)*XZ2*FP2*FP2
         IL=IL+2
  120    IOS=IOS+6
      ENDIF
*
      DO 130 IN=1,7
  130 POSE(IN)=0.0
      SIGR=SIGT*RAA
      SIGXX=4.0*SIGR*(1.0-SS3)/(3.0*(1.0+SS2))
*
      IF (SIGXX.GE.PETIT) THEN
*         NON-VOID SPHERICAL REGION
          IL=1
          IOS=1
          IF (FILLED) THEN
*            INTEGRATE SPHERE PSS
             DO 210 IPT=1,NGPT
             XE=AEGL(IL)*SIGT
             POSE(1)=POSE(1)+SEGL(IOS)*EXP(-XE)
             POSE(2)=POSE(2)+SEGL(IOS+1)*EXP(-XE)
             POSE(3)=POSE(3)+SEGL(IOS+2)*EXP(-XE)
             IL=IL+1
  210        IOS=IOS+3
*
*-----       COMPUTE PVS,PVV
*
             PSS(1)=POSE(1)
             PSS(2)=3.0*SQ*POSE(2) - 2.0*SQ*POSE(1)
             PSS(4)=18.0*POSE(3) - 24.0*POSE(2) + 8.0*POSE(1)
             PSS(3)=PSS(2)
             PVS(1)=(1.0-PSS(1))*3.0/(4.0*SIGR)
             PVS(2)=-PSS(2)*3.0/(4.0*SIGR)
             PVV=(1.0-PVS(1))/SIGT
          ELSE
*            INTEGRATE SHELL PROBABILITIES
             PVOL=4.0*(1.0-SS3)*SIGR/3.0
             SV1=SS2/PVOL
             SV2=1.0/PVOL
             DO 220 IPT=1,NGPT
             XE=AEGL(IL)*SIGT
             POSE(1)=POSE(1)+SEGL(IOS)*EXP(-XE)
             POSE(2)=POSE(2)+SEGL(IOS+1)*EXP(-XE)
             POSE(3)=POSE(3)+SEGL(IOS+2)*EXP(-XE)
             POSE(4)=POSE(4)+SEGL(IOS+3)*EXP(-XE)
             XI=AEGL(IL+1)*SIGT
             POSE(5)=POSE(5)+SEGL(IOS)*FP2*EXP(-XI)
             POSE(6)=POSE(6)+SEGL(IOS+4)*EXP(-XI)
             POSE(7)=POSE(7)+SEGL(IOS+5)*EXP(-XI)
             IL=IL+2
  220        IOS=IOS+6
*
*------      COMPUTE PVS,PVV SHELL
*
             PSS(9)=POSE(1)
             PSS(13)=3.0*SQ*POSE(2) - 2.0*SQ*POSE(1)
             PSS(10)=3.0*SQ*POSE(3) - 2.0*SQ*POSE(1)
             PSS(14)=18.0*POSE(4) - 12.0*(POSE(2)+POSE(3))+8.0*POSE(1)
             PSS(11)=POSE(5)
             PSS(15)=3.0*SQ*POSE(6) - 2.0*SQ*POSE(5)
             PSS(16)=18.0*POSE(7) - 24.0*POSE(6) + 8.0*POSE(5)
             PSS(12)=PSS(15)
             PSS(3)=SS2*PSS(9)
             PSS(4)=SS2*PSS(13)
             PSS(7)=SS2*PSS(10)
             PSS(8)=SS2*PSS(14)
             PVS(3)=(1.0-PSS(11)-PSS(3))*SV2
             PVS(1)=(1.0-PSS(9))*SV1
             PVS(2)=-SV1*PSS(10)
             PVS(4)=-SV2*(PSS(4)+PSS(15))
             PVV=(1.0-PVS(3)-PVS(1))/SIGT
           ENDIF
      ELSE
*        COMPUTE PVV FROM SERIES EXPANSION OF EXP
         SGR2=SIGR*SIGR
         IF (FILLED) THEN
*           TREAT FILLED SPHERE
            PSS(1)=1.0-4.0*SIGR/3.0+SGR2
            PSS(2)=SQ*(-SIGR/3.0+2.0*SGR2/5.0)
            PSS(4)=1.0-SIGR*304.0/15.0+SGR2*4.0/5.0
            PSS(3)=PSS(2)
            PVS(2)=SQ*(1.0-6.0*SIGR/5.0)/4.0
            PVS(1)=1.0-SIGR*0.75
            PVV=RAA*0.75
         ELSE
*           TREAT SHELLS
*
            SS4=SS1**4
            SS5=SS1**5
            DE2=FP2**1.5
            DE3=FP2**2.5
            ASV=(1.0-DE2)/SS2
            AVV=(1.0+SS2-ALOG((1.0+SS1)/SP2)*FP2*FP2/SS1)
     >           /(SS1*8.0)
            ATV=(3.0*DE3-5.0*DE2+2.0)/SS2
            AUV=(2.0*DE3+5.0*SS2-2.0)/SS3
            ARV=(SS4+SS2*8.0/3.0-1.0+ALOG((1.0+SS1)/SP2)
     >           *(FP2**3.0)/SS1)/(16.0*SS2)
            AXX=ASV*2.0/3.0-SIGR*(1.0-SS2*0.5-2.0*AVV)+SGR2*
     >         (5.0*(1.0+SS2)*ASV-2.0*ATV-10.0*SS1+4.0*SS3)/15.0
            AYY=2.0/3.0-SIGR*(2.0*AVV/SS1-SS1*0.5)+SGR2*
     >          (SS2+5.0-2.0*AUV)/15.0
            AZZ=2.0*AVV/SS1-SIGR*(10.0-4.0*SS2-2.0*AUV)/15.0+SGR2*
     >          (2.0*ARV+FP2*AVV/SS1-SS1*0.5+SS3/6.0)
            PSS(9)=1.0+SIGR*(SS1-ASV)*2.0/3.0+SGR2*(0.5-2*AVV)
            PSS(13)=3.0*SQ*AXX-2.0*SQ*PSS(9)
            PSS(10)=3.0*SQ*AYY-2.0*SQ*PSS(9)
            PSS(14)=18.0*AZZ-12.0*AXX-12.0*AYY+8.0*PSS(9)
            BXX=DE2*2.0/3.0-SIGR*(FP2*FP2)+SGR2*DE3*4.0/5.0
            BZZ=FP2*FP2*0.5-SIGR*DE3*4.0/5.0+SGR2*(FP2**3)*2.0/3.0
            PSS(11)=FP2-SIGR*DE2*4.0/3.0+SGR2*(FP2*FP2)
            PSS(15)=3.0*SQ*BXX-2.0*SQ*PSS(11)
            PSS(16)=18.0*BZZ-24.0*BXX+8.0*PSS(11)
            PSS(12)=PSS(15)
            PSS(3)=SS2*PSS(9)
            PSS(4)=SS2*PSS(13)
            PSS(7)=SS2*PSS(10)
            PSS(8)=SS2*PSS(14)
            PVS(3)=(DE2-SS2*(SS1-ASV)*0.5+((2.0*AVV-0.5)*SS2
     >             -FP2*FP2)*0.75*SIGR)/(1.0-SS3)
            PVS(1)=((ASV-SS1)*2.0-SIGR*(0.5-2.0*AVV)*3.0)*SS2/
     >             (4.0*(1.0-SS3))
            PVS(2)=(4.5*AVV/SS1-ASV-SS1/8.0+(AUV*2.0/5.0
     >             -SS2/5.0-4.0*AVV)*0.75*SIGR)*SQ*SS2/(1.0-SS3)
            PVS(4)=(4.0/3.0*(SS2*ASV-SS3)+3.0*SS2-1.5*SS4-3.0
     >             -6.0*SS2*AVV+8.0*DE2/3.0+SIGR*(ASV*SS2*FP2
     >             +6.0*SS3*AUV/15.0-2.0*SS3+4.0*SS5/5-SS2
     >             +4.0*SS2*AVV+12.0*DE3/5.0-2.0*FP2*FP2))
     >             *(-3.0*SQ)/(4*(1.0-SS3))
            PVV=(1.0+SS4-SS2-4.0*SS2*AVV)*0.75*RAA/(1.0-SS3)
         ENDIF
      ENDIF
      RETURN
      END
