*DECK SPHEQU
      SUBROUTINE SPHEQU(IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS ,MAXMAT,
     >                  NDOOR ,NSPH  ,KSPH  ,IPRINT,ITRANC,NGROUP,
     >                  NGCOND,NBMIX ,NREGIO,NMERGE,NIFISS,MATCOD,
     >                  VOLUME,KEYFLX,IGCOND,IMERGE,FLUXES,NREG2 ,
     >                  NUN2  ,ILK   ,CTITRE,SPH   ,
     >                  MAT2  ,VOL2  ,KEY2  ,MERG2 ,SIGMA ,SIGMS ,
     >                  COURIN,VOLMER,SIGT  ,SIGW  ,DIFF  ,FLXMER,
     >                  SUNMER,FUNKNO,FACTOR,SOURCE,CURH  )
C
C-----------------------------------------------------------------------
C
C CALCULATION OF THE SPH FACTORS FOR THE HOMOGENIZATION OF ANY GEOMETRY
C USING A TRANSPORT-TRANSPORT OR TRANSPORT-DIFFUSION EQUIVALENCE
C TECHNIQUE. THE MACRO-CALCULATION CAN BE PERFORMED USING COLLISION
C PROBABILITIES OR USING DIFFUSION THEORY (BIVAC).
C
C REFERENCES:
C   A. HEBERT, 'A CONSISTENT TECHNIQUE FOR THE PIN-BY-PIN HOMOGENIZATION
C   OF A PRESSURIZED WATER REACTOR ASSEMBLY', NUCL. SCI. ENG., 113, 227
C   (1993).
C
C   A. HEBERT AND P. BENOIST, 'A CONSISTENT TECHNIQUE FOR THE GLOBAL
C   HOMOGENIZATION OF A PRESSURIZED WATER REACTOR ASSEMBLY', NUCL. SCI.
C   ENG., 109, 360 (1991).
C
C   A. HEBERT AND G. MATHONNIERE, 'DEVELOPMENT OF A THIRD-GENERATION
C   SUPERMOMOGENEISATION METHOD FOR THE HOMOGENIZATION OF A
C   PRESSURIZED WATER REACTOR ASSEMBLY', NUCL. SCI. ENG., 115, 129
C   (1993).
C
C INPUT PARAMETERS:
C   IPTRK2 : POINTER TO THE TRACKING OF THE MACRO-GEOMETRY (L_TRACK
C            SIGNATURE).
C   IFTRAK : UNIT NUMBER OF THE SEQUENTIAL BINARY TRAVKING FILE.
C   IPFLUX : POINTER TO THE REFERENCE SOLUTION (L_FLUX SIGNATURE).
C   IPMACR : POINTER TO THE REFERENCE MACROLIB (L_MACROLIB SIGNATURE).
C   IPSYS  : POINTER TO THE REFERENCE PIJ (L_PIJ SIGNATURE).
C   MAXMAT : MAX(MAT2(IREG)) FOR (IREG .LE. NREG2).
C   NDOOR  : TRACKING MODULE USED TO TRACK THE MACRO-GEOMETRY.
C   NSPH   : =2 HOMOGENEOUS MACRO-CALCULATION (NON ITERATIVE PROCEDURE
C               OR HEBERT-BENOIST SPH-5 PROCEDURE).
*            =3 ANY TYPE OF PIJ MACRO-CALCULATION WITH GEOMETRY
*               PROVIDED AS INPUT.
*            =4 ANY TYPE OF DIFFUSION MACRO-CALCULATION.
*            =5 ANY TYPE OF PIJ MACRO-CALCULATION WITH TRACKING
*               PROVIDED AS INPUT.
C   KSPH   : =1 AVERAGE FLUX NORMALIZATION.
C            =2 SELENGUT NORMALIZATION.
C            =3 SELENGUT NORMALIZATION WITH SURFACE LEAKAGE.
C            IF KSPH=2 OR 3, THE 'ALBS' OPTION SHOULD DE USED IN ASM:.
C            THE HEBERT-BENOIST PROCEDURE IS USED IF NSPH=2 AND KSPH=3.
C   IPRINT : PRINT FLAG. EQUAL TO 0 FOR NO PRINT.
C   ITRANC : TYPE OF TRANSPORT CORRECTION.
C   NGROUP : NUMBER OF GROUPS.
C   NGCOND : NUMBER OF GROUPS CONDENSED.
C   NBMIX  : NUMBER OF MIXTURES.
C   NREGIO : NUMBER OF REGIONS.
C   NMERGE : NUMBER OF MERGED REGIONS (EQUAL TO 1 OR TO THE NUMBER OF
C            PHYSICAL CELLS IN EURYDICE).
C   NIFISS : NUMBER OF FISSILE ISOTOPES.
C   MATCOD : MIXTURE INDEX PER REGION. DIMENSION MATCOD(NREGIO)
C   VOLUME : VOLUME OF REGIONS. DIMENSION VOLUME(NREGIO)
C   KEYFLX : POSITION OF AVERAGE FLUXES. DIMENSION KEYFLX(NREGIO)
C   IGCOND : LIMIT CONDENSED GROUPS. DIMENSION IGCOND(NGCOND)
C   IMERGE : INDEX OF MERGED REGIONS. DIMENSION IMERGE(NREGIO)
C   FLUXES : FLUXES. DIMENSION FLUXES(NREGIO,NGROUP)
C   NREG2  : NUMBER OF MACRO-REGIONS (IN THE MACRO CALCULATION).
C   NUN2   : NUMBER OF UNKNOWNS IN A ONE-GROUP MACRO-CALCULATION.
C   ILK    : LEAKAGE SWITCH.
C   CTITRE : TITLE.
C   SPH    : SPH HOMOGENIZATION FACTORS. DIMENSION SPH(NMERGE,NGCOND)
C   MAT2   : MIXTURE INDEX PER MACRO-REGION. DIMENSION MAT2(NREG2)
C   VOL2   : VOLUME OF MACRO-REGIONS. DIMENSION VOL2(NREG2)
C   KEY2   : POINTER TO FLUX VALUES IN UNKNOWN VECTOR.
C            DIMENSION KEY2(NREG2)
C   MERG2  : INDEX OF MERGED MACRO-REGIONS. DIMENSION MERG2(NREG2)
C
C SCRATCH STORAGE:
C   SIGMA  : DIMENSION SIGMA(0:MAXMAT)
C   SIGMS  : DIMENSION SIGMS(0:MAXMAT)
C   COURIN : DIMENSION COURIN(NGCOND)
C   VOLMER : DIMENSION VOLMER(NMERGE)
C   SIGT   : DIMENSION SIGT(NMERGE,NGCOND)
C   SIGW   : DIMENSION SIGW(NMERGE,NGCOND)
C   DIFF   : DIMENSION DIFF(NMERGE,NGCOND)
C   FLXMER : DIMENSION FLXMER(NMERGE,NGCOND)
C   SUNMER : DIMENSION SUNMER(NMERGE,NGCOND,NGCOND)
C   FUNKNO : DIMENSION FUNKNO(NUN2,NGCOND)
C   FACTOR : DIMENSION FACTOR(NMERGE)
C   SOURCE : DIMENSION SOURCE(NREG2)
C   CURH   : DIMENSION CURH(NREGIO,NGROUP,3)
C
C------------------------------------- AUTHOR: A. HEBERT ; 15/04/91 ----
C
      IMPLICIT    NONE
C----
C PARAMETERS
C----
      INTEGER     MAXIT
      REAL        EPS
      PARAMETER  (EPS=1.0E-4,MAXIT=200)
C----
C  MEMORY ALLOCATION PARAMETERS
C----
      INTEGER      IBASE(1)
      REAL         RBASE
      COMMON       RBASE(1)
      EQUIVALENCE (RBASE(1),IBASE(1))
      INTEGER      IPRODU,ISIGMA,INJJ,IIJJ,IIPOS,IXSCAT,IDIFF,IWORK,
     1             IDIFFU,IPIJSY,ISOURC
C----
C INTERFACE VARIABLES
C----
      INTEGER      IPTRK2,IFTRAK,IPFLUX,IPMACR,IPSYS ,MAXMAT,NREGIO,
     >             NMERGE,NGCOND,NUN2,NGROUP,NIFISS,NBMIX
      INTEGER      MATCOD(*),KEYFLX(*),IGCOND(*),IMERGE(*),MAT2(*),
     >             KEY2(*),MERG2(*)
      REAL         VOLUME(*),FLUXES(NREGIO,*),VOL2(*),SPH(NMERGE,*),
     >             SIGMA(0:*),SIGMS(0:*),COURIN(*),VOLMER(*),
     >             SIGT(NMERGE,*),SIGW(NMERGE,*),DIFF(NMERGE,*),
     >             FLXMER(NMERGE,*),B2(4),SUNMER(NMERGE,NGCOND,*),
     >             FUNKNO(NUN2,*),FACTOR(*),SOURCE(*),
     >             CURH(NREGIO,NGROUP,*)
C----
C LOCAL VARIABLES
C----
      INTEGER      KSPH,IXSLEN,ITYLCM,ILEAKS,ILCMLN,ITYXS,IPRINT,ITRANC,
     1             IKK,IGR,JGR,NSPH,IFIN,IDEB,IGR2,IREG,IUNK,NREG2,
     2             IMERG,ITER,IPRIN2,NBIERR
      REAL         VOLTOT,FLXTOT,ERROR,SUM,SPHNEW,OLDERR
      REAL         ERRT,ERR2
      CHARACTER    NDOOR*12,CTITRE*72,TEXT12*12
      LOGICAL      ILK
C
C CALCULATION OF THE REFERENCE MERGED/CONDENSED SET OF CROSS SECTIONS.
C
      IF(ILK) CALL XABORT('SPHEQU: LEAKAGE IS FORBIDDEN.')
      IF((KSPH.EQ.2).OR.(KSPH.EQ.3)) THEN
         IF(IPSYS.EQ.0) CALL XABORT('SPHEQU: THE L_PIJ INFO IS NOT AVAI'
     >   //'LABLE.')
         WRITE(TEXT12,'(9HGROUP  1/,I3)') NGROUP
         CALL LCMSIX(IPSYS,TEXT12,1)
         CALL LCMLEN(IPSYS,'DRAGON-PIS',IXSLEN,ITYLCM)
         IF(IXSLEN.NE.NREGIO) THEN
           CALL XABORT('SPHEQU: THE ALBS OPTION OF THE ASM: MODULE HAV'
     >     //'E NOT BEEN ACTIVATED.')
         ENDIF
         CALL LCMSIX(IPSYS,' ',2)
      ENDIF
*
      ILEAKS=0
      IF(KSPH.NE.3) THEN
        CALL LCMLEN(IPFLUX,'B2  B1HOM',ILCMLN,ITYXS)
        IF(ILCMLN.EQ.1) THEN
          ILEAKS=2
          CALL LCMGET(IPFLUX,'B2  B1HOM',B2(4))
        ENDIF
        CALL LCMLEN(IPFLUX,'B2  HETE',ILCMLN,ITYXS)
        IF(ILCMLN.EQ.3) THEN
          ILEAKS=3
          CALL LCMGET(IPFLUX,'B2  HETE',B2)
        ENDIF
      ENDIF
      CALL SETARA(RBASE,NREGIO*NGCOND*NIFISS,IPRODU)
      CALL SETARA(RBASE,NBMIX*NIFISS+1,ISIGMA)
      CALL SETARA(RBASE,NBMIX,INJJ)
      CALL SETARA(RBASE,NBMIX,IIJJ)
      CALL SETARA(RBASE,NBMIX,IIPOS)
      CALL SETARA(RBASE,NBMIX*NGROUP,IXSCAT)
      CALL SETARA(RBASE,NREGIO*2,IWORK)
      CALL SETARA(RBASE,NGROUP,IDIFF)
      CALL SPHREF(IPFLUX,IPMACR,IPSYS,IPRINT,ITRANC,ILEAKS,B2,NGROUP,
     1 NGCOND,NBMIX,NREGIO,NMERGE,NIFISS,MATCOD,VOLUME,KEYFLX,IGCOND,
     2 IMERGE,FLUXES,VOLMER,FLXMER,SUNMER,SIGT,SIGW,COURIN,CURH,
     3 RBASE(IPRODU),RBASE(ISIGMA),RBASE(INJJ),RBASE(IIJJ),
     4 RBASE(IIPOS),RBASE(IXSCAT),RBASE(IWORK),RBASE(IDIFF))
      CALL RLSARA(RBASE(IDIFF))
      CALL RLSARA(RBASE(IWORK))
      CALL RLSARA(RBASE(IXSCAT))
      CALL RLSARA(RBASE(IIPOS))
      CALL RLSARA(RBASE(IIJJ))
      CALL RLSARA(RBASE(INJJ))
      CALL RLSARA(RBASE(ISIGMA))
      CALL RLSARA(RBASE(IPRODU))
      IF(IPRINT.GE.5) THEN
         WRITE(6,930) 'VOLMER',(VOLMER(IKK),IKK=1,NMERGE)
         WRITE(6,930) 'SIGT',((SIGT(IKK,IGR),IKK=1,NMERGE),IGR=1,NGCOND)
         WRITE(6,930) 'SIGW',((SIGW(IKK,IGR),IKK=1,NMERGE),IGR=1,NGCOND)
         WRITE(6,930) 'FLXMER',((FLXMER(IKK,IGR),IKK=1,NMERGE),
     >   IGR=1,NGCOND)
         WRITE(6,930) 'SUNMER',(((SUNMER(IKK,IGR,JGR),IKK=1,NMERGE),
     >   IGR=1,NGCOND),JGR=1,NGCOND)
      ENDIF
      DO 10 IGR=1,NGCOND
      DO 10 IKK=1,NMERGE
   10 SUNMER(IKK,IGR,IGR)=SUNMER(IKK,IGR,IGR)-SIGW(IKK,IGR)
      IF(NSPH.EQ.4) THEN
*        TRANSPORT-DIFFUSION EQUIVALENCE.
         DO 15 IGR=1,NGCOND
         DO 15 IKK=1,NMERGE
   15    SIGT(IKK,IGR)=SIGT(IKK,IGR)-SIGW(IKK,IGR)
*
*        COMPUTE THE DIFFUSION COEFFICIENTS.
         DO 44 IGR=1,NGCOND
         DO 44 IKK=1,NMERGE
   44    DIFF(IKK,IGR)=0.0
         IF(ILEAKS.EQ.2) THEN
           CALL SETARA(RBASE,NGROUP,IDIFFU)
           CALL LCMGET(IPFLUX,'DIFFB1HOM',RBASE(IDIFFU))
           IFIN=0
           DO 40 IGR=1,NGCOND
           IDEB=IFIN+1
           IFIN=IGCOND(IGR)
           DO 30 IGR2=IDEB,IFIN
           DO 30 IREG=1,NREGIO
           IKK=IMERGE(IREG)
           IF(IKK.EQ.0) GO TO 30
           DIFF(IKK,IGR)=DIFF(IKK,IGR)+RBASE(IDIFFU+IGR2-1)
     1     *FLUXES(IREG,IGR2)*VOLUME(IREG)
   30      CONTINUE
           DO 40 IKK=1,NMERGE
   40      DIFF(IKK,IGR)=DIFF(IKK,IGR)/(FLXMER(IKK,IGR)*VOLMER(IKK))
           CALL RLSARA(RBASE(IDIFFU))
         ELSE IF(ILEAKS.EQ.3) THEN
           IFIN=0
           DO 50 IGR=1,NGCOND
           IDEB=IFIN+1
           IFIN=IGCOND(IGR)
           DO 45 IGR2=IDEB,IFIN
           DO 45 IREG=1,NREGIO
           IKK=IMERGE(IREG)
           IF(IKK.NE.0) THEN
              DIFF(IKK,IGR)=DIFF(IKK,IGR)+(CURH(IREG,IGR2,1)
     >        +CURH(IREG,IGR2,2))*VOLUME(IREG)/2.0
           ENDIF
 45        CONTINUE
           DO 50 IKK=1,NMERGE
 50        DIFF(IKK,IGR)=DIFF(IKK,IGR)/(FLXMER(IKK,IGR)*VOLMER(IKK))
         ENDIF
         IF(IPRINT.GE.10) WRITE(6,930) 'DIFF',((DIFF(IKK,IGR),
     1   IKK=1,NMERGE),IGR=1,NGCOND)
      ENDIF
*
* ITERATIVE STRATEGY USED TO COMPUTE THE SPH FACTORS.
      IF((KSPH.EQ.2).OR.(KSPH.EQ.3)) THEN
         IF(IPRINT.GE.10) THEN
            WRITE(6,900) (COURIN(IGR),IGR=1,NGCOND)
            WRITE(6,'(/)')
         ENDIF
         DO 60 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 55 IKK=1,NMERGE
         VOLTOT=VOLTOT+VOLMER(IKK)
   55    FLXTOT=FLXTOT+FLXMER(IKK,IGR)*VOLMER(IKK)
         FLXTOT=FLXTOT/VOLTOT
         DO 60 IKK=1,NMERGE
   60    SPH(IKK,IGR)=FLXTOT/COURIN(IGR)
      ELSE
         DO 80 IGR=1,NGCOND
         VOLTOT=0.0
         FLXTOT=0.0
         DO 70 IKK=1,NMERGE
         VOLTOT=VOLTOT+VOLMER(IKK)
   70    FLXTOT=FLXTOT+FLXMER(IKK,IGR)*VOLMER(IKK)
         COURIN(IGR)=FLXTOT/VOLTOT
         DO 80 IKK=1,NMERGE
   80    SPH(IKK,IGR)=1.0
         IF(IPRINT.GE.10) THEN
            WRITE(6,910) (COURIN(IGR),IGR=1,NGCOND)
            WRITE(6,'(/)')
         ENDIF
      ENDIF
      OLDERR=1.0
      NBIERR=0
      ITER=0
      IF((NSPH.EQ.2).AND.(KSPH.NE.3)) GO TO 230
      DO 100 IGR=1,NGCOND
      DO 90 IUNK=1,NUN2
   90 FUNKNO(IUNK,IGR)=0.0
      DO 100 IREG=1,NREG2
  100 FUNKNO(KEY2(IREG),IGR)=FLXMER(MERG2(IREG),IGR)
      SIGMA(0)=0.0
      SIGMS(0)=0.0
  110 ITER=ITER+1
      IF(ITER.GE.MAXIT) THEN
        WRITE(6,'(/46H SPHEQU: MAX. NUMBER OF ITERATIONS IS REACHED.)')
        GO TO 230
      ENDIF
      ERROR=0.0
      ERR2=0.0
      DO 220 IGR=1,NGCOND
*
* MACRO FLUX CALCULATION IN A SINGLE CONDENSED GROUP AT ITERATION ITER.
      DO 120 IREG=1,NREG2
      IF(MAT2(IREG).GT.MAXMAT) THEN
         CALL XABORT('SPHEQU: INVALID MACRO-MIXTURE INDEX.')
      ENDIF
  120 SIGMA(MAT2(IREG))=SIGT(MERG2(IREG),IGR)*SPH(MERG2(IREG),IGR)
      IPRIN2=MAX(0,IPRINT-10)
      IF((NSPH.EQ.3).OR.(NSPH.EQ.5).OR.
     >   ((NSPH.EQ.2).AND.(KSPH.EQ.3))) THEN
*        TRANSPORT-TRANSPORT EQUIVALENCE.
         DO 140 IREG=1,NREG2
         IF(IREG.NE.KEY2(IREG)) CALL XABORT('SPHEQU: INVALID KEY2.')
         IMERG=MERG2(IREG)
         SIGMS(MAT2(IREG))=SIGW(IMERG,IGR)*SPH(IMERG,IGR)
         SUM=0.0
         DO 130 JGR=1,NGCOND
  130    SUM=SUM+SUNMER(IMERG,JGR,IGR)*FUNKNO(IREG,JGR)*SPH(IMERG,JGR)
  140    SOURCE(IREG)=SUM   
         IF(IGR .EQ. 1) THEN 
           CALL SETARA(RBASE,(NREG2*(NREG2+1))/2,IPIJSY)
           CALL SETARA(RBASE,NREG2*(NREG2+1),IWORK)
         ENDIF
         CALL SPHTRA(IPTRK2,IFTRAK,NDOOR ,KSPH  ,IPRIN2,NREG2 ,
     >               MAXMAT,MAT2  ,VOL2  ,ILK   ,CTITRE,COURIN(IGR),
     >               SIGMA ,SIGMS ,SOURCE,FUNKNO(1,IGR),
     >               RBASE(IPIJSY),FLXTOT,RBASE(IWORK) )
         IF(IGR .EQ. NGCOND) THEN 
           CALL RLSARA(RBASE(IWORK))
           CALL RLSARA(RBASE(IPIJSY))
         ENDIF
      ELSE IF(NSPH.EQ.4) THEN
*        TRANSPORT-DIFFUSION EQUIVALENCE.
         IF(ILEAKS.EQ.0) CALL XABORT('SPHEQU: UNABLE TO COMPUTE DIFFUS'
     1   //'ION COEFFICIENTS.')
         DO 150 IREG=1,NREG2
  150    SIGMS(MAT2(IREG))=DIFF(MERG2(IREG),IGR)*SPH(MERG2(IREG),IGR)
         CALL SETARA(RBASE,NUN2,ISOURC)
         CALL SPHDIF(IPTRK2,KSPH,IPRIN2,NREG2,NUN2,NMERGE,MAT2,VOL2,
     1   KEY2,MERG2,ILK,CTITRE,COURIN(IGR),SIGMA(1),SIGMS(1),
     2   NGCOND,FLXMER,
     3   SUNMER(1,1,IGR),FUNKNO,FLXTOT,IGR,ITER,RBASE(ISOURC))
         CALL RLSARA(RBASE(ISOURC))
      ENDIF
*
* COMPUTE THE IMPROVED SPH FACTORS.
      DO 180 IKK=1,NMERGE
      VOLMER(IKK)=0.0
  180 FACTOR(IKK)=0.0
      DO 190 IREG=1,NREG2
      IKK=MERG2(IREG)
      VOLMER(IKK)=VOLMER(IKK)+VOL2(IREG)
  190 FACTOR(IKK)=FACTOR(IKK)+FUNKNO(KEY2(IREG),IGR)*VOL2(IREG)
      DO 210 IKK=1,NMERGE
      FACTOR(IKK)=FACTOR(IKK)/VOLMER(IKK)
      SPHNEW=FLXMER(IKK,IGR)/FACTOR(IKK)
      IF(SPHNEW .LT. 0.0) THEN
        WRITE(6,970) IGR, IKK
        SPHNEW=1.0    
      ENDIF
      ERRT=ABS((SPHNEW-SPH(IKK,IGR))/SPHNEW)
      ERR2=ERR2+ERRT*ERRT
      ERROR=MAX(ERROR,ERRT)
*      ERROR=MAX(ERROR,ABS((SPHNEW-SPH(IKK,IGR))/SPHNEW))
  210 SPH(IKK,IGR)=SPHNEW
      IF(IPRINT.GE.10) THEN
         WRITE(6,930) 'SPH',(SPH(IKK,IGR),IKK=1,NMERGE)
         WRITE(6,930) 'FUNKNO',(FUNKNO(IUNK,IGR),IUNK=1,NUN2)
      ENDIF
  220 CONTINUE
      ERR2=SQRT(ERR2/(NMERGE*NGCOND))
      IF(IPRINT.GT.3) THEN
         WRITE(6,935) ITER,ERROR,ERR2
        IF(IPRINT.GE.10) THEN
           IF(ERROR.GE.EPS) WRITE(6,940) ((IKK,IGR,SPH(IKK,IGR),
     1     IKK=1,NMERGE),IGR=1,NGCOND)
        ENDIF
      ENDIF
      IF(ERR2 .LT.EPS) GO TO 230 
      IF(ITER .GT. 3) THEN
        IF(ERR2 .GT. OLDERR) THEN
          WRITE(6,960) ITER
          NBIERR=NBIERR+1
          IF(NBIERR .GE. 10) THEN
            WRITE(6,961) ITER
            GO TO 230
          ENDIF
        ENDIF
      ENDIF
      OLDERR=ERR2
      GO TO 110
  230 IF(IPRINT.GE.5) THEN
         WRITE(6,950) ITER
         WRITE(6,940) ((IKK,IGR,SPH(IKK,IGR),IKK=1,NMERGE),IGR=1,NGCOND)
      ENDIF
      RETURN
*
  900 FORMAT(/49H SPHEQU: IN-CURRENTS (4J-/S) PER MACRO-GROUPS ARE/
     > (1X,1P,10E13.5))
  910 FORMAT(/44H SPHEQU: AVERAGE FLUXES PER MACRO-GROUPS ARE/
     > (1X,1P,10E13.5))
  930 FORMAT(/26H SPHEQU: VALUES OF VECTOR ,A6,4H ARE/(1X,1P,10E13.5))
  935 FORMAT(/6H ITER=,I3,4X,6HERROR=,1P,E10.3,1X,6HERR 2=,E10.3)
  940 FORMAT(4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,
     > F9.5,:,4X,4HSPH(,I3,1H,,I3,2H)=,F9.5)
  950 FORMAT(/40H SPHEQU: CONVERGENCE OF SPH ALGORITHM IN,I5,
     > 12H ITERATIONS./)
  960 FORMAT(1X,'Warning: oscillations in SPH at iteration ',i10)
  961 FORMAT(1X,'Warning: maximum of 3 error oscillations ',
     >'in SPH convergence reached at iteration ',i10)
  970 FORMAT(1X,'Warning: SPH factor in group ',i10,' region ',i10,
     >       ' negative'/1X,'Reset SPH factor to 1.0')
      END
