*DECK SYBALZ
      FUNCTION SYBALZ(IT,NA,S,T,TAU,ALBG,ALBD,EPS1,EPS2)
*
*----------------------------------------------------------------------
*
*  Evaluation of comtribution to CP for slab. 
*
*
* INPUT PARAMETERS:
*    IT     : Type of symmetry. 
*             For IT=0, reflection symmetries are considered. 
*             For IT=1, translation symmetries are considered.
*    NA     : Exponential integral order.
*    S      : Optical length to first region (I).
*    T      : Optical length to second region (J).
*    TAU    : Optical path for a complete cycle (before symmetry).
*    ALBG   : Left albedo.
*    ALBD   : Right albedo.
*    EPS1   : Positive Z factor.
*    EPS2   : Negative Z factor.
*
* OUTPUT PARAMETERS:
*    SYBALZ : Contribution of path to collision probability
*             matrix.
*
* REFERENCE:
* A. KAVENOKY, 'CALCUL ET UTILISATION DES PROBABILITES DE PREMIERE
* COLLISION POUR LES MILIEUX HETEROGENES A UNE DIMENSION: LES PROGRAMMES
* ALCOLL ET CORTINA', NOTE CEA-N-1077, COMMISSARIAT A L'ENERGIE
* ATOMIQUE, MARS 1969.
*
*----------------------------------------------------------------------
*
      IMPLICIT   NONE
      INTEGER    IT,NA
      REAL       S,T,TAU,ALBG,ALBD,EPS1,EPS2
      REAL       SYBALZ
      REAL       TRONC,CUTALB 
      PARAMETER (TRONC=10.0,CUTALB=1.0E-04)
      REAL       TABEN
      REAL       U,BETA,TAUC
      INTEGER    ITERMS,NTERMS
      SYBALZ=0.0
      IF(IT .EQ. 0) THEN
*----
*  Reflection symmetry or void boundary
*  1) Direct term
*----
        TAUC=2.0*TAU
        U=T-S
        IF (U.LT.0.) U=0.
        SYBALZ=TABEN(NA,U)
        IF(ALBG .GT. CUTALB .AND. ALBD .GT. CUTALB) THEN
*----
*  2) Positive Z direction cyclic region
*----
          BETA=ALBG*ALBD
          NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
          DO 100 ITERMS=1,NTERMS
            U=U+TAUC
            SYBALZ=SYBALZ+BETA*TABEN(NA,U) 
            BETA=BETA*ALBG*ALBD
 100      CONTINUE
*----
*  3) Negative Z direction cyclic region
*---- 
          BETA=ALBD*ALBG
          U=(S-T)
          NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
          DO 101 ITERMS=1,NTERMS
            U=U+TAUC
            SYBALZ=SYBALZ+BETA*EPS2*TABEN(NA,U)
            BETA=BETA*ALBG*ALBD
 101      CONTINUE
*----
*  4) Positive Z direction reflective region
*     and cyclic contibution
*---- 
          BETA=ALBD
          U=-S-T
          NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
          DO 102 ITERMS=1,NTERMS
            U=U+TAUC
            SYBALZ=SYBALZ-BETA*EPS1*TABEN(NA,U)
            BETA=BETA*ALBG*ALBD
 102      CONTINUE
*----
*  5) Negative Z direction reflective region
*     and cyclic contibution
*---- 
          BETA=ALBG
          U=S+T-TAUC
          NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
          DO 103 ITERMS=1,NTERMS
            U=U+TAUC
            SYBALZ=SYBALZ-BETA*EPS1*EPS2*TABEN(NA,U)
            BETA=BETA*ALBG*ALBD
 103      CONTINUE
        ELSE IF (ALBD.GT.1.E-4) THEN
*----
*  Positive Z direction reflective region
*----
          U=TAUC-S-T
          SYBALZ=SYBALZ-ALBD*EPS1*TABEN(NA,U)
        ELSE IF (ALBG.GT.1.E-4) THEN
*----
*  Negative Z direction reflective region
*----
          U=S+T
          SYBALZ=SYBALZ-ALBG*EPS1*EPS2*TABEN(NA,U)
        ENDIF
      ELSE
*----
*  Translation symmetry 
*  1) Direct term
*----
        TAUC=TAU
        U=T-S
        IF (U.LT.0.) U=0.
        SYBALZ=TABEN(NA,U)
*----
*  2) Positive Z direction cyclic region
*----
        NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
        DO 110 ITERMS=1,NTERMS
          U=U+TAUC
          SYBALZ=SYBALZ+TABEN(NA,U)
 110    CONTINUE
*----
*  3) Negative Z direction cyclic region
*---- 
        U=(S-T)
        NTERMS=INT(MAX(TRONC-U,0.0)/TAUC)
        DO 111 ITERMS=1,NTERMS
          U=U+TAUC
          SYBALZ=SYBALZ+EPS2*TABEN(NA,U)
 111    CONTINUE
      ENDIF
      RETURN
      END
