*DECK SYBHT1
      SUBROUTINE SYBHT1 (IR,IPAS,SIGT,IPAS2,NG,NSMAX,MICRO,IQUAD,NS,
     1 IBI,RS,FRACT,VOLK,P,RRRR,SIGT0,QKDEL,QKN)
*
*-----------------------------------------------------------------------
*
* CALCULATION OF THE REDUCED COLLISION PROBABILITIES FOR THE DOUBLE
* HETEROGENEITY OPTION OF SYBIL. PART 1.
*
* GENERAL PARAMETERS:
*   IR     : NUMBER OF MIXTURES IN THE DOMAIN.
*   IPAS   : NUMBER OF VOLUMES IN THE COMPOSITE GEOMETRY.
*   SIGT   : TOTAL MACROSCOPIS CROSS SECTIONS IN EACH VOLUME OF THE
*            COMPOSITE GEOMETRY. DIMENSION SIGT(IPAS)
*   IPAS2  : NUMBER OF VOLUMES IN THE MACRO GEOMETRY.
*   NG     : NUMBER OF DIFFERENT KIND OF MICRO STRUCTURES. A KIND OF
*            MICRO STRUCTURE IS CHARACTERIZED BY THE RADIUS OF ITS
*            MICRO VOLUMES. ALL THE MICRO VOLUMES OF THE SAME KIND
*            SHOULD OWN THE SAME NUCLEAR PROPERTIES IN A GIVEN
*            MACRO VOLUME.
*   NSMAX  : MAXIMUM NUMBER OF VOLUMES (TUBES OR SHELLS) IN EACH KIND
*            OF MICRO STRUCTURE.
*   MICRO  : TYPE OF MICRO VOLUMES: =3: CYLINDER ; =4: SPHERE.
*   IQUAD  : QUADRATURE PARAMETER FOR THE TREATMENT OF THE MICRO VOLUMES
*   NS     : NUMBER OF VOLUMES IN EACH KIND OF MICRO STRUCTURE.
*            DIMENSION NS(NG)
*   IBI    : TYPE OF MIXTURE IN EACH VOLUME OF THE MACRO GEOMETRY.
*            IF IBI(IKK) IS GREATER THAN IR, THE VOLUME IKK CONTAIN A
*            MICRO STRUCTURE. DIMENSION IBI(IPAS2)
*   RS     : RADIUS OF THE MICRO VOLUMES.
*            DIMENSION RS((1+NSMAX),NG)
*   FRACT  : VOLUMIC FRACTIONS OF THE MICRO VOLUMES.
*            DIMENSION FRACT(NG,IR2) WHERE IR2 IS THE NUMBER OF
*            ORDINARY AND COMPOSITE MIXTURES.
*   VOLK   : VOLUMIC FRACTIONS OF THE TUBES OR SHELLS IN THE MICRO
*            VOLUMES. DIMENSION VOLK(NG,NSMAX)
*   P      : REDUCED COLLISION PROBABILITIES FOR THE COMPOSITE GEOMETRY.
*            DIMENSION P(IPAS,IPAS)
*   RRRR   : INFORMATION USED BY SYBHT2. DIMENSION RRRR(IPAS2)
*   SIGT0  : EQUIVALENT MACROSCOPIC CROSS SECTION IN EACH MACRO VOLUME.
*            DIMENSION SIGT0(IPAS2)
*   QKDEL  : INFORMATION USED BY SYBHT2. DIMENSION QKDEL(NG,NSMAX,IPAS2)
*
* SCRATCH STORAGE.
*   QKN    : DIMENSION QKN(NSMAX**2)
*
*---------------------------------------- AUTHOR: A. HEBERT 20/02/87 ---
*
      DIMENSION SIGT(*),NS(*),IBI(*),RS(NSMAX+1,*),FRACT(NG,*),
     1 VOLK(NG,*),P(IPAS,*),RRRR(*),SIGT0(*),QKDEL(NG,NSMAX,*),
     2 QKN(NSMAX,*),RGAR(2)
      CHARACTER*131 HSMG
      DOUBLE PRECISION DP0,DP1,RAPP,QKD,FACT
      COMMON BASE(1)
*
      DO 40 I=1,IPAS
      DO 40 L=1,IPAS
40    P(I,L)=0.0
      IND1=IPAS2
      DO 110 I=1,IPAS2
      MIL=IBI(I)
      SIGT0(I)=SIGT(I)
      RRRR(I)=1.0
      IF (MIL.GT.IR) THEN
         DILF=1.0
         DO 41 J=1,NG
41       DILF=DILF-FRACT(J,MIL)
         DP0=DILF*SIGT(I)
         DP1=DP0
         DO 73 J=1,NG
         FRT=FRACT(J,MIL)
         IF (FRT.LE.0.00001) GO TO 73
         RGAR(1)=0.0
         RGAR(2)=RS(NS(J)+1,J)
         CALL SETARA(BASE,4,IX2)
         IF (MICRO.EQ.3) THEN
            CALL SYBALC(1,NSMAX,RGAR,SIGT(I),QKN,IQUAD,0.0,BASE(IX2))
         ELSE IF (MICRO.EQ.4) THEN
            CALL SYBALS(1,NSMAX,RGAR,SIGT(I),QKN,IQUAD,0.0,BASE(IX2))
         ENDIF
         CALL RLSARA(BASE(IX2))
         RAPP=1.0D0/(1.0D0-QKN(1,1)*SIGT(I))
         CALL SETARA(BASE,4*NS(J),IX2)
         IF (MICRO.EQ.3) THEN
            CALL SYBALC(NS(J),NSMAX,RS(1,J),SIGT(IND1+1),QKN,IQUAD,0.0,
     1      BASE(IX2))
         ELSE IF (MICRO.EQ.4) THEN
            CALL SYBALS(NS(J),NSMAX,RS(1,J),SIGT(IND1+1),QKN,IQUAD,0.0,
     1      BASE(IX2))
         ENDIF
         CALL RLSARA(BASE(IX2))
         IND1P=IND1
         DO 70 K=1,NS(J)
         IND1P=IND1P+1
         QKD=1.0D0
         DO 60 N=1,NS(J)
60       QKD=QKD-QKN(K,N)*SIGT(IND1+N)
         QKDEL(J,K,I)=QKD
         FACT=FRT*VOLK(J,K)*QKD*SIGT(IND1P)
         DP0=DP0+FACT
70       DP1=DP1+FACT*RAPP
         IND1P=IND1
         DO 71 K=1,NS(J)
         IND1P=IND1P+1
         IND2P=IND1
         DO 71 N=1,NS(J)
         IND2P=IND2P+1
71       P(IND1P,IND2P)=QKN(K,N)
         IND1=IND1+NS(J)
73       CONTINUE
         SIGT0(I)=DP1
         RRRR(I)=DP1/DP0
      ENDIF
110   CONTINUE
      IF (IND1.EQ.IPAS) RETURN
      WRITE (HSMG,120) IND1,IPAS
120   FORMAT (46HSYBHT1: IND1 IS INCONSISTENT WITH IPAS. IND1 =,I5,5X,
     1 6HIPAS =,I5)
      CALL XABORT(HSMG)
      END
