*DECK XCSANA
      SUBROUTINE XCSANA(NBAN,NRT,MSROD,NRINFO,RAN,NRODS,RODS,RODR,
     >                  NINT,IFTRAK,IMPX,MRTRK,NMTRAK,MTRK,NSOUT,
     >                  COTE)
C
C----------------------------------------------------------------------
C
C 1- SUBROUTINE STATISTICS:
C     NAME     : XCSANA
C     USE      : TYPICAL DP1 TRACKING FOR CLUSTER GEOMETRY
C     MODIFIED : 91-04-10
C     AUTHOR   : G. MARLEAU
C
C 2-  PARAMETERS:
C
C INPUT
C  NBAN    : NUMBER OF CONCENTRIC REGIONS                  I
C  NRT     : NUMBER OF ROD TYPES                           I
C  MSROD   : MAXIMUM NUMBER OF SUBROD PER RODS             I
C  NRINFO  : TYPE OF CONCENTRIC REGION                     I(2,NBAN)
C            NRINFO(1,IAN) = NEW REGION NUMBER
C            NRINFO(2,IAN) = ASSOCIATED CLUSTER NUMBER
C                          = 0 NO CLUSTER ASSOCIATED
C  RAN     : RADIUS / LATTICE SIDE OF CONCENTRIC REGION    R(NBAN)
C  NRODS   : INTEGER DESCRIPTION OF ROD OF A GIVEN TYPE    I(3,NRT)
C            NRODS(1,IRT) = NUMBER OF ROD
C            NRODS(2,IRT) = NUMBER OF SUBRODS IN ROD
C            NRODS(3,IRT) = ASSOCIATED CONCENTRIC REGION
C  RODS    : REAL DESCRIPTION OF ROD OF A GIVEN TYPE       R(2,NRT)
C            RODS(1,IRT) = ROD CENTER RADIUS
C            RODS(2,IRT) = ANGULAR POSITION OF FIRST ROD
C  RODR    : SUBROD RADIUS                                 R(MSROD,NRT)
C  NINT    : INTEGRATION PARAMETERS                        I(2)
C            NINT(1) = NUMBER OF ANGLES CONSIDERED
C            NINT(2) = NUMBER OF PARALLEL TRACKS PER ANGLE
C            NINT(3) = NUMBER OF INTEGRATION POINTS FOR TUBES
C  IFTRAK  : UNIT ALLOCATED TO TRACKING FILE               I
C  IMPX    : PRINT LEVEL                                   I
C  MRTRK   : SIMPLIFIED TRACK LENGTH                       I
C  NMTRAK  : MAXIMUM POSSIBLE TRACK LENGTH                 I
C  MTRK    : CURRENT MAXIMUM TRACK LENGTH                  I
C  NSOUT   : NUMBER OF OUTER SURFACE                       I
C  COTE    : Y DIMENSION FOR RECTANGLE                     R
C
C----------------------------------------------------------------------
C
      INTEGER    IOUT
      PARAMETER (IOUT=6)
      INTEGER    NBAN,NRT,MSROD,NRINFO(2,NBAN),NRODS(3,NRT),NINT(3),
     >           IMPX,MTRK,NSOUT,NTRAK(6)
      REAL       RAN(NBAN),RODS(2,NRT),RODR(MSROD,NRT),ZGPT(128),
     >           WGPT(128),VD(6,2,2,3),COTE,RAYRE(2),VAP(11)
      COMMON     BASE(1)
C----
C  DETERMINE INTEGRATION LINES FOR CLUSTER REGIONS
C----
      CALL SETARA(BASE,4*NMTRAK,ITRAK)
      CALL SETARA(BASE,3*MRTRK,IXTRAK)
      CALL SETARA(BASE,  MRTRK,IKTRAK)
      CALL SETARA(BASE,2*MRTRK,IZTRAK)
      IF(IMPX.GT.0) THEN
        WRITE(IOUT,6000) (NINT(II),II=1,3)
      ENDIF
      ZINF=0.0
      ZSUP=1.0
      IPOS=1
      CALL ALGPT(NINT(1),ZINF,ZSUP,ZGPT,WGPT)
      IF(NINT(1).NE.NINT(2)) THEN
        IPOS=NINT(1)+1
        CALL ALGPT(NINT(2),ZINF,ZSUP,ZGPT(IPOS),WGPT(IPOS))
      ENDIF
      MTRK=0
      IF(NSOUT.EQ.1) THEN
        NTAN=NBAN
      ELSE
        NTAN=NBAN-1
      ENDIF
      DO 100 IAN=1,NTAN
        IRT=NRINFO(2,IAN)
        IF(IRT.GT.0) THEN
          IF(IAN.EQ.1) THEN
            RINER=0.0
          ELSE
            RINER=RAN(IAN-1)
          ENDIF
          ROUTR=RAN(IAN)
          RODRAD=RODR(NRODS(2,IRT),IRT)
          CALL XCSTRK(NINT(1),NINT(2),ZGPT,WGPT,RINER,ROUTR,
     >                NRODS(1,IRT),RODRAD,RODS(1,IRT),NTRAK,
     >                BASE(ITRAK),NMTRAK,VD,BASE(IXTRAK),BASE(IKTRAK),
     >                BASE(IZTRAK),MRTRK,IMPX,IVAR)
          NTRK=0
          DO 110 IT=1,6
            NTRK=NTRK+NTRAK(IT)
 110      CONTINUE
          MTRK=MAX(MTRK,NTRK)
          IF(IMPX.GT.0) THEN
            WRITE(IOUT,6001) IRT,IVAR,NTRK,NTRAK
          ENDIF
          WRITE(IFTRAK) NTRK,NTRAK,IVAR,VD
          IFIN=ITRAK-1
          DO 120 ITRK=1,6
            IF(NTRAK(ITRK).GT.0) THEN
              IDEB=IFIN+1
              IFIN=IFIN+4*NTRAK(ITRK)
              WRITE(IFTRAK) (BASE(JJ),JJ=IDEB,IFIN)
            ENDIF
 120      CONTINUE
        ENDIF
 100  CONTINUE
      CALL RLSARA(BASE(IZTRAK))
      CALL RLSARA(BASE(IKTRAK))
      CALL RLSARA(BASE(IXTRAK))
      CALL RLSARA(BASE(ITRAK))
      IF(IMPX.GT.15) THEN
        ILIGN=1
      ELSE
        ILIGN=0
      ENDIF
      NZ=2
      INORM=0
      IQW=0
      IRECT=0
      NA=NINT(1)
      NX=NINT(2)
      IF(NSOUT.EQ.4) THEN
        NA=2*NA
        RAYRE(1)=0.0
        RAYRE(2)=RAN(NTAN)
        IF(COTE.EQ.0.0) THEN
          YCOTE=RAN(NBAN)
          NLZ=5+NA*(5+15*NX)
        ELSE
          YCOTE=COTE
          NLZ=5+NA*(7+25*NX)
        ENDIF
        CALL SETARA(BASE,NLZ,IZ)
        CALL SETARA(BASE,NLZ,JZ)
        CALL SYBRTK(NA,NX,NZ,RAN(NBAN),YCOTE,RAYRE,ILIGN,INORM,IRECT,
     >              IQW,NLZ,BASE(JZ),BASE(IZ),PREC,VAP)
        LZI=NLZ
        WRITE(IFTRAK) LZI,NLZ
        WRITE(IFTRAK) (BASE(JJ),JJ=IZ,IZ+LZI-1)
        WRITE(IFTRAK) (BASE(JJ),JJ=JZ,JZ+NLZ-1)
        CALL RLSARA(BASE(JZ))
        CALL RLSARA(BASE(IZ))
      ELSE IF(NSOUT.EQ.6) THEN
        NLZ=NA*(11+27*NX)+6
        LZI=3*NA+2
        CALL SETARA(BASE,LZI,IZ)
        CALL SETARA(BASE,NLZ,JZ)
        CALL SYBHTK(NA,NX,NZ,RAN(NTAN),BASE(JZ),BASE(IZ),INORM,PREC,
     >              ILIGN,IQW,RAN(NBAN),NLZ,RAYRE,VAP,RCOS,RSQ)
        WRITE(IFTRAK) LZI,NLZ
        WRITE(IFTRAK) (BASE(JJ),JJ=IZ,IZ+LZI-1)
        WRITE(IFTRAK) (BASE(JJ),JJ=JZ,JZ+NLZ-1)
        CALL RLSARA(BASE(JZ))
        CALL RLSARA(BASE(IZ))
      ENDIF
      RETURN
C----
C  FORMATS
C----
 6000 FORMAT(' NUMBER OF INTEGRATION POINTS'/
     >       '   PHI CLUSTER ANGLES = ',I10/
     >       ' THETA CLUSTER ANGLES = ',I10/
     >       '            PHI TUBES = ',I10)
 6001 FORMAT('  ROD REGION NUMBER =',I10/
     >       '       CLUSTER TYPE =',I10/
     >       '   NUMBER OF TRACKS =',I10/
     >       ' TRACKS PER REGIONS =',6I10)
      END
