*DECK XCSINT
      SUBROUTINE XCSINT(IAPPR,IROT,NRODS,NSURF,CHORD,IVAR,NTRAK,TRAK,
     >                  VD,SIGT,TRONC,PII,PIS,PSS)
C
C---------------------------  XCSINT  ---------------------------------
C
C 1-  PROGRAMME STATISTICS:
C
C   NAME       :  XCSINT
C   FUNCTION   :  INTEGRATE ROD CLUSTER TRANSMISSION PROBABILITY
C   VERSION    :  1.0
C   REVISED    :  18-05-1990
C   AUTHOR     :  G.MARLEAU
C
C 2-  PROGRAMME PARAMETERS:
C
C   IAPPR   : ANGULAR EXPANSION ORDER                    I
C             IAPPR = 1 -> DP0 ALL
C                         NSUR=3, NDIR=1, NPRB=3
C             IAPPR = 2 -> DP1 ALL OR DP1 INSIDE DP0 OUTSIDE
C                         NSUR=3, NDIR=2, NPRB=NSUR*NDIR
C   IROT    : TYPE OF RECONSTRUCTION                     I
C             IROT = 1 -> DIRECT RECONSTRUCTION
C             IROT = 2 -> ROT2 RECONSTRUCTION
C   NRODS   : NUMBER OF RODS                             I
C   NSURF   : NUMBER OF SURFACES                         I
C   CHORD   : AVERAGE CHORD FOR SURFACE (4V/S)           R(NSURF)
C   IVAR    : TYPE OF SUBGEOMETRY                        I
C             IVAR = 1 FILLED TUBE
C             IVAR = 2 ROD CLUSTER WITHOUT INNER REGION
C             IVAR = 3 TUBE
C             IVAR = 4 ROD CLUSTER WITH INNER REGION
C   NTRAK   : NUMBER OF TRACK FOR EACH OF THE 5 TYPES    I(6)
C             OF PROBABILITIES
C             NMTRAK=SUM(NTRAK(I),I=1,6)
C   TRAK    : TRACKS AND WEIGHT FOR INTEGRATION          R(4,NMTRAK)
C             FOR TRAK IL:   TRAK(1,IL) WEIGHT
C                            TRAK(2,IL) LENGHT
C                            TRAK(3,IL) DIR(MU)
C                            TRAK(4,IL) DIR(NU)
C             TRAK  IL=NDEP1,NFIN1       -> PS3S3 (ITYP=1)
C                      NDEP1=1      ;  NFIN1=NTRAK(1)
C             TRAK  IL=NDEP2,NFIN2       -> PS3S1 (ITYP=2)
C                      NDEP2=NFIN1+1;  NFIN2=NFIN1+NTRAK(2)
C             TRAK  IL=NDEP3,NFIN3       -> PS3S2 (ITYP=3)
C                      NDEP3=NFIN2+1;  NFIN3=NFIN2+NTRAK(3)
C             TRAK  IL=NDEP4,NFIN4       -> PS2S3 (ITYP=4)
C                      NDEP4=NFIN3+1;  NFIN4=NFIN3+NTRAK(4)
C             TRAK  IL=NDEP5,NFIN5       -> PS2S1 (ITYP=5)
C                      NDEP5=NFIN4+1;  NFIN5=NFIN4+NTRAK(5)
C             TRAK  IL=NDEP6,NFIN6       -> PS2S2 (ITYP=6)
C                      NDEP6=NFIN5+1;  NFIN6=NFIN5+NTRAK(6)
C   VD        SERIES EXPANSION OF BICKLEY FUNCTIONS
C             CONTRIBUTION                              R(ITYP,J,K,L)
C             ITYP -- FOR TYPE OF PROBABILITIES (ITYP=1,6)
C             J    -- INITIAL DIRECTION         (J=1,2)
C             K    -- FINAL DIRECTION           (K=1,2)
C             L    -- ORDER OF SERIES EXPANSION (L=1,3)
C   SIGT    : REGION TOTAL CROSS SECTION                 R
C   TRONC   : SERIES EXPANSION TRONCATURE                R
C   PII     : COLLISION PROBABILITY                      R
C   PIS     : LEAKAGE PROBABILITY                        R(NPRB)
C   PSS     : TRANSMISSION PROBABILITY                   R(NPRB*NPRB)
C             STORED AS ((PSS(JJ,II),II=1,NPRB),JJ=1,NPRB)
C SURFACE NOTATION FOR PIS AND PSS
C              ROD+INNER     NO-RODS   NO-INNER   NO-RODS+NO-INNER
C   IVAR=          4            3         2              1
C DP0, IROT=1
C   SURFACE 1    INNER        INNER     ROD            OUTER
C   SURFACE 2    ROD          OUTER     OUTER          -----
C   SURFACE 3    OUTER        -----     -----          -----
C DP1, IROT=1
C   HOMOGENEOUS CURRENT COMPONENT
C   SURFACE 1    INNER        INNER     ROD            OUTER
C   SURFACE 3    ROD          OUTER     OUTER          -----
C   SURFACE 5    OUTER        -----     -----          -----
C   LINEAR  CURRENT COMPONENTS
C   SURFACE 2    INNER        INNER     ROD            OUTER
C   SURFACE 4    ROD          OUTER     OUTER          -----
C   SURFACE 6    OUTER        -----     -----          -----
C DP0, IROT=2
C   SURFACE 1    INNER        INNER     ROD/2          OUTER
C   SURFACE 2    ROD/2        OUTER     ROD/2          -----
C   SURFACE 3    ROD/2        -----     OUTER          -----
C   SURFACE 4    OUTER        -----     -----          -----
C DP1, IROT=2
C   HOMOGENEOUS CURRENT COMPONENT
C   SURFACE 1    INNER        INNER     ROD/2          OUTER
C   SURFACE 3    ROD/2        OUTER     ROD/2          -----
C   SURFACE 5    ROD/2        -----     OUTER          -----
C   SURFACE 7    OUTER        -----     -----          -----
C   LINEAR  CURRENT COMPONENTS
C   SURFACE 2    INNER        INNER     ROD/2          OUTER
C   SURFACE 4    ROD/2        OUTER     ROD/2          -----
C   SURFACE 6    ROD/2        -----     OUTER          -----
C   SURFACE 8    OUTER        -----     -----          -----
C
C 3-  ROUTINES CALLED BY THIS PROGRAMME:
C
C   XCSCVS  : COMPLETE CP USING CONSERVATION AND SYMMETRY RELATIONS
C
C 4-  PARAMETERS IN ROUTINE:
C
C   IPOS0   : PSS POSITION VECTOR           I(ISURI,IVAR,IODP,IROT)
C             IODP  -- ORDER IN DP EXPANSION  (IODP=0,1)
C             IVAR  -- TYPE OF GEOMETRY       (IVAR=1,NVAR)
C             ISURI -- SURFACE NUMBER         (ISURI=1,9)
C             IROT  -- TYPE DE RECONSTRUCTION (IROT=1,2)
C
C---------------------------   XCSINT  --------------------------------
C
      SAVE       IPOS0
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (NTYP=6,PI=3.1415926535898,SQ2=1.414213562373095,
     >           PIO2=PI/2.0,NDIR=2,NOR=3,NSUR=3,NVAR=4)
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
      INTEGER    IAPPR,IROT,NRODS,NSURF,NTRAK(NTYP),IPOS0(NTYP,NVAR,2,2)
      REAL       CHORD(*),TRAK(4,*),SIGT,TRONC,
     >           VD(NTYP,NDIR,NDIR,NOR),PII,PIS(*),PSS(*)
      DATA IPOS0/ 1, 0, 0, 0, 0, 0,   4, 0, 2, 3, 0, 1,
     >            4, 2, 0, 0, 0, 0,   9, 3, 6, 8, 2, 5,
     >            1, 0, 0, 0, 0, 0,  11, 0, 3, 9, 0, 1,
     >           11, 3, 0, 0, 0, 0,  29, 5,17,27, 3,15,
     >            1, 0, 0, 0, 0, 0,   9, 0, 6, 8, 0, 1,
     >            4, 2, 0, 0, 0, 0,  16, 4,12,15, 2, 6,
     >            1, 0, 0, 0, 0, 0,  29, 0,17,27, 0, 1,
     >           11, 3, 0, 0, 0, 0,  55, 7,39,53, 3,19/
      NPSS=NSURF*NSURF
      CHRD=0.0
      DO 90 ISU=1,NSURF
        CHRD=CHORD(ISU)+CHRD
 90   CONTINUE
      CHRD=CHRD/FLOAT(NSURF)
C----
C  INITIALIZE PSS AND PIS TO 0.0
C----
      DO 100 ISS=1,NPSS
        PSS(ISS)=0.0
 100  CONTINUE
C----
C  INTEGRATE PROBABILITIES
C----
      IF(CHRD*SIGT.LT.TRONC) THEN
C----
C  USE SERIES EXPANSION IN VOID REGIONS FOR PROBABILITIES
C  EVALUATE DP0 CONTRIBUTIONS IN ALL CASES
C  ORDER SIGT**2, SIGT**1 AND SIGT**0 CONTRIBUTIONS
C----
        PII=0.0
        XSIGT=1.0
        DO 200 IOR=NOR,1,-1
          DO 210 ITYP=1,NTYP
            IP00=IPOS0(ITYP,IVAR,IAPPR,IROT)
            IF( (ITYP.GT.3) .AND. (IROT.EQ.2) ) THEN
              FACT=2.0
            ELSE
              FACT=1.0
            ENDIF
            IF(IP00.GT.0) THEN
              PSS00=FACT*VD(ITYP,1,1,IOR)
              PSS(IP00)=SIGT*PSS(IP00)+PSS00
C----
C  EVALUATE DP1 CONTRIBUTIONS WHEN REQUIRED
C----
              IF(IAPPR.EQ.2) THEN
                IP10=IP00+1
                IP11=IP10+NSURF
                IP01=IP11-1
                IF(ITYP.EQ.6) THEN
                  PSS01=0.5*FACT*(VD(ITYP,1,2,IOR)+VD(ITYP,2,1,IOR))
                  PSS10=PSS01
                ELSE
                  PSS01=FACT*VD(ITYP,1,2,IOR)
                  PSS10=FACT*VD(ITYP,2,1,IOR)
                ENDIF
                PSS11=FACT*VD(ITYP,2,2,IOR)
                PSS(IP11)=SIGT*PSS(IP11)+18.0*PSS11
     >                   -12.0*(PSS01+PSS10)+8.0*PSS00
                PSS(IP10)=SIGT*PSS(IP10)
     >                   +SQ2*(3.0*PSS10-2.0*PSS00)
                PSS(IP01)=SIGT*PSS(IP01)
     >                   +SQ2*(3.0*PSS01-2.0*PSS00)
              ENDIF
            ENDIF
 210      CONTINUE
C----
C  COMPLETE CP BY CONSERVATIONS AND SYMMETRY RELATIONS
C----
          IF(IOR.GT.1) THEN
            DO 190 ISTPSS=1,NSURF
              PIS(ISTPSS)=0.0
 190        CONTINUE
          ENDIF
          CALL XCSCVS(IAPPR,IROT,IOR,IVAR,NSURF,CHORD,XSIGT,
     >                PII,PIS,PSS)
 200    CONTINUE
      ELSE
C----
C  INTEGRATE EXPLICITELY TRANSMISSION PROBABILITIES
C  EVALUATE DP0 CONTRIBUTIONS IN ALL CASES
C----
        ITFIN=0
        DO 300 ITYP=1,NTYP
          IF( (ITYP.GT.3) .AND. (IROT.EQ.2) ) THEN
            FACT=2.0
          ELSE
            FACT=1.0
          ENDIF
          ITDEB=ITFIN+1
          ITFIN=ITFIN+NTRAK(ITYP)
          IP00=IPOS0(ITYP,IVAR,IAPPR,IROT)
          IF(IP00.GT.0) THEN
            DO 310 ITRK=ITDEB,ITFIN
              XL=TRAK(2,ITRK)*SIGT
              K=MIN(NINT(XL*PAS3),MKI3)
              ZI=FACT*(BI3(K,1)+XL*(BI3(K,2)+XL*BI3(K,3)))
              PSS(IP00)=PSS(IP00)+TRAK(1,ITRK)*ZI
 310        CONTINUE
          ENDIF
 300    CONTINUE
C----
C  EVALUATE DP1 CONTRIBUTIONS WHEN REQUIRED
C----
        IF(IAPPR.EQ.2) THEN
          ITFIN=0
          DO 320 ITYP=1,NTYP
            IF( (ITYP.GT.3) .AND. (IROT.EQ.2) ) THEN
              FACT=2.0
            ELSE
              FACT=1.0
            ENDIF
            ITDEB=ITFIN+1
            ITFIN=ITFIN+NTRAK(ITYP)
            IP00=IPOS0(ITYP,IVAR,IAPPR,IROT)
            IF(IP00.GT.0) THEN
              IP10=IP00+1
              IP11=IP10+NSURF
              IP01=IP11-1
              DO 330 ITRK=ITDEB,ITFIN
                XL=TRAK(2,ITRK)*SIGT
                K=MIN(NINT(XL*PAS4),MKI4)
                ZI=FACT*(BI4(K,1)+XL*(BI4(K,2)+XL*BI4(K,3)))*
     >             TRAK(1,ITRK)
                IF(ITYP.EQ.6) THEN
                  PSSAVG=0.5*ZI*(TRAK(4,ITRK)+TRAK(3,ITRK))
                  PSS(IP01)=PSS(IP01)+PSSAVG
                  PSS(IP10)=PSS(IP10)+PSSAVG
                ELSE
                  PSS(IP01)=PSS(IP01)+TRAK(4,ITRK)*ZI
                  PSS(IP10)=PSS(IP10)+TRAK(3,ITRK)*ZI
                ENDIF
                K=MIN(NINT(XL*PAS5),MKI5)
                ZI=FACT*(BI5(K,1)+XL*(BI5(K,2)+XL*BI5(K,3)))*
     >             TRAK(1,ITRK)
                PSS(IP11)=PSS(IP11)+TRAK(4,ITRK)*TRAK(3,ITRK)*ZI
 330          CONTINUE
              PSS(IP11)=18.0*PSS(IP11)-12.0*(PSS(IP01)+PSS(IP10))
     >                 +8.0*PSS(IP00)
              PSS(IP10)=SQ2*(3.0*PSS(IP10)-2.0*PSS(IP00))
              PSS(IP01)=SQ2*(3.0*PSS(IP01)-2.0*PSS(IP00))
            ENDIF
 320      CONTINUE
        ENDIF
C----
C  COMPLETE CP BY CONSERVATIONS AND SYMMETRY RELATIONS
C----
        PII=1.0
        DO 191 ISTPSS=1,NSURF,IAPPR
          PIS(ISTPSS)=1.0
 191    CONTINUE
        IF(IAPPR.GT.1) THEN
          DO 192 ISTPSS=2,NSURF,IAPPR
            PIS(ISTPSS)=0.0
 192      CONTINUE
        ENDIF
        IOR=3
        CALL XCSCVS(IAPPR,IROT,IOR,IVAR,NSURF,CHORD,SIGT,
     >              PII,PIS,PSS)
      ENDIF
      RETURN
      END
