*DECK XCSNUM
      SUBROUTINE XCSNUM(NSOUT,NVOL,NBAN,NRT,MSROD,IPRT,
     >                  IROT,IAPP,NSURF,RAN,COTE,NRODS,
     >                  RODR,NRINFO,MATALB,VOLSUR,ZCODE,ISS,
     >                  VOL,ISURF,VOLW,CHORD,IFR,MIX,ALB,DVX)
C
C----------------------------------------------------------------------
C
C 1-  SUBROUTINE STATISTICS:
C
C          NAME      -> XCSNUM
C          USE       -> SET CURRENT INFORMATION FOR 2-D CLUSTER
C          DATE      -> 15-06-1990
C          AUTHOR    -> G. MARLEAU
C
C 2-  PARAMETERS:
C
C INPUT
C  NSOUT   : NUMBER OF OUTER SURFACE                     I
C  NVOL    : MAXIMUM NUMBER OF REGIONS                   I
C  NBAN    : NUMBER OF CONCENTRIC REGIONS                I
C  NRT     : NUMBER OF ROD TYPES                         I
C  MSROD   : MAXIMUM NUMBER OF SUBRODS PER RODS          I
C  IPRT    : IMPRESSION LEVEL                            I
C  IROT    : TYPE OF JPM RECONSTRUCTION                  I
C            IROT = 1 -> DIRECT JPM
C            IROT = 2 -> SURFACE FRACTIONING JPM
C  IAPP    : LEVEL OF DP APPROXIMATION                   I
C            NOT USED FOR CP
C            IAPP = 1 -> DP0 ALL
C            IAPP = 2 -> DP1 ALL (DEFAULT)
C            IAPP = 3 -> DP1 INSIDE DP0 OUTSIDE
C  NSURF   : TOTAL NUMBER OF CURRENTS                    I
C  RAN     : RADIUS OF ANNULAR REGIONS                   R(NBAN)
C  COTE    : Y-SIDE OF RECTANGLE                         R
C  NRODS   : INTEGER DESCRIPTION OF ROD OF A GIVEN TYPE  I(3,NRT)
C            NRODS(1,IRT) = NUMBER OF ROD
C            NRODS(2,IRT) = NUMBER OF SUBRODS IN ROD
C            NRODS(3,IRT) = ASSOCIATED CONCENTRIC REGION
C  RODR    : SUBROD RADIUS                               R(MSROD,NRT)
C  NRINFO  : ANNULAR REGION CONTENT                      I(2,NBAN)
C            NRINFO(1,IAN) = NEW REGION NUMBER
C            NRINFO(2,IAN) = +N CLUSTER NUMBER
C                          = 0 NO CLUSTER ASSOCIATED
C                          = -N CLUSTER AT CENTER
C  MATALB  : ALBEDO-MATERIAL OF REGIONS                  I(-NSOUT:NVOL)
C  VOLSUR  : SURFACE/4-VOLUME OF REGIONS                 R(-NSOUT:NVOL)
C  ZCODE   : ALBEDO                                      R(6)
C OUTPUT
C  ISS     : MATERIAL OF REGION                          I(NVOL)
C  VOL     : VOLUME OF REGION                            R(NVOL)
C  ISURF   : NUMBER OF SURFACES PER REGIONS              I(NVOL)
C  VOLW    : WEIGHT ASSOCIATED WITH EACH BLOCK.          R(NVOL)
C  CHORD   : MEAN CHORD FOR EACH CURRENT                 R(NSURF)
C  IFR     : INDEX-NUMBER OF IN-CURRENTS                 I(NSURF)
C  MIX     : INDEX NUMBER OF OUT CURRENT                 I(NSURF)
C  ALB     : TRANSMISSION/ALBEDO FOR EACH CURRENT        R(NSURF)
C  DVX     : WEIGHT ASSOCIATED FOR EACH CURRENT          R(NSURF)
C
C----------------------------------------------------------------------
C
      PARAMETER (IUNOUT=6,NMCOD=6,PI=3.1415926535898)
      CHARACTER  CN*1
      INTEGER    NSOUT,NVOL,NBAN,NRT,MSROD,IPRT,IROT,IAPP,NSURF,
     >           NRODS(3,NRT),NRINFO(2,NBAN),MATALB(-NSOUT:NVOL),
     >           ISS(NVOL),ISURF(NVOL),IFR(NSURF),MIX(NSURF)
      REAL       RAN(NBAN),COTE,RODR(MSROD,NRT),VOLSUR(-NSOUT:NVOL),
     >           ZCODE(NMCOD),VOL(NVOL),VOLW(NVOL),CHORD(NSURF),
     >           ALB(NSURF),DVX(NSURF),SUR(3),SURS(6)
C----
C  FILL ISS AND VOL VECTORS
C----
      DO 100 IREG=1,NVOL
        ISS(IREG)=MATALB(IREG)
        VOL(IREG)=VOLSUR(IREG)
 100  CONTINUE
      IF(NSOUT.EQ.1) THEN
        NTAN=NBAN
      ELSE
        NTAN=NBAN-1
      ENDIF
C----
C  FILL IN CHORD, ISS, ISURF, IFR, VOLW, MIX AND IDX
C  VECTORS
C----
      IF(IAPP.EQ.3) THEN
        IAPPR=2
      ELSE
        IAPPR=IAPP
      ENDIF
      NSUR=2
      JSUR=2
      IREG=0
      ICHRD=0
      IPOS=0
      DO 200 IAN=1,NTAN
        IF(NRINFO(2,IAN).EQ.0) THEN
          IPOS=IPOS+1
          SUR(2)=2.0*PI*RAN(IAN)
          VOLW(IPOS)=1.0
          DO 210 ISUR=JSUR,NSUR
            CHRD=4.0*VOLSUR(IPOS)/SUR(ISUR)
            IADD=(-1)**ISUR*IAPPR
            DO 220 IP=1,IAPPR
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              MIX(ICHRD)=ICHRD
              IFR(ICHRD)=ICHRD+IADD
              DVX(ICHRD)=1.0
              ALB(ICHRD)=1.0
 220        CONTINUE
 210      CONTINUE
          SUR(1)=SUR(2)
        ELSE IF(NRINFO(2,IAN).LT.0) THEN
          IRT=ABS(NRINFO(2,IAN))
          XNROD=FLOAT(NRODS(1,IRT))
          FCHRD=4.0/XNROD
          NSUR=3
          FACTS=2.0*PI
          IFSTS=ICHRD
          IF(JSUR.EQ.2) THEN
            IFSTO=0
          ELSE
            IFSTO=IAPPR
          ENDIF
          NSURS=2
          JSURS=2
          DO 301 ISV=1,NRODS(2,IRT)
            IPOS=IPOS+1
            SURS(2)=FACTS*RODR(ISV,IRT)
            VOLW(IPOS)=1.0
            DO 311 ISURS=JSURS,NSURS
              CHRD=FCHRD*VOLSUR(IPOS)/SURS(ISURS)
              IADDS=(-1)**ISURS*IAPPR
              DO 321 IP=1,IAPPR
                ICHRD=ICHRD+1
                ICHRDS=ICHRD+IFSTO
                CHORD(ICHRD)=CHRD
                MIX(ICHRD)=ICHRDS
                IFR(ICHRD)=ICHRDS+IADDS
                DVX(ICHRD)=1.0
                ALB(ICHRD)=1.0
 321          CONTINUE
 311        CONTINUE
            SURS(1)=SURS(2)
            ISURF(IPOS)=(NSURS-JSURS+1)*IAPPR
            JSURS=1
 301      CONTINUE
          IPOS=IPOS+1
          SUR(2)=SURS(2)*XNROD
          SUR(3)=2.0*PI*RAN(IAN)
          VOLW(IPOS)=1.0
          DO 331 ISUR=JSUR,1
            CHRD=4.0*VOLSUR(IPOS)/SUR(ISUR)
            DO 341 IP=1,IAPPR
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              IFSTS=IFSTS+1
              MIX(ICHRD)=IFSTS
              IFR(ICHRD)=IFSTS-IAPPR
              DVX(ICHRD)=1.0
              ALB(ICHRD)=1.0
 341        CONTINUE
 331      CONTINUE
          IADDS=IAPPR
          CHRD=4.0*VOLSUR(IPOS)/SUR(2)
          DO 361 IP=1,IAPPR
            ICHRD=ICHRD+1
            CHORD(ICHRD)=CHRD
            MIX(ICHRD)=ICHRD
            IFR(ICHRD)=ICHRD-IADDS
            DVX(ICHRD)=1.0
            ALB(ICHRD)=1.0
 361      CONTINUE
          IADDS=IAPPR
          CHRD=4.0*VOLSUR(IPOS)/SUR(3)
          DO 371 IP=1,IAPPR
            ICHRD=ICHRD+1
            CHORD(ICHRD)=CHRD
            MIX(ICHRD)=ICHRD
            IFR(ICHRD)=ICHRD+IADDS
            DVX(ICHRD)=1.0
            ALB(ICHRD)=1.0
 371      CONTINUE
          SUR(1)=SUR(3)
        ELSE
          NSUR=2+IROT
          FACTS=2.0*PI/FLOAT(IROT)
          IFSTS=ICHRD
          IF(JSUR.EQ.2) THEN
            IFSTO=0
          ELSE
            IFSTO=IAPPR
          ENDIF
          NSURS=2
          JSURS=2
          IRT=ABS(NRINFO(2,IAN))
          XNROD=FLOAT(NRODS(1,IRT))
          FCHRD=4.0/XNROD
          DO 300 ISV=1,NRODS(2,IRT)
            IPOS=IPOS+1
            SURS(2)=FACTS*RODR(ISV,IRT)
            VOLW(IPOS)=1.0
            DO 310 ISURS=JSURS,NSURS
              CHRD=FCHRD*VOLSUR(IPOS)/SURS(ISURS)
              IADDS=(-1)**ISURS*IAPPR*IROT
              DO 315 IZZ=1,IROT
                DO 320 IP=1,IAPPR
                  ICHRD=ICHRD+1
                  ICHRDS=ICHRD+IFSTO
                  CHORD(ICHRD)=CHRD
                  MIX(ICHRD)=ICHRDS
                  IFR(ICHRD)=ICHRDS+IADDS
                  DVX(ICHRD)=1.0
                  ALB(ICHRD)=1.0
 320            CONTINUE
 315          CONTINUE
 310        CONTINUE
            SURS(1)=SURS(2)
            ISURF(IPOS)=IROT*(NSURS-JSURS+1)*IAPPR
            JSURS=1
 300      CONTINUE
          IPOS=IPOS+1
          SUR(2)=SURS(2)*XNROD
          SUR(3)=2.0*PI*RAN(IAN)
          VOLW(IPOS)=1.0
          DO 330 ISUR=JSUR,1
            CHRD=4.0*VOLSUR(IPOS)/SUR(ISUR)
            DO 340 IP=1,IAPPR
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              IFSTS=IFSTS+1
              MIX(ICHRD)=IFSTS
              IFR(ICHRD)=IFSTS-IAPPR
              DVX(ICHRD)=1.0
              ALB(ICHRD)=1.0
 340        CONTINUE
 330      CONTINUE
          IADDS=IROT*IAPPR
          CHRD=4.0*VOLSUR(IPOS)/SUR(2)
          DO 350 IZZ=1,IROT
            DO 360 IP=1,IAPPR
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              MIX(ICHRD)=ICHRD
              IFR(ICHRD)=ICHRD-IADDS
              DVX(ICHRD)=1.0
              ALB(ICHRD)=1.0
 360        CONTINUE
 350      CONTINUE
          IADDS=IAPPR
          CHRD=4.0*VOLSUR(IPOS)/SUR(3)
          DO 370 IP=1,IAPPR
            ICHRD=ICHRD+1
            CHORD(ICHRD)=CHRD
            MIX(ICHRD)=ICHRD
            IFR(ICHRD)=ICHRD+IADDS
            DVX(ICHRD)=1.0
            ALB(ICHRD)=1.0
 370      CONTINUE
          SUR(1)=SUR(3)
        ENDIF
        ISURF(IPOS)=(NSUR-JSUR+1)*IAPPR
        NSUR=2
        JSUR=1
 200  CONTINUE
C----
C  FINAL REGION ANALYSIS FOR RECTANGLE AND HEXAGONE
C----
      IF(NSOUT.GT.1) THEN
        IPOS=IPOS+1
        ISURF(IPOS)=IAPPR+NSOUT*(2*IAPPR-1)
        DO 201 IISS=1,NSOUT
          SURS(IISS)=RAN(NBAN)
 201    CONTINUE
        IF(NSOUT.EQ.4) THEN
          SURS(1)=COTE
          SURS(3)=COTE
        ENDIF
        VOLW(IPOS)=1.0
        CHRD=4.0*VOLSUR(IPOS)/SUR(1)
        IADD=-IAPPR
        DO 500 IP=1,IAPPR
          ICHRD=ICHRD+1
          CHORD(ICHRD)=CHRD
          MIX(ICHRD)=ICHRD
          IFR(ICHRD)=ICHRD+IADD
          DVX(ICHRD)=1.0
          ALB(ICHRD)=1.0
 500    CONTINUE
        IF(IAPP.EQ.3) THEN
          DO 550 IXX=1,NSOUT
            CHRD=4.0*VOLSUR(IPOS)/SURS(IXX)
            ICHRD=ICHRD+1
            CHORD(ICHRD)=CHRD
            MIX(ICHRD)=ICHRD
            IFR(ICHRD)=ICHRD
            DVX(ICHRD)=1.0
            ALB(ICHRD)=ZCODE(IXX)
            DO 560 IP=2,2*IAPPR-1
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              MIX(ICHRD)=ICHRD
              IFR(ICHRD)=ICHRD
              DVX(ICHRD)=1.0
              ALB(ICHRD)=0.0
 560        CONTINUE
 550      CONTINUE
        ELSE
          DO 510 IXX=1,NSOUT
            CHRD=4.0*VOLSUR(IPOS)/SURS(IXX)
            DO 520 IP=1,2*IAPPR-1
              ICHRD=ICHRD+1
              CHORD(ICHRD)=CHRD
              MIX(ICHRD)=ICHRD
              IFR(ICHRD)=ICHRD
              DVX(ICHRD)=1.0
              ALB(ICHRD)=ZCODE(IXX)
 520        CONTINUE
 510      CONTINUE
        ENDIF
      ELSE
C----
C  FINAL ALBEDO CONTRIBUTION AND FINAL CURRENT CONTACT
C  FOR CYLINDRICAL GEOMETRY
C----
        ICHRD=ICHRD-IAPPR+1
        IFR(ICHRD)=ICHRD
        ALB(ICHRD)=ZCODE(2)
        IF(IAPP.EQ.3) THEN
          ICHRD=ICHRD+1
          IFR(ICHRD)=ICHRD
          ALB(ICHRD)=0.0
        ELSE IF(IAPP.EQ.2) THEN
          ICHRD=ICHRD+1
          IFR(ICHRD)=ICHRD
          ALB(ICHRD)=ZCODE(2)
        ENDIF
      ENDIF
C----
C  PRINT REGION VOLUME AND MATERIAL INFORMATION WHEN REQUIRED
C----
      IF(IPRT.GT.2) THEN
        WRITE(IUNOUT,6000)
        IS=0
        IREG=0
        DO 400 IAN=1,NTAN
          IREG=IREG+1
          NC=ISURF(IREG)
          IF(NRINFO(2,IAN).EQ.0) THEN
            IF(IAN.EQ.1) THEN
              WRITE(IUNOUT,6001) IAN,IREG,'A O',1,MIX(IS+1),IFR(IS+1),
     >        CHORD(IS+1),DVX(IS+1),ALB(IS+1)
              IF(IAPPR.GT.1) THEN
                WRITE(IUNOUT,6003) ('A O',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
              ENDIF
              IS=IS+IAPPR
            ELSE
              WRITE(IUNOUT,6001) IAN,IREG,'A I',1,MIX(IS+1),IFR(IS+1),
     >        CHORD(IS+1),DVX(IS+1),ALB(IS+1)
              IF(IAPPR.GT.1) THEN
                WRITE(IUNOUT,6003) ('A I',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
              ENDIF
              IS=IS+IAPPR
              WRITE(IUNOUT,6003) ('A O',JA,MIX(IS+JA),IFR(IS+JA),
     >        CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
              IS=IS+IAPPR
            ENDIF
          ELSE IF(NRINFO(2,IAN).LT.0) THEN
            IRT=ABS(NRINFO(2,IAN))
            DO 410 ISV=1,NRODS(2,IRT)
              IF(ISV.EQ.1) THEN
                WRITE(IUNOUT,6002) ISV,IREG,'R1O',1,MIX(IS+1),IFR(IS+1),
     >          CHORD(IS+1),DVX(IS+1),ALB(IS+1)
                IF(IAPPR.GT.1) THEN
                  WRITE(IUNOUT,6003) ('R1O',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
                ENDIF
                IS=IS+IAPPR
              ELSE
                WRITE(IUNOUT,6002) ISV,IREG,'R1I',1,MIX(IS+1),IFR(IS+1),
     >          CHORD(IS+1),DVX(IS+1),ALB(IS+1)
                IF(IAPPR.GT.1) THEN
                  WRITE(IUNOUT,6003) ('R1I',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
                ENDIF
                IS=IS+IAPPR
                WRITE(IUNOUT,6003) ('R1O',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
                IS=IS+IAPPR
              ENDIF
              IREG=IREG+1
              NC=ISURF(IREG)
 410        CONTINUE
            WRITE(IUNOUT,6001) IAN,IREG,'A I',1,MIX(IS+1),IFR(IS+1),
     >      CHORD(IS+1),DVX(IS+1),ALB(IS+1)
            IF(IAPPR.GT.1) THEN
              WRITE(IUNOUT,6003) ('A I',JA,MIX(IS+JA),IFR(IS+JA),
     >        CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
            ENDIF
            IS=IS+IAPPR
            WRITE(IUNOUT,6003) ('A O',JA,MIX(IS+JA),IFR(IS+JA),
     >      CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
            IS=IS+IAPPR
          ELSE
            IRT=NRINFO(2,IAN)
            DO 415 ISV=1,NRODS(2,IRT)
              IF(ISV.EQ.1) THEN
                WRITE(IUNOUT,6002) ISV,IREG,'R1O',1,MIX(IS+1),IFR(IS+1),
     >          CHORD(IS+1),DVX(IS+1),ALB(IS+1)
                IF(IAPPR.GT.1) THEN
                  WRITE(IUNOUT,6003) ('R1O',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
                ENDIF
                IS=IS+IAPPR
                IF(IROT.EQ.2) THEN
                  WRITE(IUNOUT,6003) ('R2O',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
                  IS=IS+IAPPR
                ENDIF
              ELSE
                WRITE(IUNOUT,6002) ISV,IREG,'R1I',1,MIX(IS+1),IFR(IS+1),
     >          CHORD(IS+1),DVX(IS+1),ALB(IS+1)
                IF(IAPPR.GT.1) THEN
                  WRITE(IUNOUT,6003) ('R1I',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
                ENDIF
                IS=IS+IAPPR
                IF(IROT.EQ.2) THEN
                  WRITE(IUNOUT,6003) ('R2I',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
                  IS=IS+IAPPR
                ENDIF
                WRITE(IUNOUT,6003) ('R1O',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
                IS=IS+IAPPR
                IF(IROT.EQ.2) THEN
                  WRITE(IUNOUT,6003) ('R2O',JA,MIX(IS+JA),IFR(IS+JA),
     >            CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
                  IS=IS+IAPPR
                ENDIF
              ENDIF
              IREG=IREG+1
              NC=ISURF(IREG)
 415        CONTINUE
            IF(IAN.NE.1) THEN
              WRITE(IUNOUT,6001) IAN,IREG,'A I',1,MIX(IS+1),IFR(IS+1),
     >        CHORD(IS+1),DVX(IS+1),ALB(IS+1)
              IF(IAPPR.GT.1) THEN
                WRITE(IUNOUT,6003) ('A I',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
              ENDIF
              IS=IS+IAPPR
              WRITE(IUNOUT,6003) ('C1O',JA,MIX(IS+JA),IFR(IS+JA),
     >        CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
            ELSE
              WRITE(IUNOUT,6001) IAN,IREG,'C1O',1,MIX(IS+1),IFR(IS+1),
     >        CHORD(IS+1),DVX(IS+1),ALB(IS+1)
              IF(IAPPR.GT.1) THEN
                WRITE(IUNOUT,6003) ('C1O',JA,MIX(IS+JA),IFR(IS+JA),
     >          CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
              ENDIF
            ENDIF
            IS=IS+IAPPR
            IF(IROT.EQ.2) THEN
              WRITE(IUNOUT,6003) ('C2O',JA,MIX(IS+JA),IFR(IS+JA),
     >        CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
              IS=IS+IAPPR
            ENDIF
            WRITE(IUNOUT,6003) ('A O',JA,MIX(IS+JA),IFR(IS+JA),
     >      CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,IAPPR)
            IS=IS+IAPPR
          ENDIF
 400    CONTINUE
C----
C  LAST REGION FOR SQUARE AND HEXAGONES
C----
        IF(NSOUT.GT.1) THEN
          IREG=IREG+1
          NC=ISURF(IREG)
          WRITE(IUNOUT,6001) IAN,IREG,'A I',1,MIX(IS+1),IFR(IS+1),
     >    CHORD(IS+1),DVX(IS+1),ALB(IS+1)
          IF(IAPPR.GT.1) THEN
            WRITE(IUNOUT,6003) ('A I',JA,MIX(IS+JA),IFR(IS+JA),
     >      CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=2,IAPPR)
          ENDIF
          IS=IS+IAPPR
          IF(NSOUT.EQ.4) THEN
            CN='R'
          ELSE
            CN='H'
          ENDIF
          DO 420 ISOUT=1,NSOUT
            WRITE(IUNOUT,6004) (CN,ISOUT,'O',JA,MIX(IS+JA),IFR(IS+JA),
     >      CHORD(IS+JA),DVX(IS+JA),ALB(IS+JA),JA=1,2*IAPPR-1)
            IS=IS+2*IAPPR-1
 420      CONTINUE
        ENDIF
      ENDIF
      RETURN
C----
C  GEOMETRY DESCRIPTION FORMATS
C----
 6000 FORMAT(//1X,'CLUSTER JPM CURRENT COUPLING.'/
     >1X,'ANN',2X,'ROD',2X,'REG',2X,'SURFACE TYPE',
     >2X,'DP-DIRECTION',2X,'   J+/   J-',2X,'      4V/S    ',
     >2X,'    WEIGHT    ',2X,'    ALBEDO')
 6001 FORMAT(1X,I3,6X,I4,7X,A3,11X,I2,7X,I5,'/',I5,2X,1P,3E15.7)
 6002 FORMAT(5X,I4,1X,I4,7X,A3,11X,I2,7X,I5,'/',I5,2X,1P,3E15.7)
 6003 FORMAT((21X,A3,11X,I2,7X,I5,'/',I5,2X,1P,3E15.7))
 6004 FORMAT((21X,A1,I1,A1,11X,I2,7X,I5,'/',I5,2X,1P,3E15.7))
      END
