*DECK XCSRNU
      SUBROUTINE XCSRNU(IAPPR,NSURF,PIS,PSS)
C
C---------------------------  XCSRNU  ---------------------------------
C
C 1-  PROGRAMME STATISTICS:
C
C   NAME       :  XCSRNU
C   FUNCTION   :  RENUMBER TRANSMISSION AND LEAKAGE PROBABILITY MATRIX
C                 FOR DOUBLING SURFACE
C   VERSION    :  1.0
C   REVISED    :  22-10-1990
C   AUTHOR     :  G.MARLEAU
C
C 2-  PROGRAMME PARAMETERS:
C
C   IAPPR   : ANGULAR EXPANSION ORDER                    I
C             IAPPR = 1 -> DP0 ALL
C             IAPPR = 2 -> DP1 ALL OR DP1 INSIDE DP0 OUTSIDE
C   NSURF   : NUMBER OF SURFACES
C   PIS     : LEAKAGE PROBABILITY                        R(NSURF)
C   PSS     : TRANSMISSION PROBABILITY                   R(NSURF*
C                                                          NSURF)
C
C 3-  PARAMETERS IN ROUTINE:
C
C   IPOSI   : LOCATION OF DP00 AND DP01 TERMS IN ORIGINAL BLOCK
C             MATRIX WITH RESPECT TO LAST ELEMENT IN BLOCK
C   IPOSF   : LOCATION OF DP00 AND DP01 TERMS IN FINAL BLOCK
C             MATRIX WITH RESPECT TO LAST ELEMENT IN BLOCK
C   IBLK    : POSITION OF ORIGINAL BLOCKS IN SUB BLOCK OF
C             FINAL BLOCK MATRIX
C
C---------------------------   XCSRNU  --------------------------------
C
      INTEGER  IAPPR,NSURF,IPOSI(2,2,2),IPOSF(2,2,2),IBLK(0:1,2,2),IL(4)
      LOGICAL  LEVEN,LZERO
      REAL     PIS(*),PSS(*),PTSS(16)
      DATA     IPOSI /0,-1,-2,-3,0,-1,-4,-5/
      DATA     IPOSF /0,-1,-4,-5,0,-1,-8,-9/
      DATA     IBLK  /0,1, 2,0, 3,0, 0,4/
C----
C  DEFINE BLOCKS LOCATION NUMBERS
C----
      NBLK=NSURF/(2*IAPPR)
      NS2=NSURF*NSURF
      NSFZ=NS2
      NSFK=NSFZ/4
      IAPPR2=2*IAPPR
      NNZD=NBLK*IAPPR2*IAPPR
C----
C  FILL IL MATRIX FOR POSITION OF LAST BLOCK ELEMENT
C----
      IF(NBLK.EQ.1) THEN
        IF(IAPPR.EQ.1) THEN
          IL(1)=1
        ELSE
          IL(1)=4
        ENDIF
      ELSE
        IF(IAPPR.EQ.1) THEN
          DO 300 JJ=1,4
            IL(JJ)=JJ
 300      CONTINUE
        ELSE
          IL(1)= 6
          IL(2)= 8
          IL(3)=14
          IL(4)=16
        ENDIF
      ENDIF
C----
C  STORE IN TEMPORARY MATRIX OLD PSS MATRIX
C----
      DO 310 ISFK=1,NSFK
        PTSS(ISFK)=PSS(ISFK)
 310  CONTINUE
      DO 100 IB1=NBLK,1,-1
        DO 110 IB2=NBLK,1,-1
          IBT=MOD(IB1+IB2,2)
C----
C  TYPE OF BLOCKS
C----
          IF(IBT.EQ.0) THEN
            LEVEN=.TRUE.
          ELSE
            LEVEN=.FALSE.
          ENDIF
          NNZ=NSFZ
          DO 120 IDB1=2,1,-1
            NZ=NNZ
            DO 130 IDB2=2,1,-1
              IDBT=MOD(IDB1+IDB2,2)
C----
C  TYPE OF SUB BLOCKS
C----
              IF(IDBT.EQ.0) THEN
                IF(LEVEN) THEN
                  LZERO=.TRUE.
                ELSE
                  LZERO=.FALSE.
                  NK=IL(IBLK(IDBT,IB2,IB1))
                ENDIF
              ELSE
                IF(LEVEN) THEN
                  LZERO=.FALSE.
                  NK=IL(IBLK(IDBT,IB2,IB1))
                ELSE
                  LZERO=.TRUE.
                ENDIF
              ENDIF
              IF(LZERO) THEN
                DO 140 IA1=1,IAPPR
                  DO 150 IA2=1,IAPPR
                    IZ=NZ+IPOSF(IA2,IA1,NBLK)
                    PSS(IZ)=0.0
 150              CONTINUE
 140            CONTINUE
              ELSE
                DO 160 IA1=1,IAPPR
                  DO 170 IA2=1,IAPPR
                    IZ=NZ+IPOSF(IA2,IA1,NBLK)
                    IK=NK+IPOSI(IA2,IA1,NBLK)
                    PSS(IZ)=PTSS(IK)
 170              CONTINUE
 160            CONTINUE
              ENDIF
              NZ=NZ-IAPPR
 130        CONTINUE
            NNZ=NNZ-NNZD
 120      CONTINUE
          NSFZ=NSFZ-IAPPR2
 110    CONTINUE
        NSFZ=NS2/2
 100  CONTINUE
C----
C  LEAKAGE PROBABILITIES
C----
      NSFZ=NSURF
      NSFK=NSFZ/2
      NSFT=NSFK/IAPPR
      DO 200 IT=1,NSFT
        ISFZ=NSFZ
        DO 210 IK=1,IAPPR
          DO 220 IA=1,2
            PIS(ISFZ)=0.5*PIS(NSFK)
            ISFZ=ISFZ-IAPPR
 220      CONTINUE
          ISFZ=NSFZ-1
          NSFK=NSFK-1
 210    CONTINUE
        NSFZ=NSURF-2*IAPPR
 200  CONTINUE
      RETURN
      END
