*DECK XDRKIN
      SUBROUTINE XDRKIN( DX, NBX, MLOG, C0, C1, C2)
C
C
C**** ROUTINE    XDRKIN: R.ROY (07/1994), VERSION 0.1
C                        CALLED BY DRAGON.
C
C                        ROUTINE WILL CONSTRUCT BICKLEY TABLES FOR
C                        KI1(X), KI2(X), KI3(X), KI4(X), KI5(X).
C                        LOGARITHMIC SINGULARITIES FOR KI1(X) AND KI2(X)
C                        ARE EXTRACTED.
C
C****  INPUT:   *DX    * STEP FOR TABLES        (HERE: DX=0.02D0)
C               *NBX   * ORDER OF TABLES        (HERE: NBX=600)
C               *MLOG  * INTERVALS FOR LOG-SING (SUGGESTED: MLOG(1)=30,
C                                                           MLOG(2)=15,
C                                                           MLOG(3)= 0,
C                                                           MLOG(4)= 0,
C                                                           MLOG(5)= 0)
C
C WORK SPACE:   *C0    * DOUBLE PRECISION ARRAY (OF ORDER 601*5 AT LEAST
C               *C1    * DOUBLE PRECISION ARRAY (OF ORDER 601*5 AT LEAST
C               *C2    * DOUBLE PRECISION ARRAY (OF ORDER 601*5 AT LEAST
C
C**** OUTPUT:   THE FIVE COMMONS OF BICKLEY QUADRATIC TABLES ARE FILLED
C               AND SAVED WITH RESPECTIVE NAMES:
C                  /BICKL1/,/BICKL2/,/BICKL3/,/BICKL4/,/BICKL5/
C
C**** CALLS:    *AKIN10* ROUTINE FOR ACCURATE KIN(X) BICKLEY VALUES
C               *AK0BES* ROUTINE FOR ACCURATE K0(X)   BESSEL VALUES
C               *AK1BES* ROUTINE FOR ACCURATE K1(X)   BESSEL VALUES
C
C**** NOTE:     YOU MAY UNCOMMENT THE FOLLOWING FORTRAN INSTRUCTION:
C     IMPLICIT NONE
C
      INTEGER  I, KI, NBX, MLOG(5)
      DOUBLE PRECISION DEPS, DX, X, AKIN(-1:10), AK0BES, AK1BES
      DOUBLE PRECISION C0(5,0:NBX), C1(5,0:NBX), C2(5,0:NBX)
      DOUBLE PRECISION GAMMA, PIO2, PAS, XLIM
      PARAMETER ( GAMMA=0.57721566490153D0, DEPS= 1.D-10,
     >            PIO2= 1.57079632679490D0 )
C
C     SPACE AND VALUES FOR BICKLEY KI1 TO KI5 TABLES
      INTEGER MKI1, MKI2, MKI3, MKI4, MKI5,
     >        L1,   L2,   L3,   L4,   L5
      REAL    BI1,  BI2,  BI3,  BI4,  BI5,
     >        XLIM1,XLIM2,XLIM3,XLIM4,XLIM5,
     >        PAS1, PAS2, PAS3, PAS4, PAS5
      PARAMETER (MKI1=600,MKI2=600,MKI3=600,MKI4=600,MKI5=600)
      COMMON /BICKL1/BI1(0:MKI1,3),PAS1,XLIM1,L1
      COMMON /BICKL2/BI2(0:MKI2,3),PAS2,XLIM2,L2
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
      SAVE   /BICKL1/,/BICKL2/,/BICKL3/,/BICKL4/,/BICKL5/
C
      IF( MLOG(3).GT.0.OR.MLOG(4).GT.0.OR.MLOG(5).GT.0 )GOTO 99
C
      IF( NBX.NE.   600 )GOTO 97
      IF( DX .GT. 0.02D0+DEPS .OR. DX .LT. 0.02D0-DEPS  )GOTO 98
      PAS=   1.0D0/DX
      XLIM=  DBLE(NBX)*DX
      L1= MLOG(1)
      L2= MLOG(2)
      L3= MLOG(3)
      L4= MLOG(4)
      L5= MLOG(5)
      XLIM1= XLIM
      XLIM2= XLIM
      XLIM3= XLIM
      XLIM4= XLIM
      XLIM5= XLIM
      PAS1= PAS
      PAS2= PAS
      PAS3= PAS
      PAS4= PAS
      PAS5= PAS
C
C     FIRST, WE CONSTRUCT THE TABLES USING ACCURATE *AKIN10* VALUES
C
      X= 0.D0
      CALL AKIN10(X,AKIN(1))
      AKIN( 0)= 0.D0
      AKIN(-1)= 0.D0
      DO 20 I= 0, NBX-1
         DO 10 KI= 1, 5
            C2(KI,I)=   0.5D0 * AKIN(KI-2)
            C1(KI,I)= -(AKIN(KI-1)+X*AKIN(KI-2))
            C0(KI,I)=   AKIN(KI)+X*(AKIN(KI-1)+X*C2(KI,I))
   10    CONTINUE
         X= X + DX
         CALL AKIN10(X,AKIN(1))
         AKIN( 0)= AK0BES(X)
         AKIN(-1)= AK1BES(X)
   20 CONTINUE
      DO 30 KI= 1, 5
         C0(KI,NBX)= 0.D0
         C1(KI,NBX)= 0.D0
         C2(KI,NBX)= 0.D0
   30 CONTINUE
C
C     KI1(X) ADJUSTMENTS
      X= 0.D0
      DO 11 I= 1, MLOG(1)-1
         X= X + DX
         C0(1,I)= C0(1,I) + 0.5D0*X
         C1(1,I)= C1(1,I) - LOG(X)
         C2(1,I)= C2(1,I) - 0.5D0/X
   11 CONTINUE
C
      C1(1,0)= GAMMA-(LOG(2.D0)+1.D0)
C
C     KI2(X) ADJUSTMENTS
      X= 0.D0
      DO 12 I= 1, MLOG(2)-1
         X= X + DX
         C0(2,I)= C0(2,I) + 0.25D0*X*X
         C1(2,I)= C1(2,I) - X
         C2(2,I)= C2(2,I) + 0.5D0*LOG(X) +0.75D0
   12 CONTINUE
C
      C1(2,0)=  -PIO2
      C2(2,0)=   0.5D0*(LOG(2.D0)+1.5D0-GAMMA)
C
C     CHARGE COMMONS
      DO 13 I= 0, NBX
         BI1(I,1)=  C0(1,I)
         BI1(I,2)= C1(1,I)
         BI1(I,3)= C2(1,I)
         BI2(I,1)=  C0(2,I)
         BI2(I,2)= C1(2,I)
         BI2(I,3)= C2(2,I)
         BI3(I,1)=  C0(3,I)
         BI3(I,2)= C1(3,I)
         BI3(I,3)= C2(3,I)
         BI4(I,1)=  C0(4,I)
         BI4(I,2)= C1(4,I)
         BI4(I,3)= C2(4,I)
         BI5(I,1)=  C0(5,I)
         BI5(I,2)= C1(5,I)
         BI5(I,3)= C2(5,I)
   13 CONTINUE
C
      RETURN
C
C     ERROR SECTION
   97 CALL XABORT('XDRKIN: KIN TABLES HAVE 601 ELEMENTS')
   98 CALL XABORT('XDRKIN: KIN TABLES HAVE A STEP OF 0.02')
   99 CALL XABORT('XDRKIN: NO LOG SINGULARITY TAKEN FOR KI345')
      END
