*DECK XDRLGX
      SUBROUTINE XDRLGX(IPLIB ,IGS   ,IPRINT,IORD  ,NGROUP,INDPRO,
     >                  XSREC ,ITYPRO)
*
*-----------------------------------------------------------------------
*
*Purpose:
* get/save standard vector cross section from/on IPLIB
*
*Copyright:
* Copyright (C) 2007 Ecole Polytechnique de Montreal
* This library is free software; you can redistribute it and/or
* modify it under the terms of the GNU Lesser General Public
* License as published by the Free Software Foundation; either
* version 2.1 of the License, or (at your option) any later version.
*
*Author(s): G. Marleau
*
*Parameters: input
* IPLIB   pointer to the internal library
* IGS     get or save flag:
*         >0 save;
*         <0 get
* IPRINT  Print level (cross sections printed if IPRINT>9)
* IORD    cross section order:
*         =1 constant;
*         =2 linear;
*         =3 quadratic
* NGROUP  number of energy groups
* INDPRO  index for cross section to process:
*         =0 -> not process;
*         =0 do not process;
*         >0 -> process.
*
*Parameters: input/output
* XSREC   cross section table.
*
*Parameters: output
* ITYPRO  vector for cross section processed indices:
*         =0 -> absent  (not processed);
*         >0 -> present (processed)
*
*-----------------------------------------------------------------------
*
      IMPLICIT NONE
*----
*  Local parameters
*  NDPROC = number of standard vector cross sections
*----
      INTEGER          IOUT,NDPROC
      CHARACTER        NAMSBR*6
      PARAMETER       (IOUT=6,NDPROC=20,NAMSBR='XDRLGX')      
*----
*  Subroutine arguments
*----
      INTEGER          IPLIB,IGS,IPRINT,IORD,NGROUP,
     >                 INDPRO(NDPROC)
      REAL             XSREC(NGROUP,NDPROC)
      INTEGER          ITYPRO(NDPROC)
*----
*  Local variables and data
*----
      INTEGER          IODIV,LONG,ITYP,IXSR,IXSTN,IG,JG
      CHARACTER*12     NAMXS
      CHARACTER*6      NAMDXS(NDPROC),NORD(3)
      SAVE             NAMDXS
      DATA    NAMDXS  /'TOTAL ','TRANC ','NUSIGF','NFTOT ','CHI   ',
     >                 'NU    ','NG    ','NHEAT ','N2N   ','N3N   ',
     >                 'N4N   ','NP    ','NA    ','GOLD  ','ABS   ',
     >                 'NWT0  ','STRD  ','STRD X','STRD Y','STRD Z'/
      DATA    NORD    /'      ','   LIN','   QUA'/
*----
*  Test input options
*----
      IF(IORD .LT. 1 .OR. IORD .GT. 3) CALL XABORT(NAMSBR//
     >  ': Invalid XS order -> only 1, 2 or 3 permitted.')
      IODIV=2**(IORD-1)
*----
*  Read/initialize state vector
*----
      CALL LCMLEN(IPLIB,'XS-SAVED',LONG,ITYP)
      IF(LONG.EQ.NDPROC) THEN
        CALL LCMGET(IPLIB,'XS-SAVED',ITYPRO)
      ELSE IF(LONG.EQ.0) THEN
        CALL XDISET(ITYPRO,NDPROC,0)
      ELSE
        WRITE(IOUT,9000) NDPROC,LONG
        CALL XABORT(NAMSBR//': Length of record XS-SAVED is invalid')
      ENDIF
      IF(IGS.GT.0) THEN
*----
*  Saving standard vector XS
*----
        DO IXSR=1,NDPROC
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          IF(INDPRO(IXSR).EQ.1) THEN
            IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
*----
*  Find if all XS not 0.0
*----
            DO IG=1,NGROUP
              IF(XSREC(IG,IXSR).NE.0.0) THEN
                IF(IXSTN.EQ.0) THEN
                  ITYPRO(IXSR)=ITYPRO(IXSR)+IODIV
                  IXSTN=1
                ENDIF
                GO TO 100
              ENDIF
            ENDDO
 100        CONTINUE
*----
*  Save only if all XS not 0.0
*----
            IF(IXSTN.NE.0) THEN
              CALL LCMPUT(IPLIB,NAMXS,NGROUP,2,XSREC(1,IXSR))
            ENDIF
          ENDIF
        ENDDO
        CALL LCMPUT(IPLIB,'XS-SAVED',NDPROC,1,ITYPRO)
      ELSE IF(IGS .LT. 0) THEN
*----
*  Get vector XS
*----
        DO IXSR=1,NDPROC
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          IF(INDPRO(IXSR).EQ.1) THEN
            IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
*----
*  Read if IXSTN = 1 (cross sections exists)
*  Initialize to 0.0 if IXSTN = 0 (cross sections absent)
*----
            IF(IXSTN.EQ.1) THEN
              CALL LCMLEN(IPLIB,NAMXS,LONG,ITYP)
              IF(LONG .EQ. 0) THEN 
                CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
              ELSE
                CALL LCMGET(IPLIB,NAMXS,XSREC(1,IXSR))
              ENDIF
            ELSE
              CALL XDRSET(XSREC(1,IXSR),NGROUP,0.0)
            ENDIF
          ENDIF
        ENDDO
      ENDIF
      IF(IPRINT .GE. 10) THEN
*----
*  Print XS
*----
        DO IXSR=1,NDPROC
          NAMXS=NAMDXS(IXSR)//NORD(IORD)
          IF(INDPRO(IXSR).EQ.1) THEN
            IXSTN=MOD(ITYPRO(IXSR)/IODIV,2)
            IF(IXSTN.NE.0) THEN
              DO IG=1,NGROUP
                IF(XSREC(IG,IXSR).NE.0.0) THEN
                  WRITE(IOUT,6000) NAMXS
                  WRITE(IOUT,6010) (XSREC(JG,IXSR),JG=1,NGROUP)
                  GO TO 110
                ENDIF
              ENDDO
            ENDIF
 110        CONTINUE
          ENDIF
        ENDDO
      ENDIF
      RETURN
*----
*  Formats
*----
 9000 FORMAT(' XDRLGX: ****** ABORT ******'/
     >       ' INVALID LENGTH OF RECORD XS-SAVED '/
     >       ' STORAGE SPACE NDPROC   = ',I10/
     >       ' LENGTH OF RECORD LONG  = ',I10/
     >       ' ***************************')
 6000 FORMAT(/' CROSS SECTION TYPE    = ',A12)
 6010 FORMAT(1P,5E16.7)
      END
