*DECK XELPA0
      SUBROUTINE XELPA0( NGAUSS, XGAUSS, WGAUSS,
     >                        A,      B,      C,  SURFC,
     >                      PCC,    PCB,    PCA,    PVC,   PVVC )
************************************************************************
*                                                                      *
*           NAME: XELPA0                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: (CALLED BY 'XELPA3')                                 *
*        VERSION: 0.8                                                  *
*       CREATION: 87/01                                                *
*       MODIFIED: 00/03 (R.R.) BRING GAUSS PTS/WGTS AS 1ST ARGUMENTS   *
*                              DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE WILL COMPUTE SURFACE-TO-SURFACE "CP"    *
*                 FOR A RECTANGULAR SINGLE **VOID** BLOCK, USING GAUSS *
*                 INTEGRATION POINTS.                                  *
*                                                                      *
*      REFERENCE: 'ANALYTIC REDUCTIONS FOR TRANSMISSION AND LEAKAGE    *
*                  PROBABILITIES IN FINITE TUBES AND HEXAHEDRA',       *
*                  G.MARLEAU, R.ROY AND A.HEBERT,                      *
*                  NUCLEAR SCIENCE & ENGINEERING, V 104, P 204 (1990). *
*                                                                      *
*--------+------------- P A R A M E T E R S --------------+--+---------*
*  NAME  /                  DESCRIPTION                   /IO/  MODE   *
*--------+------------------------------------------------+--+---------*
* NGAUSS / # OF GAUSS INTEGRATION POINTS & WEIGHTS.       /I./INT      *
* XGAUSS / "ARRAY" WITH GAUSS POINTS.                     /I./REL      *
*        /                                                /  / (NGAUSS)*
* WGAUSS / "ARRAY" WITH GAUSS WEIGHTS.                    /I./REL      *
*        /                                                /  / (NGAUSS)*
*      A / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*      B / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*      C / SIDE OF A RECT. HOMOGENEOUS BLOCK ( A . B . C )/I./REL      *
*  SURFC / SURFACE SURF. C ( SURFC = PI * A * B )         /I./REL      *
*    PCC / "CP" FOR 2 RECT PARALLEL SURF. (S.C TO S.C )   /.O/REL      *
*    PCB / "CP" FOR 2 RECT PERPEND. SURF. (S.C TO S.B )   /.O/REL      *
*    PCA / "CP" FOR 2 RECT PERPEND. SURF. (S.C TO S.A )   /.O/REL      *
*    PVC / "CP" FROM A VOLUME TO SURF. C.                 /.O/REL      *
*   PVVC / PART OF A "CP" FROM A VOLUME TO ITSELF.        /.O/REL      *
************************************************************************
C
      IMPLICIT NONE
C
C     DECLARE  DUMMY ARGUMENTS
      INTEGER  NGAUSS
      REAL     XGAUSS(NGAUSS), WGAUSS(NGAUSS)
      REAL     A, B, C, SURFC, PCC, PCB, PCA, PVC, PVVC
C
C     DECLARE  LOCAL VARIABLES
      REAL     AA, BB, CC, AALNAA, BBLNBB, CCLNCC, SQRTAB, SQRTAC,
     >         SQRTBC, SQABC, PVCA, PVCB, PVCC, PVVCA, PVVCB, PVVCC,
     >         XAB, X, W
      INTEGER  IGAU
C
      AA    =  A * A
      BB    =  B * B
      CC    =  C * C
      AALNAA= AA * LOG( AA )
      BBLNBB= BB * LOG( BB )
      CCLNCC= CC * LOG( CC )
      SQRTAB= SQRT( AA + BB )
      SQRTAC= SQRT( AA + CC )
      SQRTBC= SQRT( BB + CC )
C
C     ANALYTICAL REDUCTION FOR "SURFACE PROBABILITIES" FROM SURF. C
C
C       > TO SURF. A               ( ORDER "0" )
      PCA   = 0.25 * (  (BB-AA)*LOG(AA+BB) + (BB-CC)*LOG(BB+CC)
     >                - (BB-AA-CC)*LOG(AA+BB+CC) - (AA+CC)*LOG(AA+CC)
     >                +  AALNAA + CCLNCC - BBLNBB )
     >        + A*B*ATAN(B/A) + B*C*ATAN(B/C) - B*SQRTAC*ATAN(B/SQRTAC)
C
C       > TO SURF. B               ( ORDER "0" )
      PCB   = 0.25 * (  (AA-BB)*LOG(AA+BB) + (AA-CC)*LOG(AA+CC)
     >                - (AA-BB-CC)*LOG(AA+BB+CC) - (BB+CC)*LOG(BB+CC)
     >                + BBLNBB + CCLNCC - AALNAA )
     >        + A*B*ATAN(A/B) + A*C*ATAN(A/C) - A*SQRTBC*ATAN(A/SQRTBC)
C
C       > TO SURF. C               ( ORDER "0" )
      PCC   = SURFC - 2. * (PCA + PCB)
      SQABC = SQRT( AA + BB + CC )
C
C       > TO SURF. A               ( ORDER "1" )
      PVCA  = ( (AA-2.*BB)*SQRTAB + (AA-2.*CC)*SQRTAC - 2.*SQRTBC**3
     >         + 2.*(BB*(B+SQABC) +CC*(C+SQABC)) - AA*(A+SQABC) )/3.
     >      + A * (  BB*(LOG(A+SQRTAB)-LOG(B))
     >             + CC*(LOG(A+SQRTAC)-LOG(C))
     >             + (BB+CC)*(LOG(SQRTBC)-LOG(A+SQABC)))
C
C       > TO SURF. B               ( ORDER "1" )
      PVCB  = ( (BB-2.*AA)*SQRTAB + (BB-2.*CC)*SQRTBC - 2.*SQRTAC**3
     >         + 2.*(AA*(A+SQABC) +CC*(C+SQABC)) - BB*(B+SQABC) )/3.
     >      + B * (  AA*(LOG(B+SQRTAB)-LOG(A))
     >             + CC*(LOG(B+SQRTBC)-LOG(C))
     >             + (AA+CC)*(LOG(SQRTAC)-LOG(B+SQABC)))
C
C       > TO SURF. C               ( ORDER "1" )
      PVCC  = 4.*CC*( A * (  LOG(A+SQABC)-LOG(SQRTBC)
     >                     + B/C*ATAN(A*B/C/SQABC) + LOG(C)
     >                     - LOG(A+SQRTAC)   )
     >                - SQABC + SQRTBC + SQRTAC - C
     >                + 0.5 * B * (  LOG(SQABC+B)-LOG(SQABC-B)
     >                             + LOG(SQRTBC-B)-LOG(SQRTBC+B) ) )
      PVC   =  PVCC + 2. * ( PVCA + PVCB )
C
C       > TO SURF. A               ( ORDER "2" )
      PVVCA = 0.5 * ( ( (BB*BB/12. + BB*(AA+CC)/2. -(AA+CC)**2/4.)
     >                 * LOG(BB + AA+CC ) + (AA+CC)**2/4.* LOG(AA+CC)
     >                 + 4./3.*B*SQRTAC**3 * ATAN(B/SQRTAC) )
     >               -( (BB*BB/12. + BB* AA    /2. - AA    **2/4.)
     >                 * LOG(BB + AA    ) +  AA    **2/4.* LOG(AA   )
     >                 + 4./3.*B*    A **3 * ATAN(B/    A ) )
     >               -( (BB*BB/12. + BB*    CC /2. -    CC **2/4.)
     >                 * LOG(BB +    CC ) +     CC **2/4.* LOG(   CC)
     >                 + 4./3.*B*     C**3 * ATAN(B/     C) )
     >               + BB*BB/12. * LOG(BB) )
C
C       > TO SURF. B               ( ORDER "2" )
      PVVCB = 0.5 * ( ( (AA*AA/12. + AA*(BB+CC)/2. -(BB+CC)**2/4.)
     >                 * LOG(AA + BB+CC ) + (BB+CC)**2/4.* LOG(BB+CC)
     >                 + 4./3.*A*SQRTBC**3 * ATAN(A/SQRTBC) )
     >               -( (AA*AA/12. + AA* BB    /2. - BB    **2/4.)
     >                 * LOG(AA + BB    ) +  BB    **2/4.* LOG(BB   )
     >                 + 4./3.*A*    B **3 * ATAN(A/    B ) )
     >               -( (AA*AA/12. + AA*    CC /2. -    CC **2/4.)
     >                 * LOG(AA +    CC ) +     CC **2/4.* LOG(   CC)
     >                 + 4./3.*A*     C**3 * ATAN(A/     C) )
     >               + AA*AA/12. * LOG(AA) )
C
C       > TO SURF. C               ( ORDER "2" )
      XAB   = 0.0
      DO 10 IGAU= 1, NGAUSS
          X     = XGAUSS(IGAU)
          W     = WGAUSS(IGAU)
          XAB  = XAB + W*ATAN(B/SQRT(AA*X*X+CC))/SQRT(AA*X*X+CC)
   10 CONTINUE
      XAB   = XAB * A
      PVVCC = CC*(  4. * A*B * XAB
     >            +(       -AA -BB + CC) * LOG(AA+BB+CC)
     >            +(               + CC) * LOG(      CC)
     >            +(        AA     - CC) * LOG(AA+   CC)
     >            +(            BB - CC) * LOG(   BB+CC)
     >            +4.*( C * (A*ATAN(A/C) + B*ATAN(B/C) )
     >                 - A*SQRTBC*ATAN(A/SQRTBC)
     >                 - B*SQRTAC*ATAN(B/SQRTAC) ) )
      PVVC  = PVVCC + 2. * ( PVVCA + PVVCB )
C
      RETURN
      END
