*DECK XHX2D0
      SUBROUTINE XHX2D0 (NGPT,ZGAUS,WGAUS,COTE,SIGT,TRONC,PII,PIS,PSS)
C
C---------------------------  XHX2D0  ---------------------------------
C
C   1-  PROGRAMME STATISTICS:
C
C          NAME       :  XHX2D0
C          FUNCTION   :  COMPUTE DP0 COLLISION, LEAKAGE AND TRANSMISSION
C                        PROBABILITIES FOR HEXAGONAL 2D GEOMETRIES
C          REVISED    :  14-03-1990
C          AUTHOR     :  M.OUISLOUMEN
C
C   2-  PROGRAMME PARAMETERS:
C
C INPUT PARAMETERS.
C   NGPT    : NUMBER OF GAUSS INTEGRATION POINTS.        I
C   COTE    : LENGTH OF ONE OF SIDES OF THE HEXAGONE     R
C   SIGT    : TOTAL CROSS SECTION.                       R
C   TRONC   : VOIDED BLOCK CUTOFF CRITERION.             R
C   ZGAUS   : GAUSS-LEGENDRE INTEGRATION POINTS
C   WGAUS   : GAUSS-LEGENDRE INTEGRATION WEIGHTS
C
C OUTPUT PARAMETERS.
C   PII     : VOLUME TO VOLUME REDUCED PROBABILITY.      R
C   PIS     : LEAKAGE PROBABILITY                        R(6)
C             PIS(I)  -> VOLUME TO SIDE I (I=1,6)
C   PSS     : TRANSMISSION PROBABILITY                   R(6,6)
C             PSS(I,J) -> SIDE I TO SIDE J (I=1,6;J=1,6)
C
C                      SIDE 4
C                     XXXXXXXX
C                    X        X
C          SIDE 5   X          X SIDE 3
C                  X            X
C                 X              X
C                  X            X
C          SIDE 6   X          X SIDE 2
C                    X        X
C                     XXXXXXXX
C                      SIDE 1
C
C
C---------------------------   XHX2D0  --------------------------------
C
      PARAMETER (MKI3=600,MKI4=600,MKI5=600)
      PARAMETER (PI=3.141592653589793,SQRT3=1.732050807568877,
     +           SQRT2=1.414213562373095,ALOG2=.693147180559945,
     +           ALOG3=1.0986122886681097,ALOGX=.7676517525907618)
C
      DIMENSION  PIS(6),PSS(6,*),ZGAUS(NGPT),WGAUS(NGPT),P(3)
      DIMENSION  TAU(3),FKI3(3),FKI4(3),FKI5(3),FKI6(3),IROT(6,6)
C
C ASSUME THAT BICKLEY KI TABLES HAVE THE SAME TABULATION POINTS AND
C THE SAME TRUNCATION LIMIT.
C
      COMMON /BICKL3/BI3(0:MKI3,3),PAS3,XLIM3,L3
      COMMON /BICKL4/BI4(0:MKI4,3),PAS4,XLIM4,L4
      COMMON /BICKL5/BI5(0:MKI5,3),PAS5,XLIM5,L5
C
      SAVE IROT
      DATA IROT /
     +          0, 1, 2, 3, 2, 1,
     +          1, 0, 1, 2, 3, 2,
     +          2, 1, 0, 1, 2, 3,
     +          3, 2, 1, 0, 1, 2,
     +          2, 3, 2, 1, 0, 1,
     +          1, 2, 3, 2, 1, 0/
C
      FUNC3(X,K)=BI3(K,1)+X*(BI3(K,2)+X*BI3(K,3))
      FUNC4(X,K)=BI4(K,1)+X*(BI4(K,2)+X*BI4(K,3))
      FUNC5(X,K)=BI5(K,1)+X*(BI5(K,2)+X*BI5(K,3))
C
C    INITIALISATION DES PROBABILITES DE COLLISION
C
      P(3)=0.
      P(2)=0.
      P(1)=0.
C----
C  COMPUTE CORDE =4*V*SIGMA/S=SQRT(3)*COTE*SIGMA  (CORDE MOYENNE)
C----
      CORDE=SQRT3*COTE*SIGT
      IF (CORDE.LE.TRONC) GO TO 300
C----
C  CONSIDER EXPLICIT INTEGRATION OF F PSS
C----
      PI12=PI/12.
C
C  ON CONSIDERE 2 CAS  1) SI ZGAUS(I)<0 => 1/COSFI>1/SINA>1/COSA
C                      2) SI ZGAUS(I)>0 => 1/COSFI>1/COSA>1/SINA
C
      NGPT2=IFIX(FLOAT(NGPT)/2.)
      DO 50 I=1,NGPT
      FI=PI12*(1.+ZGAUS(I))
      COSFI=COS(FI)
      SINFI=SIN(FI)
      COSA=SQRT3*COSFI-SINFI
      AUX=SQRT3*SINFI
      SINA=AUX+COSFI
      SINB=COSFI-AUX
C
C   FONCTIONS DE BICKLEY NAYLOR
C
      TAU(1)=CORDE/COSFI
      IAUX1=2
      IAUX2=3
      IF(I.GT.NGPT2) THEN
      IAUX1=3
      IAUX2=2
      ENDIF
      TAU(IAUX1)=CORDE/SINA
      TAU(IAUX2)=CORDE/COSA
C
      LB=4
      IF(TAU(1).LT.XLIM3) THEN
      LB=1
      ELSEIF(TAU(2).LT.XLIM3) THEN
      LB=2
      ELSEIF(TAU(3).LT.XLIM3) THEN
      LB=3
      ENDIF
C
      DO 10 J=LB,3
      K1=NINT(PAS3*TAU(J))
      FKI3(J)=FUNC3(TAU(J),K1)
      FKI4(J)=FUNC4(TAU(J),K1)
      FKI5(J)=FUNC5(TAU(J),K1)
   10 FKI6(J)=.8*FKI4(J)+.2*TAU(J)*(FKI3(J)-FKI5(J))
C
      GO TO (20,25,30,50),LB
C
C   CALCUL DES PSS
C
   20 WEIGHT=WGAUS(I)
      P(3)=P(3)+WEIGHT*SINB*FKI3(1)
      P(2)=P(2)+WEIGHT*COSFI*SINA*(FKI4(IAUX1)-FKI4(1))
      GO TO 30
C
   25 WEIGHT=WGAUS(I)
      P(2)=P(2)+WEIGHT*COSFI*SINA*FKI4(IAUX1)
   30 P(1)=P(1)+WEIGHT*SINFI*COSA*(BI4(0,1)-FKI4(IAUX2))
   50 CONTINUE
C
C    NORMALISATION
C
      X1=1./(3.*CORDE)
      P(1)=X1*P(1)
      P(2)=X1*P(2)
      P(3)=P(3)/3.
      PIS10=(1.-2.*(P(1)+P(2))-P(3))/(6.*CORDE)
      PII=(1.-6.*PIS10)/SIGT
C
      GO TO 350
C----
C  USE SERIES EXPANSION FOR CORDE->0: DEVELOPPEMENT EN SERIE DE MCLAURIN
C  DES FONCTIONS KI
C----
  300 TAU0=CORDE*.5
      TAU02=TAU0*TAU0
      AUX=SQRT3/PI
      AUX0=1.-.5*SQRT3
      AUX1=AUX*ALOG3-.33333333333333333
      AUX2=2./SQRT3-(2.+ALOGX)/3.
      P(1)=AUX0-.5*(TAU0*AUX1-TAU02*AUX2)
      AUX1=AUX*(2.5*ALOG3-4.*ALOG2)-.5
      AUX2=5.*SQRT3-9.-2.*ALOG3+.5*ALOGX
      P(2)=SQRT3-1.5+TAU0*AUX1-TAU02*AUX2/3.
      AUX0=2.*AUX0
      AUX1=AUX*(.5*ALOG3-ALOG2)
      P(3)=AUX0-8.*(1./6.+AUX1)*TAU0-4.*TAU02*(AUX0-.5*ALOG3)
      PII=COTE*SQRT3*(4.*SQRT3-8.-2.*ALOGX+10.*ALOG3)/12.
      PIS10=(1.-SIGT*PII)/6.
C
  350 CONTINUE
C
C MATRICE DE TRANSMISSION
C
      DO  58 I=1,6
      DO  58 J=1,6
      IB=IROT(J,I)
      IF(IB.GT.0) THEN
         PSS(I,J)=P(IB)
      ELSE
         PSS(I,J)=0.
      ENDIF
   58 CONTINUE
C
C PROBABILITE DE FUITE
C
      DO 56 I=1,6
   56 PIS(I)=PIS10
C
      RETURN
      END
