*DECK XL3SIG
      SUBROUTINE XL3SIG(  NGRT, MAXMIX, XSSIGT, XSSIGW, ALBEDO,
     >                    LGRP,   NGRP,     NS,     NR, MATALB,
     >                  SIGTAL, SIGINV,  SIGSS, SWVOID, SWNZBC)
************************************************************************
*                                                                      *
*           NAME: XL3SIG                                               *
*      COMPONENT: EXCELL                                               *
*          LEVEL: 4 (CALLED BY 'XL3TRK')                               *
*        VERSION: 1.0                                                  *
*       CREATION: 96/09 (R.R.)                                         *
*       MODIFIED: 00/03 (R.R.) DECLARE ALL VARIABLE TYPES              *
*         AUTHOR: ROBERT ROY                                           *
*                                                                      *
*     SUBROUTINE: THIS ROUTINE IS USED TO UNFOLD CROSS-SECTION DATA    *
*                 WHICH BECOMES AVAILABLE BY SUBSET OF GROUPS          *
*                                                                      *
*--------+-------------- V A R I A B L E S -------------+--+-----------*
*  NAME  /                  DESCRIPTION                 /IO/MOD(DIMENS)*
*--------+----------------------------------------------+--+-----------*
* NGRT   / TOTAL NUMBER OF GROUPS                       /I./INT        *
* MAXMIX / # OF MIXTURES IN THE MACROLIB                /I./INT        *
* XSSIGT / TOTAL XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* XSSIGW / SCATT XS FOR MIXTURES IN THE MACROLIB        /I./REL(MAXMIX *
*        /                                              /I./  +1,NGRT) *
* ALBEDO / GEOMETRIC ALBEDOS.                           /I./REL(6)     *
* LGRP   / GROUP NUMBERS IN THE SUBSET                  /I./INT(NGRP)  *
* NGRP   / NUMBER OF GROUPS IN THE SUBSET               /I./INT        *
* NS     / # OF SURFACES IN THE ASSEMBLY.               /I./INT        *
* NR     / # OF ZONES IN THE ASSEMBLY.                  /I./INT        *
* MATALB / MATERIAL #S FOR ZONES IN THE SUPERCELL       /I./INT(NS:NR) *
* SIGTAL / SUBSET TOTAL XS & ALBEDOS BY REGION & SURFACE/.O/REL(NGRP,  *
*        /                                              /  /    NS:NR) *
* SIGINV / SUBSET INVERSE TOTAL XS BY REGION            /.O/REL(NGRP,  *
*        /                                              /  /       NR) *
* SIGSS  / SUBSET SCAT XS BY REGION                     /.O/REL(NGRP,  *
*        /                                              /  /       NR) *
* SWVOID / LOGICAL SWITCH (.TRUE. IF VOID REGIONS)      /.O/LOG        *
* SWNZBC / LOGICAL SWITCH (.TRUE. IF NON-ZERO B.C.)     /.O/LOG        *
************************************************************************
      IMPLICIT   NONE
C
      INTEGER    NGRT,MAXMIX, NGRP,NS,NR
      REAL       XSSIGT(0:MAXMIX,NGRT),XSSIGW(0:MAXMIX,NGRT),
     >           SIGTAL(NGRP,NS:NR),SIGINV(NGRP,NR),SIGSS(NGRP,NR),
     >           ALBEDO(6)
      INTEGER    MATALB(NS:NR),LGRP(NGRP)
      INTEGER    IUN,JG,I
      LOGICAL    SWVOID,SWNZBC
      REAL       ZERO
      INTEGER    IOUT
      PARAMETER (ZERO=0.0,IOUT=6)
C
      SWVOID= .FALSE.
      SWNZBC= .FALSE.
C
      IF( NS.GT.0 ) CALL XABORT('XL3SIG: # OF SURFACES IS > 0')
      IF( NR.LT.0 ) CALL XABORT('XL3SIG: # OF REGIONS  IS < 0')
C
      DO 10 IUN= NS, NR
      DO 20 JG= 1, NGRP
         IF( IUN.LT.0 )THEN
            SIGTAL(JG,IUN)= ALBEDO(-MATALB(IUN))
            SWNZBC=SWNZBC.OR.(ALBEDO(-MATALB(IUN)).NE.ZERO)
         ELSEIF( IUN.EQ.0 )THEN
            SIGTAL(JG,IUN)= ZERO
         ELSE
            IF( MATALB(IUN).LT.0.OR.MATALB(IUN).GT.MAXMIX)THEN
               WRITE(IOUT,*) 'MAXMIX=',MAXMIX
               WRITE(IOUT,*) 'IG/NGRT=',LGRP(JG),NGRT
               WRITE(IOUT,*) 'XSSIGT=',(XSSIGT(I,LGRP(JG)),I=0,MAXMIX)
               WRITE(IOUT,*) 'MATALB<=0 =',(MATALB(I),I=NS,0)
               WRITE(IOUT,*) 'MATALB >0 =',(MATALB(I),I=1,NR)
               CALL XABORT('XL3SIG: INVALID NUMBER OF MIXTURES')
            ENDIF
            SIGTAL(JG,IUN)= XSSIGT(MATALB(IUN),LGRP(JG))
            SIGSS (JG,IUN)= XSSIGW(MATALB(IUN),LGRP(JG))
            IF( SIGTAL(JG,IUN).EQ.ZERO )THEN
               SWVOID=.TRUE.
               SIGINV(JG,IUN)= 1.
            ELSE
               SIGINV(JG,IUN)= 1./SIGTAL(JG,IUN)
            ENDIF
         ENDIF
   20 CONTINUE
   10 CONTINUE
C
      RETURN
      END
